@@ -1,7 +1,8 @@
 use crate::app::widget::first_window;
 use crate::enums::{Event, Key, Shortcut};
 use crate::prelude::*;
-use fltk_sys::fl;
+use crate::utils::FlString;
+use fltk_sys::fl::{self, Fl_open_display};
 use std::{
     cmp,
     ffi::{CStr, CString},
@@ -18,7 +19,7 @@ pub fn event() -> Event {
     unsafe { mem::transmute(fl::Fl_event()) }
 }
 
-/// Returns the presed key
+/// Returns the pressed key
 pub fn event_key() -> Key {
     unsafe { mem::transmute(fl::Fl_event_key()) }
 }
@@ -117,7 +118,7 @@ pub enum MouseWheel {
 }
 
 /// Returns the current horizontal mouse scrolling associated with the Mousewheel event.
-/// Returns `MouseWheel::None`, `Right` or `Left`
+/// Returns [`MouseWheel::None`](`crate::enums::MouseWheel::None`), `Right` or `Left`
 pub fn event_dx() -> MouseWheel {
     match 0.cmp(unsafe { &fl::Fl_event_dx() }) {
         cmp::Ordering::Greater => MouseWheel::Right,
@@ -127,7 +128,7 @@ pub fn event_dx() -> MouseWheel {
 }
 
 /// Returns the current horizontal mouse scrolling associated with the Mousewheel event.
-/// Returns `MouseWheel::None`, `Up` or `Down`.
+/// Returns [`MouseWheel::None`](`crate::enums::MouseWheel::None`), `Up` or `Down`.
 /// Doesn't indicate scrolling direction which depends on system preferences
 pub fn event_dy() -> MouseWheel {
     match 0.cmp(unsafe { &fl::Fl_event_dy() }) {
@@ -157,7 +158,7 @@ pub fn event_state() -> Shortcut {
     unsafe { mem::transmute(fl::Fl_event_state()) }
 }
 
-/// Returns whether an event occured within a widget
+/// Returns whether an event occurred within a widget
 pub fn event_inside_widget<Wid: WidgetExt>(wid: &Wid) -> bool {
     assert!(!wid.was_deleted());
     let x = wid.x();
@@ -167,7 +168,7 @@ pub fn event_inside_widget<Wid: WidgetExt>(wid: &Wid) -> bool {
     unsafe { fl::Fl_event_inside(x, y, w, h) != 0 }
 }
 
-/// Returns whether an event occured within a region
+/// Returns whether an event occurred within a region
 pub fn event_inside(x: i32, y: i32, w: i32, h: i32) -> bool {
     unsafe { fl::Fl_event_inside(x, y, w, h) != 0 }
 }
@@ -308,10 +309,10 @@ pub unsafe fn handle_raw(event: Event, w: WindowPtr) -> bool {
     The event dispatch function is called after native events are converted to
     FLTK events, but before they are handled by FLTK. If the dispatch function
     handler is set, it is up to the dispatch function to call
-    `app::handle2(Event, WindowPtr)` or to ignore the event.
+    [`app::handle2(Event, WindowPtr)`](crate::app::handle2) or to ignore the event.
 
     The dispatch function itself must return false if it ignored the event,
-    or true if it used the event. If you call `app::handle2()`, then
+    or true if it used the event. If you call [`app::handle2()`](crate::app::handle2), then
     this will return the correct value.
     # Safety
     The window pointer must not be invalidated
@@ -330,6 +331,50 @@ pub fn get_mouse() -> (i32, i32) {
     }
 }
 
+/// Text editing widget should call this for each FL_KEYBOARD event.
+pub fn compose() -> Option<i32> {
+    unsafe {
+        let mut del = 0;
+        if fl::Fl_compose(&mut del) != 0 {
+            Some(del)
+        } else {
+            None
+        }
+    }
+}
+
+/// Reset the length of bytes of app::compose()
+pub fn compose_reset() {
+    unsafe {
+        fl::Fl_compose_reset();
+    }
+}
+
+/// Return the length of bytes written in app::compose()
+pub fn compose_state() -> i32 {
+    unsafe { fl::Fl_compose_state() }
+}
+
+/// Copy text to the clipboard
+pub fn copy(stuff: &str) {
+    unsafe {
+        Fl_open_display();
+        let len = stuff.len();
+        let stuff = CString::safe_new(stuff);
+        fl::Fl_copy(stuff.as_ptr() as _, len as _, 1);
+    }
+}
+
+/// Copy text to the selection buffer
+pub fn copy2(stuff: &str) {
+    unsafe {
+        Fl_open_display();
+        let len = stuff.len();
+        let stuff = CString::safe_new(stuff);
+        fl::Fl_copy(stuff.as_ptr() as _, len as _, 0);
+    }
+}
+
 /// Types of Clipboard contents
 #[derive(Debug, Clone, Copy)]
 pub enum ClipboardContent {
@@ -376,6 +421,17 @@ where
     }
 }
 
+/// Pastes textual content from the selection buffer
+pub fn paste_text2<T>(widget: &T)
+where
+    T: WidgetExt,
+{
+    assert!(!widget.was_deleted());
+    unsafe {
+        fl::Fl_paste_text(widget.as_widget_ptr() as *mut fltk_sys::fl::Fl_Widget, 0);
+    }
+}
+
 /// Pastes image content from the clipboard
 pub fn paste_image<T>(widget: &T)
 where
@@ -387,6 +443,17 @@ where
     }
 }
 
+/// Pastes image content from the selection buffer
+pub fn paste_image2<T>(widget: &T)
+where
+    T: WidgetExt,
+{
+    assert!(!widget.was_deleted());
+    unsafe {
+        fl::Fl_paste_image(widget.as_widget_ptr() as *mut fltk_sys::fl::Fl_Widget, 0);
+    }
+}
+
 /// Adds a custom handler for unhandled events
 pub fn add_handler(cb: fn(Event) -> bool) {
     unsafe {
@@ -425,17 +492,13 @@ pub fn add_handler(cb: fn(Event) -> bool) {
     # Errors
     Returns Err on error or in use of one of the reserved values.
 */
-pub fn handle<I: Into<i32> + Copy + PartialEq + PartialOrd, W: WindowExt>(
+pub fn handle<I: Into<Event> + Copy + PartialEq + PartialOrd, W: WindowExt>(
     msg: I,
     w: &W,
 ) -> Result<bool, FltkError> {
     let val = msg.into();
-    if (0..=30).contains(&val) {
-        Err(FltkError::Internal(FltkErrorKind::FailedOperation))
-    } else {
-        let ret = unsafe { fl::Fl_handle(val, w.as_widget_ptr() as _) != 0 };
-        Ok(ret)
-    }
+    let ret = unsafe { fl::Fl_handle(val.bits(), w.as_widget_ptr() as _) != 0 };
+    Ok(ret)
 }
 
 /**
@@ -464,19 +527,15 @@ pub fn handle<I: Into<i32> + Copy + PartialEq + PartialOrd, W: WindowExt>(
     # Errors
     Returns Err on error or in use of one of the reserved values.
 */
-pub fn handle_main<I: Into<i32> + Copy + PartialEq + PartialOrd>(
+pub fn handle_main<I: Into<Event> + Copy + PartialEq + PartialOrd>(
     msg: I,
 ) -> Result<bool, FltkError> {
     let val = msg.into();
-    if (0..=30).contains(&val) {
-        Err(FltkError::Internal(FltkErrorKind::FailedOperation))
-    } else {
-        first_window().map_or(
-            Err(FltkError::Internal(FltkErrorKind::FailedOperation)),
-            |win| {
-                let ret = unsafe { fl::Fl_handle(val, win.as_widget_ptr() as _) != 0 };
-                Ok(ret)
-            },
-        )
-    }
+    first_window().map_or(
+        Err(FltkError::Internal(FltkErrorKind::FailedOperation)),
+        |win| {
+            let ret = unsafe { fl::Fl_handle(val.bits(), win.as_widget_ptr() as _) != 0 };
+            Ok(ret)
+        },
+    )
 }
