@@ -1,4 +1,4 @@
-use crate::app::init::{CURRENT_FONT, FONTS, LOADED_FONT};
+use crate::app::init::{CURRENT_FONT, FONTS};
 use crate::enums::Font;
 use crate::prelude::*;
 use crate::utils::FlString;
@@ -23,8 +23,13 @@ pub fn set_font(new_font: Font) {
 }
 
 /// Set the app's font size
-pub fn set_font_size(sz: u8) {
-    unsafe { fl::Fl_set_font_size(sz as i32) }
+pub fn set_font_size<I: Into<i32>>(sz: I) {
+    unsafe { fl::Fl_set_font_size(sz.into() as i32) }
+}
+
+/// Get the app's font size
+pub fn font_size() -> i32 {
+    unsafe { fl::Fl_font_size() }
 }
 
 /// Get the font's name
@@ -56,9 +61,11 @@ pub fn get_font_sizes(font: Font) -> Vec<i32> {
 }
 
 /// Initializes loaded fonts of a certain pattern `name`
-pub fn set_fonts(name: &str) -> u8 {
+/// Returns the number of fonts found.
+/// For safety purposes, the name can be queried using `get_font_name(unsafe { transmute(index) })`
+pub fn set_fonts(name: &str) -> i32 {
     let name = CString::safe_new(name);
-    unsafe { fl::Fl_set_fonts(name.as_ptr() as *mut raw::c_char) as u8 }
+    unsafe { fl::Fl_set_fonts(name.as_ptr() as *mut raw::c_char) }
 }
 
 /// Gets the name of a font through its index
@@ -118,27 +125,37 @@ pub fn fonts() -> Vec<String> {
 /// Load a font from a file
 pub(crate) fn load_font(path: &str) -> Result<String, FltkError> {
     unsafe {
-        let path = CString::new(path)?;
-        if let Some(load_font) = LOADED_FONT {
-            unload_font(load_font)?;
-        }
-        let ptr = fl::Fl_load_font(path.as_ptr());
-        if ptr.is_null() {
-            Err::<String, FltkError>(FltkError::Internal(FltkErrorKind::FailedOperation))
-        } else {
-            let name = CStr::from_ptr(ptr as *mut _).to_string_lossy().to_string();
-            if let Some(f) = &FONTS {
-                let mut f = f.lock().unwrap();
-                if f.len() < 17 {
-                    f.push(name.clone());
-                } else {
-                    f[16] = name.clone();
-                }
-                fl::Fl_set_font2(16, ptr);
-                Ok(name)
-            } else {
-                Err::<String, FltkError>(FltkError::Internal(FltkErrorKind::FailedOperation))
+        let font_data = std::fs::read(path)?;
+        let face = match ttf_parser::Face::from_slice(&font_data, 0) {
+            Ok(f) => f,
+            Err(_) => {
+                return Err(FltkError::Internal(FltkErrorKind::FailedOperation));
             }
+        };
+        let family_name = face
+            .names()
+            .into_iter()
+            .find(|name| name.name_id == ttf_parser::name_id::FULL_NAME && name.is_unicode())
+            .and_then(|name| name.to_string());
+        let path = CString::new(path)?;
+        let ret = fl::Fl_load_font(path.as_ptr());
+        if let Some(family_name) = family_name {
+            if ret > 0 {
+                if let Some(f) = &FONTS {
+                    let mut f = f.lock().unwrap();
+                    if f.len() < 17 {
+                        f.push(family_name.clone());
+                    } else {
+                        f[16] = family_name.clone();
+                    }
+                    fl::Fl_set_font2(16, CString::safe_new(&family_name).into_raw());
+                }
+                Ok(family_name)
+            } else {
+                Err(FltkError::Internal(FltkErrorKind::FailedOperation))
+            }
+        } else {
+            Err(FltkError::Internal(FltkErrorKind::FailedOperation))
         }
     }
 }
