@@ -24,7 +24,7 @@ pub use visual::*;
 mod widget;
 pub use widget::*;
 
-/// Basic Application struct, used to instatiate, set the scheme and run the event loop
+/// Basic Application struct, used to instantiate, set the scheme and run the event loop
 #[derive(Debug, Copy, Clone)]
 pub struct App {}
 
@@ -77,7 +77,7 @@ impl App {
         Loads a font from a path.
         On success, returns a String with the ttf Font Family name. The font's index is always 16.
         As such only one font can be loaded at a time.
-        The font name can be used with `Font::by_name`, and index with `Font::by_index`.
+        The font name can be used with [`Font::by_name`](`crate::enums::Font::by_name`), and index with [`Font::by_index`](`crate::enums::Font::by_index`).
         # Examples
         ```rust,no_run
         use fltk::{prelude::*, *};
