@@ -116,12 +116,12 @@ pub fn swap_frame_type(new_frame: FrameType) {
     }
 }
 
-/// Get the shadwo width for frames types with shadows
+/// Get the shadow width for frames types with shadows
 pub fn frame_shadow_width() -> i32 {
     unsafe { fl::Fl_box_shadow_width() }
 }
 
-/// Set the shadwo width for frames types with shadows
+/// Set the shadow width for frames types with shadows
 pub fn set_frame_shadow_width(width: i32) {
     unsafe { fl::Fl_set_box_shadow_width(width) }
 }
@@ -186,6 +186,12 @@ pub fn set_color(old: Color, r: u8, g: u8, b: u8) {
     unsafe { fl::Fl_set_color(old.bits() as u32, r, g, b) }
 }
 
+#[cfg(feature = "enable-glwindow")]
+/// Swap a color with a custom RGBA value
+pub fn set_color_with_alpha(old: Color, r: u8, g: u8, b: u8, a: u8) {
+    unsafe { fl::Fl_set_color_with_alpha(old.bits() as u32, r, g, b, a) }
+}
+
 /// Gets the system colors
 pub fn get_system_colors() {
     unsafe { fl::Fl_get_system_colors() }
@@ -201,12 +207,12 @@ pub fn reload_scheme() -> Result<(), FltkError> {
     }
 }
 
-/// Get the default menu linespacing
+/// Get the default menu line-spacing
 pub fn menu_linespacing() -> i32 {
     unsafe { fl::Fl_menu_linespacing() }
 }
 
-/// Set the menu linespacing
+/// Set the menu line-spacing
 pub fn set_menu_linespacing(val: i32) {
     unsafe { fl::Fl_set_menu_linespacing(val) }
 }
@@ -265,6 +271,7 @@ pub fn display() -> Display {
 
 /// Causes all the windows that need it to be redrawn and graphics forced out through the pipes.
 pub fn flush() {
+    assert!(crate::app::is_ui_thread());
     unsafe { fl::Fl_flush() }
 }
 
@@ -303,3 +310,16 @@ pub fn frame_color(col: Color) -> Color {
 pub fn set_frame_color(col: Color) {
     unsafe { fl::Fl_set_box_color(col.bits()) }
 }
+
+/// Add a new symbol, that can be accessed using the `@` prefix
+pub fn add_symbol(label: &str, scalable: bool, draw_cb: fn(Color)) -> Result<(), FltkError> {
+    unsafe {
+        let label = CString::safe_new(label);
+        let ret = fltk_sys::draw::Fl_add_symbol(label.into_raw() as _, mem::transmute(Some(draw_cb)), scalable as _);
+        if ret == 0 {
+            Err(FltkError::Internal(FltkErrorKind::FailedOperation))
+        } else {
+            Ok(())
+        }
+    }
+}
\ No newline at end of file
