@@ -1,5 +1,4 @@
-use crate::enums::{Align, CallbackTrigger, Color, Damage, Event, Font, FrameType, LabelType};
-use crate::image::Image;
+use crate::enums::{Color, Font};
 use crate::prelude::*;
 use crate::utils::FlString;
 use fltk_sys::browser::*;
@@ -35,16 +34,20 @@ use std::{
     }
     ```
 */
-#[derive(WidgetBase, WidgetExt, BrowserExt, Debug)]
+#[derive(Debug)]
 pub struct Browser {
     inner: *mut Fl_Browser,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(Browser, Fl_Browser);
+crate::macros::widget::impl_widget_base!(Browser, Fl_Browser);
+crate::macros::browser::impl_browser_ext!(Browser, Fl_Browser);
+
 /// Defines the browser type
 #[repr(i32)]
-#[derive(WidgetType, Debug, Copy, Clone, PartialEq)]
+#[derive(Debug, Copy, Clone, PartialEq)]
 pub enum BrowserType {
     /// Normal browser
     Normal = 0,
@@ -56,6 +59,8 @@ pub enum BrowserType {
     Multi = 3,
 }
 
+crate::macros::widget::impl_widget_type!(BrowserType);
+
 /// Defines the type of Scrollbar associated with the browser
 #[repr(u8)]
 #[derive(Debug, Copy, Clone, PartialEq)]
@@ -79,37 +84,53 @@ pub enum BrowserScrollbar {
 }
 
 /// Creates a select browser
-#[derive(WidgetBase, WidgetExt, BrowserExt, Debug)]
+#[derive(Debug)]
 pub struct SelectBrowser {
     inner: *mut Fl_Select_Browser,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(SelectBrowser, Fl_Select_Browser);
+crate::macros::widget::impl_widget_base!(SelectBrowser, Fl_Select_Browser);
+crate::macros::browser::impl_browser_ext!(SelectBrowser, Fl_Select_Browser);
+
 /// Creates a multi-browser
-#[derive(WidgetBase, WidgetExt, BrowserExt, Debug)]
+#[derive(Debug)]
 pub struct MultiBrowser {
     inner: *mut Fl_Multi_Browser,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(MultiBrowser, Fl_Multi_Browser);
+crate::macros::widget::impl_widget_base!(MultiBrowser, Fl_Multi_Browser);
+crate::macros::browser::impl_browser_ext!(MultiBrowser, Fl_Multi_Browser);
+
 /// Creates a hold browser
-#[derive(WidgetBase, WidgetExt, BrowserExt, Debug)]
+#[derive(Debug)]
 pub struct HoldBrowser {
     inner: *mut Fl_Hold_Browser,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(HoldBrowser, Fl_Hold_Browser);
+crate::macros::widget::impl_widget_base!(HoldBrowser, Fl_Hold_Browser);
+crate::macros::browser::impl_browser_ext!(HoldBrowser, Fl_Hold_Browser);
+
 /// Creates a file browser
-#[derive(WidgetBase, WidgetExt, BrowserExt, Debug)]
+#[derive(Debug)]
 pub struct FileBrowser {
     inner: *mut Fl_File_Browser,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(FileBrowser, Fl_File_Browser);
+crate::macros::widget::impl_widget_base!(FileBrowser, Fl_File_Browser);
+crate::macros::browser::impl_browser_ext!(FileBrowser, Fl_File_Browser);
+
 /// File types for the `FileBrowser`
 #[repr(i32)]
 #[derive(Copy, Clone, Debug)]
@@ -183,13 +204,16 @@ impl FileBrowser {
 }
 
 /// Creates a `CheckBrowser` widget
-#[derive(WidgetBase, WidgetExt, Debug)]
+#[derive(Debug)]
 pub struct CheckBrowser {
     inner: *mut Fl_Check_Browser,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(CheckBrowser, Fl_Check_Browser);
+crate::macros::widget::impl_widget_base!(CheckBrowser, Fl_Check_Browser);
+
 impl CheckBrowser {
     /// Add an item, returns the number of current items
     pub fn add(&mut self, s: &str, checked: bool) -> i32 {
@@ -234,7 +258,7 @@ impl CheckBrowser {
         unsafe { Fl_Check_Browser_set_checked(self.inner, item) }
     }
 
-    /// Ckeck all of the items
+    /// Check all of the items
     pub fn check_all(&mut self) {
         assert!(!self.was_deleted());
         unsafe { Fl_Check_Browser_check_all(self.inner) }
