@@ -1,27 +1,23 @@
-use crate::enums::{
-    Align, CallbackTrigger, Color, Damage, Event, Font, FrameType, LabelType, Shortcut,
-};
-use crate::image::Image;
 use crate::prelude::*;
 use crate::utils::FlString;
 use fltk_sys::button::*;
-use std::{
-    ffi::{CStr, CString},
-    mem,
-    os::raw,
-};
+use std::ffi::{CStr, CString};
 
 /// Creates a normal button
-#[derive(WidgetBase, WidgetExt, ButtonExt, Debug)]
+#[derive(Debug)]
 pub struct Button {
     inner: *mut Fl_Button,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(Button, Fl_Button);
+crate::macros::widget::impl_widget_base!(Button, Fl_Button);
+crate::macros::button::impl_button_ext!(Button, Fl_Button);
+
 /// Defines the button type, which can be changed dynamically using the `set_type()`.
 #[repr(i32)]
-#[derive(WidgetType, Debug, Copy, Clone, PartialEq)]
+#[derive(Debug, Copy, Clone, PartialEq)]
 pub enum ButtonType {
     /// Normal button
     Normal = 0,
@@ -33,14 +29,21 @@ pub enum ButtonType {
     Hidden = 3,
 }
 
-/// Creates a radio button
-#[derive(WidgetBase, WidgetExt, ButtonExt, Debug)]
+crate::macros::widget::impl_widget_type!(ButtonType);
+
+/// Creates a radio button.
+/// Radio meaning only one can be toggled in the same group
+#[derive(Debug)]
 pub struct RadioButton {
     inner: *mut Fl_Radio_Button,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(RadioButton, Fl_Radio_Button);
+crate::macros::widget::impl_widget_base!(RadioButton, Fl_Radio_Button);
+crate::macros::button::impl_button_ext!(RadioButton, Fl_Radio_Button);
+
 impl RadioButton {
     /// Check whether a `RadioButton` is toggled
     pub fn is_toggled(&self) -> bool {
@@ -57,14 +60,19 @@ impl RadioButton {
     }
 }
 
-/// Creates a radio round button
-#[derive(WidgetBase, WidgetExt, ButtonExt, Debug)]
+/// Creates a radio round button.
+/// Radio meaning only one can be toggled in the same group
+#[derive(Debug)]
 pub struct RadioRoundButton {
     inner: *mut Fl_Radio_Round_Button,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(RadioRoundButton, Fl_Radio_Round_Button);
+crate::macros::widget::impl_widget_base!(RadioRoundButton, Fl_Radio_Round_Button);
+crate::macros::button::impl_button_ext!(RadioRoundButton, Fl_Radio_Round_Button);
+
 impl RadioRoundButton {
     /// Check whether a `RadioRoundButton` is toggled
     pub fn is_toggled(&self) -> bool {
@@ -81,14 +89,19 @@ impl RadioRoundButton {
     }
 }
 
-/// Creates a radio light button
-#[derive(WidgetBase, WidgetExt, ButtonExt, Debug)]
+/// Creates a radio light button.
+/// Radio meaning only one can be toggled in the same group
+#[derive(Debug)]
 pub struct RadioLightButton {
     inner: *mut Fl_Radio_Light_Button,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(RadioLightButton, Fl_Radio_Light_Button);
+crate::macros::widget::impl_widget_base!(RadioLightButton, Fl_Radio_Light_Button);
+crate::macros::button::impl_button_ext!(RadioLightButton, Fl_Radio_Light_Button);
+
 impl RadioLightButton {
     /// Check whether a `RadioLightButton` is toggled
     pub fn is_toggled(&self) -> bool {
@@ -106,13 +119,17 @@ impl RadioLightButton {
 }
 
 /// Creates a round button
-#[derive(WidgetBase, WidgetExt, ButtonExt, Debug)]
+#[derive(Debug)]
 pub struct RoundButton {
     inner: *mut Fl_Round_Button,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(RoundButton, Fl_Round_Button);
+crate::macros::widget::impl_widget_base!(RoundButton, Fl_Round_Button);
+crate::macros::button::impl_button_ext!(RoundButton, Fl_Round_Button);
+
 impl RoundButton {
     /// Check whether a `RoundButton` is toggled
     pub fn is_toggled(&self) -> bool {
@@ -130,13 +147,17 @@ impl RoundButton {
 }
 
 /// Creates a check button
-#[derive(WidgetBase, WidgetExt, ButtonExt, Debug)]
+#[derive(Debug)]
 pub struct CheckButton {
     inner: *mut Fl_Check_Button,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(CheckButton, Fl_Check_Button);
+crate::macros::widget::impl_widget_base!(CheckButton, Fl_Check_Button);
+crate::macros::button::impl_button_ext!(CheckButton, Fl_Check_Button);
+
 impl CheckButton {
     /// Check whether a `CheckButton` is checked
     pub fn is_checked(&self) -> bool {
@@ -156,13 +177,17 @@ impl CheckButton {
 }
 
 /// Creates a toggle button
-#[derive(WidgetBase, WidgetExt, ButtonExt, Debug)]
+#[derive(Debug)]
 pub struct ToggleButton {
     inner: *mut Fl_Toggle_Button,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(ToggleButton, Fl_Toggle_Button);
+crate::macros::widget::impl_widget_base!(ToggleButton, Fl_Toggle_Button);
+crate::macros::button::impl_button_ext!(ToggleButton, Fl_Toggle_Button);
+
 impl ToggleButton {
     /// Check whether a `ToggleButton` is toggled
     pub fn is_toggled(&self) -> bool {
@@ -180,13 +205,17 @@ impl ToggleButton {
 }
 
 /// Creates a light button
-#[derive(WidgetBase, WidgetExt, ButtonExt, Debug)]
+#[derive(Debug)]
 pub struct LightButton {
     inner: *mut Fl_Light_Button,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(LightButton, Fl_Light_Button);
+crate::macros::widget::impl_widget_base!(LightButton, Fl_Light_Button);
+crate::macros::button::impl_button_ext!(LightButton, Fl_Light_Button);
+
 impl LightButton {
     /// Check whether a `LightButton` is on
     pub fn is_on(&self) -> bool {
@@ -204,17 +233,25 @@ impl LightButton {
 }
 
 /// Creates a repeat button
-#[derive(WidgetBase, WidgetExt, ButtonExt, Debug)]
+#[derive(Debug)]
 pub struct RepeatButton {
     inner: *mut Fl_Repeat_Button,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(RepeatButton, Fl_Repeat_Button);
+crate::macros::widget::impl_widget_base!(RepeatButton, Fl_Repeat_Button);
+crate::macros::button::impl_button_ext!(RepeatButton, Fl_Repeat_Button);
+
 /// Creates a return button
-#[derive(WidgetBase, WidgetExt, ButtonExt, Debug)]
+#[derive(Debug)]
 pub struct ReturnButton {
     inner: *mut Fl_Return_Button,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
+
+crate::macros::widget::impl_widget_ext!(ReturnButton, Fl_Return_Button);
+crate::macros::widget::impl_widget_base!(ReturnButton, Fl_Return_Button);
+crate::macros::button::impl_button_ext!(ReturnButton, Fl_Return_Button);
