@@ -6,6 +6,7 @@ use std::{
     ffi::{CStr, CString},
     mem,
     os::raw,
+    path::{Path, PathBuf},
 };
 
 /// Color modes to be used with the color chooser
@@ -33,7 +34,7 @@ pub type NativeFileChooser = FileDialog;
 
 /// Defines the type of dialog, which can be changed dynamically using the `set_type()` method
 #[repr(i32)]
-#[derive(WidgetType, Debug, Copy, Clone, PartialEq)]
+#[derive(Debug, Copy, Clone, PartialEq)]
 pub enum FileDialogType {
     /// Browse file
     BrowseFile = 0,
@@ -49,12 +50,14 @@ pub enum FileDialogType {
     BrowseSaveDir,
 }
 
+crate::macros::widget::impl_widget_type!(FileDialogType);
+
 /// Alias for `NativeFileChooserType`
 pub type NativeFileChooserType = FileDialogType;
 
 /// Defines the File dialog options, which can be set using the `set_option()` method.
 #[repr(i32)]
-#[derive(WidgetType, Debug, Copy, Clone, PartialEq)]
+#[derive(Debug, Copy, Clone, PartialEq)]
 pub enum FileDialogOptions {
     /// No options
     NoOptions = 0,
@@ -68,6 +71,8 @@ pub enum FileDialogOptions {
     UseFilterExt = 8,
 }
 
+crate::macros::widget::impl_widget_type!(FileDialogOptions);
+
 /// Alias to `NativeFileChooserOptions`
 pub type NativeFileChooserOptions = FileDialogOptions;
 
@@ -89,15 +94,15 @@ impl FileDialog {
     }
 
     /// Returns the chosen file name
-    pub fn filename(&self) -> std::path::PathBuf {
+    pub fn filename(&self) -> PathBuf {
         assert!(!self.inner.is_null());
         unsafe {
             let cnt = Fl_Native_File_Chooser_count(self.inner);
             if cnt == 0 {
-                return std::path::PathBuf::from("");
+                return PathBuf::from("");
             }
             let x = Fl_Native_File_Chooser_filenames(self.inner, 0);
-            std::path::PathBuf::from(
+            PathBuf::from(
                 CStr::from_ptr(x as *mut raw::c_char)
                     .to_string_lossy()
                     .to_string(),
@@ -106,14 +111,14 @@ impl FileDialog {
     }
 
     /// Returns the chosen file names
-    pub fn filenames(&self) -> Vec<std::path::PathBuf> {
+    pub fn filenames(&self) -> Vec<PathBuf> {
         assert!(!self.inner.is_null());
         unsafe {
             let cnt = Fl_Native_File_Chooser_count(self.inner);
-            let mut names: Vec<std::path::PathBuf> = vec![];
+            let mut names: Vec<PathBuf> = vec![];
             for i in 0..cnt {
                 let x = Fl_Native_File_Chooser_filenames(self.inner, i);
-                names.push(std::path::PathBuf::from(
+                names.push(PathBuf::from(
                     CStr::from_ptr(x as *mut raw::c_char)
                         .to_string_lossy()
                         .to_string(),
@@ -124,14 +129,14 @@ impl FileDialog {
     }
 
     /// Returns the preset directory
-    pub fn directory(&self) -> std::path::PathBuf {
+    pub fn directory(&self) -> PathBuf {
         assert!(!self.inner.is_null());
         unsafe {
             let x = Fl_Native_File_Chooser_directory(self.inner);
             if x.is_null() {
-                std::path::PathBuf::from("")
+                PathBuf::from("")
             } else {
-                std::path::PathBuf::from(
+                PathBuf::from(
                     CStr::from_ptr(x as *mut raw::c_char)
                         .to_string_lossy()
                         .to_string(),
@@ -143,12 +148,12 @@ impl FileDialog {
     /// Sets the starting directory
     /// # Errors
     /// Errors on non-existent path
-    pub fn set_directory<P: AsRef<std::path::Path>>(&mut self, dir: P) -> Result<(), FltkError> {
+    pub fn set_directory<P: AsRef<Path>>(&mut self, dir: &P) -> Result<(), FltkError> {
         assert!(!self.inner.is_null());
         self.set_directory_(dir.as_ref())
     }
 
-    fn set_directory_(&mut self, dir: &std::path::Path) -> Result<(), FltkError> {
+    fn set_directory_(&mut self, dir: &Path) -> Result<(), FltkError> {
         assert!(!self.inner.is_null());
         let dir = CString::new(dir.to_str().ok_or_else(|| {
             FltkError::Unknown(String::from("Failed to convert path to string"))
@@ -185,11 +190,11 @@ impl FileDialog {
     }
 
     /// Sets the filter for the dialog, can be:
-    /// A single wildcard (eg. `"*.txt"`).
-    /// Multiple wildcards (eg. `"*.{cxx,h,H}"`).
-    /// A descriptive name followed by a `\t` and a wildcard (eg. `"Text Files\t*.txt"`).
-    /// A list of separate wildcards with a `\n` between each (eg. `"*.{cxx,H}\n*.txt"`).
-    /// A list of descriptive names and wildcards (eg. `"C++ Files\t*.{cxx,H}\nTxt Files\t*.txt"`)
+    /// A single wildcard (e.g. `"*.txt"`).
+    /// Multiple wildcards (e.g. `"*.{cxx,h,H}"`).
+    /// A descriptive name followed by a `\t` and a wildcard (e.g. `"Text Files\t*.txt"`).
+    /// A list of separate wildcards with a `\n` between each (e.g. `"*.{cxx,H}\n*.txt"`).
+    /// A list of descriptive names and wildcards (e.g. `"C++ Files\t*.{cxx,H}\nTxt Files\t*.txt"`)
     pub fn set_filter(&mut self, f: &str) {
         assert!(!self.inner.is_null());
         let f = CString::safe_new(f);
@@ -246,7 +251,7 @@ pub fn alert(x: i32, y: i32, txt: &str) {
     }
 }
 
-/// Displays a choice box with upto three choices.
+/// Displays a choice box with up to three choices.
 /// An empty choice will not be shown
 pub fn choice(x: i32, y: i32, txt: &str, b0: &str, b1: &str, b2: &str) -> i32 {
     unsafe {
@@ -258,6 +263,23 @@ pub fn choice(x: i32, y: i32, txt: &str, b0: &str, b1: &str, b2: &str) -> i32 {
     }
 }
 
+/// Displays a choice box with up to three choices.
+/// An empty choice will not be shown. Closing the dialog returns None
+pub fn choice2(x: i32, y: i32, txt: &str, b0: &str, b1: &str, b2: &str) -> Option<i32> {
+    unsafe {
+        let txt = CString::safe_new(txt);
+        let b0 = CString::safe_new(b0);
+        let b1 = CString::safe_new(b1);
+        let b2 = CString::safe_new(b2);
+        let ret = Fl_choice_n(x, y, txt.as_ptr(), b0.as_ptr(), b1.as_ptr(), b2.as_ptr()) as i32;
+        if ret < 0 {
+            None
+        } else {
+            Some(ret)
+        }
+    }
+}
+
 /// Displays an input box, which returns the inputted string.
 /// Can be used for gui io
 pub fn input(x: i32, y: i32, txt: &str, deflt: &str) -> Option<String> {
@@ -311,7 +333,7 @@ pub fn alert_default(txt: &str) {
     }
 }
 
-/// Displays a choice box with upto three choices.
+/// Displays a choice box with up to three choices.
 /// An empty choice will not be shown
 pub fn choice_default(txt: &str, b0: &str, b1: &str, b2: &str) -> i32 {
     unsafe {
@@ -323,6 +345,23 @@ pub fn choice_default(txt: &str, b0: &str, b1: &str, b2: &str) -> i32 {
     }
 }
 
+/// Displays a choice box with up to three choices.
+/// An empty choice will not be shown. Closing the dialog returns None
+pub fn choice2_default(txt: &str, b0: &str, b1: &str, b2: &str) -> Option<i32> {
+    unsafe {
+        let txt = CString::safe_new(txt);
+        let b0 = CString::safe_new(b0);
+        let b1 = CString::safe_new(b1);
+        let b2 = CString::safe_new(b2);
+        let ret = Fl_choice2_n(txt.as_ptr(), b0.as_ptr(), b1.as_ptr(), b2.as_ptr()) as i32;
+        if ret < 0 {
+            None
+        } else {
+            Some(ret)
+        }
+    }
+}
+
 /// Displays an input box, which returns the inputted string.
 /// Can be used for gui io
 pub fn input_default(txt: &str, deflt: &str) -> Option<String> {
@@ -391,11 +430,11 @@ impl HelpDialog {
     /// Loads a file for the help dialog
     /// # Errors
     /// Errors on non-existent path
-    pub fn load<P: AsRef<std::path::Path>>(&mut self, file: P) -> Result<(), FltkError> {
+    pub fn load<P: AsRef<Path>>(&mut self, file: P) -> Result<(), FltkError> {
         self.load_(file.as_ref())
     }
 
-    fn load_(&mut self, file: &std::path::Path) -> Result<(), FltkError> {
+    fn load_(&mut self, file: &Path) -> Result<(), FltkError> {
         let f = file
             .to_str()
             .ok_or_else(|| FltkError::Unknown(String::from("Failed to convert path to string")))?;
@@ -429,7 +468,7 @@ impl HelpDialog {
     }
 
     /// Returns the text size
-    pub fn text_size(&mut self) -> i32 {
+    pub fn text_size(&self) -> i32 {
         unsafe { Fl_Help_Dialog_text_size(self.inner) as i32 }
     }
 
@@ -451,38 +490,43 @@ impl HelpDialog {
         }
     }
 
-    /// Returs whether the help dialog is visible
-    pub fn visible(&mut self) -> bool {
+    /// Returns whether the help dialog is visible
+    pub fn visible(&self) -> bool {
+        unsafe { Fl_Help_Dialog_visible(self.inner) != 0 }
+    }
+
+    /// Returns whether the help dialog is visible
+    pub fn shown(&self) -> bool {
         unsafe { Fl_Help_Dialog_visible(self.inner) != 0 }
     }
 
     /// Returns the width of the help dialog
-    pub fn width(&mut self) -> i32 {
+    pub fn width(&self) -> i32 {
         unsafe { Fl_Help_Dialog_w(self.inner) }
     }
 
     /// Returns the height of the help dialog
-    pub fn height(&mut self) -> i32 {
+    pub fn height(&self) -> i32 {
         unsafe { Fl_Help_Dialog_h(self.inner) }
     }
 
     /// Returns the width of the help dialog
-    pub fn w(&mut self) -> i32 {
+    pub fn w(&self) -> i32 {
         unsafe { Fl_Help_Dialog_w(self.inner) }
     }
 
     /// Returns the height of the help dialog
-    pub fn h(&mut self) -> i32 {
+    pub fn h(&self) -> i32 {
         unsafe { Fl_Help_Dialog_h(self.inner) }
     }
 
     /// Returns the x position of the help dialog
-    pub fn x(&mut self) -> i32 {
+    pub fn x(&self) -> i32 {
         unsafe { Fl_Help_Dialog_x(self.inner) }
     }
 
     /// Returns the y position of the help dialog
-    pub fn y(&mut self) -> i32 {
+    pub fn y(&self) -> i32 {
         unsafe { Fl_Help_Dialog_y(self.inner) }
     }
 }
@@ -564,7 +608,7 @@ pub struct FileChooser {
     inner: *mut Fl_File_Chooser,
 }
 
-bitflags! {
+bitflags::bitflags! {
     /// The types of FileChooser
     pub struct FileChooserType: i32 {
         /// Single file
@@ -580,7 +624,16 @@ bitflags! {
 
 impl FileChooser {
     /// Instantiates a new `FileChooser`
-    pub fn new(dir: &str, pattern: &str, typ: FileChooserType, title: &str) -> FileChooser {
+    pub fn new<P: AsRef<Path>>(dir: P, pattern: &str, typ: FileChooserType, title: &str) -> FileChooser {
+        Self::new_(dir.as_ref(), pattern, typ, title)
+    }
+
+    fn new_(dir: &Path, pattern: &str, typ: FileChooserType, title: &str) -> FileChooser {
+        let dir = if let Some(dir) = dir.to_str() {
+            dir
+        } else {
+            "."
+        };
         let dir = CString::safe_new(dir);
         let pattern = CString::safe_new(pattern);
         let title = CString::safe_new(title);
@@ -595,7 +648,6 @@ impl FileChooser {
             FileChooser { inner: ptr }
         }
     }
-
     /// Deletes a `FileChooser`
     /// # Safety
     /// Can invalidate the underlying pointer
@@ -604,7 +656,7 @@ impl FileChooser {
     }
 
     /// Gets the new button of the `FileChooser`
-    pub fn new_button(&mut self) -> Option<impl ButtonExt> {
+    pub fn new_button(&self) -> Option<impl ButtonExt> {
         assert!(!self.inner.is_null());
         unsafe {
             let ptr = Fl_File_Chooser_newButton(self.inner);
@@ -617,7 +669,7 @@ impl FileChooser {
     }
 
     /// Gets the preview button of the `FileChooser`
-    pub fn preview_button(&mut self) -> Option<impl ButtonExt> {
+    pub fn preview_button(&self) -> Option<impl ButtonExt> {
         assert!(!self.inner.is_null());
         unsafe {
             let ptr = Fl_File_Chooser_previewButton(self.inner);
@@ -632,7 +684,7 @@ impl FileChooser {
     }
 
     /// Gets the show hidden button of the `FileChooser`
-    pub fn show_hidden_button(&mut self) -> Option<impl ButtonExt> {
+    pub fn show_hidden_button(&self) -> Option<impl ButtonExt> {
         assert!(!self.inner.is_null());
         unsafe {
             let ptr = Fl_File_Chooser_showHiddenButton(self.inner);
@@ -674,26 +726,32 @@ impl FileChooser {
     }
 
     /// Gets the color of the `FileChooser`
-    pub fn color(&mut self) -> Color {
+    pub fn color(&self) -> Color {
         assert!(!self.inner.is_null());
         unsafe { mem::transmute(Fl_File_Chooser_color(self.inner)) }
     }
 
     /// Gets the count of chosen items
-    pub fn count(&mut self) -> i32 {
+    pub fn count(&self) -> i32 {
         assert!(!self.inner.is_null());
         unsafe { Fl_File_Chooser_count(self.inner) as i32 }
     }
 
     /// Sets the directory of the `FileChooser`
-    pub fn set_directory(&mut self, dir: &str) {
+    pub fn set_directory<P: AsRef<Path>>(&mut self, dir: P) {
+        self.set_directory_(dir.as_ref())
+    }
+
+    fn set_directory_(&mut self, dir: &Path) {
         assert!(!self.inner.is_null());
-        let dir = CString::safe_new(dir);
-        unsafe { Fl_File_Chooser_set_directory(self.inner, dir.as_ptr()) }
+        if let Some(dir) = dir.to_str() {
+            let dir = CString::safe_new(dir);
+            unsafe { Fl_File_Chooser_set_directory(self.inner, dir.as_ptr()) }
+        }
     }
 
     /// Gets the directory of the `FileChooser`
-    pub fn directory(&mut self) -> Option<String> {
+    pub fn directory(&self) -> Option<String> {
         assert!(!self.inner.is_null());
         unsafe {
             let ptr = Fl_File_Chooser_directory(self.inner);
@@ -710,11 +768,10 @@ impl FileChooser {
     }
 
     /// Sets the filter for the dialog, can be:
-    /// A single wildcard (eg. `"*.txt"`).
-    /// Multiple wildcards (eg. `"*.{cxx,h,H}"`).
-    /// A descriptive name followed by a `\t` and a wildcard (eg. `"Text Files\t*.txt"`).
-    /// A list of separate wildcards with a `\n` between each (eg. `"*.{cxx,H}\n*.txt"`).
-    /// A list of descriptive names and wildcards (eg. `"C++ Files\t*.{cxx,H}\nTxt Files\t*.txt"`)
+    /// Multiple patterns can be used by separating them with tabs, like "*.jpg\t*.png\t*.gif\t*". 
+    /// In addition, you can provide human-readable labels with the patterns inside parenthesis, 
+    /// like "JPEG Files (*.jpg)\tPNG Files (*.png)\tGIF Files (*.gif)\tAll Files (*)
+    /// And "Rust Files (*.{rs,txt,toml})"
     pub fn set_filter(&mut self, pattern: &str) {
         assert!(!self.inner.is_null());
         let pattern = CString::safe_new(pattern);
@@ -722,7 +779,7 @@ impl FileChooser {
     }
 
     /// Gets the filter of the `FileChooser`
-    pub fn filter(&mut self) -> Option<String> {
+    pub fn filter(&self) -> Option<String> {
         assert!(!self.inner.is_null());
         unsafe {
             let ptr = Fl_File_Chooser_filter(self.inner);
@@ -739,17 +796,12 @@ impl FileChooser {
     }
 
     /// Gets the current filename filter selection
-    pub fn filter_value(&mut self) -> i32 {
+    pub fn filter_value(&self) -> i32 {
         assert!(!self.inner.is_null());
         unsafe { Fl_File_Chooser_filter_value(self.inner) as i32 }
     }
 
-    /// Sets the filter for the dialog, can be:
-    /// A single wildcard (eg. `"*.txt"`).
-    /// Multiple wildcards (eg. `"*.{cxx,h,H}"`).
-    /// A descriptive name followed by a `\t` and a wildcard (eg. `"Text Files\t*.txt"`).
-    /// A list of separate wildcards with a `\n` between each (eg. `"*.{cxx,H}\n*.txt"`).
-    /// A list of descriptive names and wildcards (eg. `"C++ Files\t*.{cxx,H}\nTxt Files\t*.txt"`)
+    /// Sets the filter value using an index to the '\t'separated filters
     pub fn set_filter_value(&mut self, f: i32) {
         assert!(!self.inner.is_null());
         unsafe { Fl_File_Chooser_set_filter_value(self.inner, f as i32) }
@@ -768,7 +820,7 @@ impl FileChooser {
     }
 
     /// Gets the icon size of the `FileChooser`
-    pub fn icon_size(&mut self) -> u8 {
+    pub fn icon_size(&self) -> u8 {
         assert!(!self.inner.is_null());
         unsafe { Fl_File_Chooser_iconsize(self.inner) }
     }
@@ -777,11 +829,12 @@ impl FileChooser {
     pub fn set_label(&mut self, l: &str) {
         assert!(!self.inner.is_null());
         let l = CString::safe_new(l);
-        unsafe { Fl_File_Chooser_set_label(self.inner, l.as_ptr()) }
+        let _old = unsafe { CString::from_raw(Fl_File_Chooser_label(self.inner) as _) };
+        unsafe { Fl_File_Chooser_set_label(self.inner, l.into_raw()) }
     }
 
     /// Gets the label of the `FileChooser`
-    pub fn label(&mut self) -> String {
+    pub fn label(&self) -> String {
         assert!(!self.inner.is_null());
         unsafe {
             let ptr = Fl_File_Chooser_label(self.inner);
@@ -796,14 +849,14 @@ impl FileChooser {
     }
 
     /// Sets the label of the Ok button
-    pub fn set_ok_label(&mut self, l: &str) {
+    pub fn set_ok_label(&mut self, l: &'static str) {
         assert!(!self.inner.is_null());
         let l = CString::safe_new(l);
-        unsafe { Fl_File_Chooser_set_ok_label(self.inner, l.as_ptr()) }
+        unsafe { Fl_File_Chooser_set_ok_label(self.inner, l.into_raw()) }
     }
 
     /// Gets the label of the Ok button
-    pub fn ok_label(&mut self) -> String {
+    pub fn ok_label(&self) -> String {
         assert!(!self.inner.is_null());
         unsafe {
             let ptr = Fl_File_Chooser_ok_label(self.inner);
@@ -860,7 +913,7 @@ impl FileChooser {
     }
 
     /// Gets the text color of the file chooser
-    pub fn text_color(&mut self) -> Color {
+    pub fn text_color(&self) -> Color {
         assert!(!self.inner.is_null());
         unsafe { mem::transmute(Fl_File_Chooser_text_color(self.inner)) }
     }
@@ -872,7 +925,7 @@ impl FileChooser {
     }
 
     /// Gets the text font of the file chooser
-    pub fn text_font(&mut self) -> Font {
+    pub fn text_font(&self) -> Font {
         assert!(!self.inner.is_null());
         unsafe { mem::transmute(Fl_File_Chooser_text_font(self.inner)) }
     }
@@ -884,7 +937,7 @@ impl FileChooser {
     }
 
     /// Gets the text size of the file chooser
-    pub fn text_size(&mut self) -> i32 {
+    pub fn text_size(&self) -> i32 {
         assert!(!self.inner.is_null());
         unsafe { Fl_File_Chooser_text_size(self.inner) as i32 }
     }
@@ -896,7 +949,7 @@ impl FileChooser {
     }
 
     /// Gets the type of the `FileChooser`
-    pub fn get_type(&mut self) -> FileChooserType {
+    pub fn get_type(&self) -> FileChooserType {
         assert!(!self.inner.is_null());
         unsafe { mem::transmute(Fl_File_Chooser_type(self.inner)) }
     }
@@ -942,13 +995,13 @@ impl FileChooser {
     }
 
     /// Returns whether the `FileChooser` is visible or not
-    pub fn visible(&mut self) -> bool {
+    pub fn visible(&self) -> bool {
         assert!(!self.inner.is_null());
         unsafe { Fl_File_Chooser_visible(self.inner) != 0 }
     }
 
     /// Return dialog window
-    pub fn window(&mut self) -> impl WindowExt {
+    pub fn window(&self) -> impl WindowExt {
         // Shouldn't fail
         unsafe {
             let win_ptr = self
@@ -1041,11 +1094,57 @@ impl FileChooser {
         unsafe { Fl_File_Chooser_set_show_label(msg.into_raw()) }
     }
 
-    /// Set "Hidden" label
+    /// Set "hidden" label
     pub fn set_hidden_label(msg: &'static str) {
         let msg = CString::safe_new(msg);
         unsafe { Fl_File_Chooser_set_hidden_label(msg.into_raw()) }
     }
+
+    /// Set the position of the file chooser
+    pub fn set_position(&mut self, x: i32, y: i32) {
+        assert!(!self.inner.is_null());
+        unsafe { Fl_File_Chooser_set_position(self.inner, x, y) }
+    }
+
+    /// Set the size of the file chooser
+    pub fn set_size(&mut self, w: i32, h: i32) {
+        assert!(!self.inner.is_null());
+        unsafe { Fl_File_Chooser_set_size(self.inner, w, h) }
+    }
+
+    /// Get the x pos of the file chooser
+    pub fn x(&self) -> i32 {
+        assert!(!self.inner.is_null());
+        unsafe { Fl_File_Chooser_x(self.inner) }
+    }
+
+    /// Get the y pos of the file chooser
+    pub fn y(&self) -> i32 {
+        assert!(!self.inner.is_null());
+        unsafe { Fl_File_Chooser_y(self.inner) }
+    }
+
+    /// Get the width of the file chooser
+    pub fn w(&self) -> i32 {
+        assert!(!self.inner.is_null());
+        unsafe { Fl_File_Chooser_w(self.inner) }
+    }
+
+    /// Get the width of the file chooser
+    pub fn h(&self) -> i32 {
+        assert!(!self.inner.is_null());
+        unsafe { Fl_File_Chooser_h(self.inner) }
+    }
+
+    /// Get the size of the file chooser
+    pub fn size(&self) -> (i32, i32) {
+        (self.w(), self.h())
+    }
+
+    /// Get the position of the file chooser
+    pub fn pos(&self) -> (i32, i32) {
+        (self.x(), self.y())
+    }
 }
 
 impl Drop for FileChooser {
@@ -1074,6 +1173,7 @@ pub fn dir_chooser(message: &str, fname: &str, relative: bool) -> Option<String>
 
 /**
     Shows a file chooser returning a String.
+    The pattern field takes the same argument the [`FileChooser::set_filter`](`crate::dialog::FileChooser::set_filter`) method.
     Example:
     ```rust,no_run
     use fltk::{prelude::*, *};
@@ -1088,26 +1188,34 @@ pub fn dir_chooser(message: &str, fname: &str, relative: bool) -> Option<String>
     }
     ```
 */
-pub fn file_chooser(message: &str, pattern: &str, dir: &str, relative: bool) -> Option<String> {
-    let message = CString::safe_new(message);
-    let pattern = CString::safe_new(pattern);
-    let dir = CString::safe_new(dir);
-    unsafe {
-        let ptr = Fl_file_chooser(
-            message.as_ptr(),
-            pattern.as_ptr(),
-            dir.as_ptr(),
-            relative as i32,
-        );
-        if ptr.is_null() {
-            None
-        } else {
-            Some(
-                CStr::from_ptr(ptr as *mut raw::c_char)
-                    .to_string_lossy()
-                    .to_string(),
-            )
+pub fn file_chooser<P: AsRef<Path>>(message: &str, pattern: &str, dir: P, relative: bool) -> Option<String> {
+    file_chooser_(message, pattern, dir.as_ref(), relative)
+}
+
+fn file_chooser_(message: &str, pattern: &str, dir: &Path, relative: bool) -> Option<String> {
+    if let Some(dir) = dir.to_str() {
+        let message = CString::safe_new(message);
+        let pattern = CString::safe_new(pattern);
+        let dir = CString::safe_new(dir);
+        unsafe {
+            let ptr = Fl_file_chooser(
+                message.as_ptr(),
+                pattern.as_ptr(),
+                dir.as_ptr(),
+                relative as i32,
+            );
+            if ptr.is_null() {
+                None
+            } else {
+                Some(
+                    CStr::from_ptr(ptr as *mut raw::c_char)
+                        .to_string_lossy()
+                        .to_string(),
+                )
+            }
         }
+    } else {
+        None
     }
 }
 
@@ -1142,3 +1250,41 @@ pub fn color_chooser_with_default(name: &str, cmode: ColorMode, col: (u8, u8, u8
         }
     }
 }
+
+/// Set the next dialog's title
+pub fn message_title(title: &str) {
+    let title = CString::safe_new(title);
+    unsafe { Fl_message_title(title.as_ptr() as _) }
+}
+
+/// Set the default title for a dialog
+pub fn message_title_default(title: &str) {
+    let title = CString::safe_new(title);
+    unsafe { Fl_message_title_default(title.as_ptr() as _) }
+}
+
+/// Get the frame holding the icon of FLTK's dialog boxes
+pub fn message_icon() -> impl WidgetExt {
+    unsafe { crate::frame::Frame::from_widget_ptr(Fl_message_icon() as _) }
+}
+
+/// Set whether hotspot is enabled for FLTK's dialog boxes
+pub fn message_set_hotspot(enabled: bool) {
+    unsafe { Fl_message_set_hotspot(enabled as _) }
+}
+
+/// Get whether hotspot is enabled for FLTK's dialog boxes
+pub fn message_hotspot() -> bool {
+    unsafe { Fl_message_hotspot() != 0 }
+}
+
+/// Set the font and font size of FLTK's dialog boxes
+pub fn message_set_font(font: Font, sz: i32) {
+    unsafe { Fl_message_set_font(font.bits(), sz) }
+}
+
+/// Set the next dialog's icon label
+pub fn message_icon_label(label: &str) {
+    let label = CString::safe_new(label);
+    unsafe { Fl_message_icon_label(label.into_raw() as _) }
+}
