@@ -11,7 +11,7 @@ use std::os::raw;
 #[derive(Copy, Clone, Debug)]
 pub struct Coord<T: Copy>(pub T, pub T);
 
-bitflags! {
+bitflags::bitflags! {
     /// Defines the line styles supported by fltk
     pub struct LineStyle: i32 {
         /// Solid line
@@ -48,8 +48,9 @@ pub struct Offscreen {
     inner: *mut raw::c_void,
 }
 
+#[cfg(not(feature = "single-threaded"))]
 unsafe impl Sync for Offscreen {}
-
+#[cfg(not(feature = "single-threaded"))]
 unsafe impl Send for Offscreen {}
 
 impl PartialEq for Offscreen {
@@ -103,7 +104,7 @@ impl Offscreen {
     /// Rescales the offscreen
     pub fn rescale(&mut self) {
         assert!(!self.inner.is_null());
-        unsafe { Fl_rescale_offscreen(self.inner) }
+        unsafe { Fl_rescale_offscreen(&mut self.inner) }
     }
 
     /// Checks the validity of the offscreen
@@ -257,6 +258,8 @@ pub fn draw_pie(x: i32, y: i32, width: i32, height: i32, a: f64, b: f64) {
 }
 
 /// Sets the line style
+/// # Warning
+/// You are required to change this back to [`set_line_style(LineStyle::Solid, 0)`](`crate::draw::set_line_style`) after finishing
 pub fn set_line_style(style: LineStyle, width: i32) {
     unsafe {
         crate::app::open_display();
@@ -431,13 +434,8 @@ pub fn scale_xy(x: f64, y: f64) {
 }
 
 /// Concatenates scaling transformation onto the current one for both x & y
-pub fn scale_x(x: f64) {
-    unsafe { Fl_scale2(x) }
-}
-
-/// Concatenates scaling transformation onto the current one for both x & y
-pub fn scale(x: f64) {
-    unsafe { Fl_scale2(x) }
+pub fn scale(val: f64) {
+    unsafe { Fl_scale2(val) }
 }
 
 /// Concatenates translation transformation onto the current one
@@ -571,6 +569,31 @@ pub fn measure(txt: &str, draw_symbols: bool) -> (i32, i32) {
     (x, y)
 }
 
+/// Measure the width and height of a text
+/// If `width` is non-zero, it will wrap to that width
+pub fn wrap_measure(txt: &str, width: i32, draw_symbols: bool) -> (i32, i32) {
+    let txt = CString::safe_new(txt);
+    let mut x = width;
+    let mut y = 0;
+    unsafe {
+        Fl_measure(txt.as_ptr(), &mut x, &mut y, draw_symbols as i32);
+    }
+    (x, y)
+}
+
+/// Measure the coordinates and size of the text where a bounding box using the returned data would fit the text
+pub fn text_extents(txt: &str) -> (i32, i32, i32, i32) {
+    let txt = CString::safe_new(txt);
+    let mut x = 0;
+    let mut y = 0;
+    let mut w = 0;
+    let mut h = 0;
+    unsafe {
+        Fl_text_extents(txt.as_ptr(), &mut x, &mut y, &mut w, &mut h);
+    }
+    (x, y, w, h)
+}
+
 /// Returns the typographical width of a single character
 pub fn char_width(c: char) -> f64 {
     unsafe { Fl_width3(c as u32) }
@@ -602,24 +625,36 @@ pub fn local_to_latin1(txt: &str, n: i32) -> String {
 
 /// Draws a string starting at the given x, y location
 pub fn draw_text(txt: &str, x: i32, y: i32) {
+    if size() == -1 && txt.len() == 1 {
+        return;
+    }
     let txt = CString::safe_new(txt);
     unsafe { Fl_draw(txt.as_ptr(), x, y) }
 }
 
 /// Draws a string starting at the given x, y location with width and height and alignment
 pub fn draw_text2(string: &str, x: i32, y: i32, width: i32, height: i32, align: Align) {
+    if size() == -1 && string.len() == 1 {
+        return;
+    }
     let s = CString::safe_new(string);
     unsafe { Fl_draw_text2(s.as_ptr(), x, y, width, height, align.bits() as i32) }
 }
 
 /// Draws a string starting at the given x, y location, rotated to an angle
 pub fn draw_text_angled(angle: i32, txt: &str, x: i32, y: i32) {
+    if size() == -1 && txt.len() == 1 {
+        return;
+    }
     let txt = CString::safe_new(txt);
     unsafe { Fl_draw2(angle, txt.as_ptr(), x, y) }
 }
 
 /// Draws a UTF-8 string right to left starting at the given x, y location
 pub fn rtl_draw(txt: &str, x: i32, y: i32) {
+    if size() == -1 && txt.len() == 1 {
+        return;
+    }
     let len = txt.len() as i32;
     let txt = CString::safe_new(txt);
     unsafe { Fl_rtl_draw(txt.as_ptr(), len, x, y) }
@@ -827,7 +862,7 @@ pub unsafe fn draw_rgb_nocopy<T: WidgetBase>(wid: &mut T, fb: &[u8]) {
 }
 
 /// Draw an image into a widget.
-/// Requires a call to `app::set_visual(Mode::Rgb8).unwrap()`
+/// Requires a call to [`app::set_visual(Mode::Rgb8).unwrap()`](`crate::app::set_visual`)
 /// # Errors
 /// Errors on invalid or unsupported image formats
 pub fn draw_image(
@@ -848,8 +883,15 @@ pub fn draw_image(
     Ok(())
 }
 
+/// Draw a check mark
+pub fn draw_check(x: i32, y: i32, w: i32, h: i32, col: Color) {
+    unsafe {
+        Fl_draw_check(x, y, w, h, col.bits());
+    }
+}
+
 /// Draw an image into a widget.
-/// Requires a call to `app::set_visual(Mode::Rgb8).unwrap()`.
+/// Requires a call to [`app::set_visual(Mode::Rgb8).unwrap()`](`crate::app::set_visual`).
 /// A negative depth flips the image horizontally,
 /// while a negative line data flips it vertically.
 /// Allows passing a line-data parameter
