@@ -210,57 +210,57 @@ impl FrameType {
     }
 }
 
-bitflags! {
+bitflags::bitflags! {
     /// Defines alignment rules used by FLTK for labels
     pub struct Align: i32 {
         /// Center
-        const Center = 0;
+        const Center = 0x0000;
         /// Top
-        const Top = 1;
+        const Top = 0x0001;
         /// Bottom
-        const Bottom = 2;
+        const Bottom = 0x0002;
         /// Left
-        const Left = 4;
+        const Left = 0x0004;
         /// Right
-        const Right = 8;
+        const Right = 0x0008;
         /// Inside
-        const Inside = 16;
+        const Inside = 0x0010;
         /// Text Over Image
-        const TextOverImage = 20;
+        const TextOverImage = 0x0020;
         /// Clip
-        const Clip = 40;
+        const Clip = 0x0040;
         /// Wrap
-        const Wrap = 80;
+        const Wrap = 0x0080;
         /// Image Next To Text
-        const ImageNextToText = 100;
+        const ImageNextToText = 0x0100;
         /// Text Next To Image
-        const TextNextToImage = 120;
+        const TextNextToImage = 0x0120;
         /// Image Backdrop
-        const ImageBackdrop = 200;
+        const ImageBackdrop = 0x0200;
         /// Top Left
-        const TopLeft = 1 | 4;
+        const TopLeft = 0x0001 | 0x0004;
         /// Top Right
-        const TopRight = 1 | 8;
+        const TopRight = 0x0001 | 0x0008;
         /// Bottom Left
-        const BottomLeft = 2 | 4;
+        const BottomLeft = 0x0002 | 0x0004;
         /// Bottom Right
-        const BottomRight = 2 | 8;
+        const BottomRight = 0x0002 | 0x0008;
         /// Left Top
-        const LeftTop = 7;
+        const LeftTop = 0x0007;
         /// Right Top
-        const RightTop = 11;
+        const RightTop = 0x000B;
         /// Left Bottom
-        const LeftBottom = 13;
+        const LeftBottom = 0x000D;
         /// Right Bottom
-        const RightBottom = 14;
+        const RightBottom = 0x000E;
         /// Position Mask
-        const PositionMask = 15;
+        const PositionMask = 0x000F;
         /// Image Mask
-        const ImageMask = 320;
+        const ImageMask = 0x0320;
     }
 }
 
-bitflags! {
+bitflags::bitflags! {
     /// Defines fonts used by FLTK
     pub struct Font: i32 {
         /// Helvetica
@@ -299,7 +299,7 @@ bitflags! {
 }
 
 impl Font {
-    /// Returns a font by index, can be queried via the `app::get_font_names()`
+    /// Returns a font by index, can be queried via the [`app::get_font_names()`](`crate::app::get_font_names()`)
     pub fn by_index(idx: usize) -> Font {
         unsafe {
             if let Some(f) = &FONTS {
@@ -314,7 +314,7 @@ impl Font {
         }
     }
 
-    /// Gets the font by its name, can be queried via the `app::get_font_names()`
+    /// Gets the font by its name, can be queried via the [`app::get_font_names()`](`crate::app::get_font_names`)
     pub fn by_name(name: &str) -> Font {
         match font_index(name) {
             Some(val) => Font::by_index(val),
@@ -357,16 +357,31 @@ impl Font {
                 ));
             }
             if let Some(p) = path.to_str() {
+                let font_data = std::fs::read(path)?;
+                let face = match ttf_parser::Face::from_slice(&font_data, 0) {
+                    Ok(f) => f,
+                    Err(_) => {
+                        return Err(FltkError::Internal(FltkErrorKind::FailedOperation));
+                    }
+                };
+                let family_name = face
+                    .names()
+                    .into_iter()
+                    .find(|name| name.name_id == ttf_parser::name_id::FULL_NAME && name.is_unicode())
+                    .and_then(|name| name.to_string());
                 let path = CString::safe_new(p);
-                let ptr = fl::Fl_load_font(path.as_ptr());
-                if ptr.is_null() {
-                    Err::<String, FltkError>(FltkError::Internal(FltkErrorKind::FailedOperation))
+                let ret = fl::Fl_load_font(path.as_ptr());
+                if let Some(family_name) = family_name {
+                    if ret > 0 {
+                        Ok(family_name)
+                    } else {
+                        Err(FltkError::Internal(FltkErrorKind::FailedOperation))
+                    }
                 } else {
-                    let name = CStr::from_ptr(ptr as *mut _).to_string_lossy().to_string();
-                    Ok(name)
+                    Err(FltkError::Internal(FltkErrorKind::FailedOperation))
                 }
             } else {
-                Err(FltkError::Internal(FltkErrorKind::ResourceNotFound))
+                Err(FltkError::Internal(FltkErrorKind::FailedOperation))
             }
         }
     }
@@ -381,7 +396,7 @@ impl Font {
     }
 }
 
-bitflags! {
+bitflags::bitflags! {
     /// Defines colors used by FLTK.
     /// Colors are stored as RGBI values, the last being the index for FLTK colors in this enum.
     /// Colors in this enum don't have an RGB stored. However, custom colors have an RGB, and don't have an index.
@@ -395,8 +410,12 @@ bitflags! {
     pub struct Color: u32 {
         /// ForeGround, label colors
         const ForeGround = 0;
+        /// Foreground, label colors
+        const Foreground = 0;
         /// BackGround2, Is the color inside input, output and text display widgets
         const BackGround2 = 7;
+        /// Background2, Is the color inside input, output and text display widgets
+        const Background2 = 7;
         /// Inactive
         const Inactive = 8;
         /// Selection
@@ -417,6 +436,8 @@ bitflags! {
         const FrameDefault = 49;
         /// BackGround
         const BackGround = 49;
+        /// Background
+        const Background = 49;
         /// Light1
         const Light1 = 50;
         /// Light2
@@ -456,38 +477,87 @@ bitflags! {
 
 impl Color {
     /// Returns a color from RGB
-    pub fn from_rgb(r: u8, g: u8, b: u8) -> Color {
-        unsafe { mem::transmute(fl::Fl_get_rgb_color(r, g, b)) }
+    pub const fn from_rgb(r: u8, g: u8, b: u8) -> Color {
+        let r = r as u32;
+        let g = g as u32;
+        let b = b as u32;
+        let val: u32 = ((r & 0xff) << 24) + ((g & 0xff) << 16) + ((b & 0xff) << 8);
+        Color::from_rgbi(val)
+    }
+
+    /// Get a color from an RGBI value, the I stands for the Fltk colormap index.
+    pub const fn from_rgbi(val: u32) -> Color {
+        let mut c = Color::Black;
+        c.bits = val;
+        c
+    }
+
+    /// Create color from RGBA using alpha compositing
+    pub fn from_rgba_tuple(tup: (u8, u8, u8, u8)) -> Color {
+        if tup.3 != 255 {
+            let bg_col = if let Some(grp) = crate::group::Group::try_current() {
+                use crate::prelude::WidgetExt;
+                grp.color()
+            } else {
+                Color::BackGround
+            };
+            let bg_col = bg_col.to_rgb();
+            let alpha = tup.3 as f32 / 255.0;
+            let r = alpha * tup.0 as f32 + (1.0 - alpha) * bg_col.0 as f32;
+            let r = r as u8;
+            let g = alpha * tup.1 as f32 + (1.0 - alpha) * bg_col.1 as f32;
+            let g = g as u8;
+            let b = alpha * tup.2 as f32 + (1.0 - alpha) * bg_col.2 as f32;
+            let b = b as u8;
+            Color::from_rgb(r, g, b)
+        } else {
+            Color::from_rgb(tup.0, tup.1, tup.2)
+        }
     }
 
     /// Returns a color from hex or decimal
-    pub fn from_u32(val: u32) -> Color {
+    pub const fn from_u32(val: u32) -> Color {
         let (r, g, b) = utils::hex2rgb(val);
         Color::from_rgb(r, g, b)
     }
 
     /// Returns a color from hex or decimal
-    pub fn from_hex(val: u32) -> Color {
+    pub const fn from_hex(val: u32) -> Color {
         let (r, g, b) = utils::hex2rgb(val);
         Color::from_rgb(r, g, b)
     }
 
+    /// Return a Color from a hex color format (`#xxxxxx`)
+    pub fn from_hex_str(col: &str) -> Result<Color, FltkError> {
+        if !col.starts_with('#') || col.len() != 7 {
+            Err(FltkError::Internal(FltkErrorKind::InvalidColor))
+        } else {
+            Ok(Color::from_hex(u32::from_str_radix(&col[1..7], 16)?))
+        }
+    }
+
+    /// Returns the color in hex string format
+    pub fn to_hex_str(&self) -> String {
+        let (r, g, b) = self.to_rgb();
+        format!("#{:02x}{:02x}{:02x}", r, g, b)
+    }
+
     /// Returns a color by index of RGBI
     pub fn by_index(idx: u8) -> Color {
         unsafe { mem::transmute(idx as u32) }
     }
 
-    /// Retuns an inactive form of the color
+    /// Returns an inactive form of the color
     pub fn inactive(&self) -> Color {
         unsafe { mem::transmute(fl::Fl_inactive(self.bits)) }
     }
 
-    /// Retuns an darker form of the color
+    /// Returns an darker form of the color
     pub fn darker(&self) -> Color {
         unsafe { mem::transmute(fl::Fl_darker(self.bits)) }
     }
 
-    /// Retuns an lighter form of the color
+    /// Returns an lighter form of the color
     pub fn lighter(&self) -> Color {
         unsafe { mem::transmute(fl::Fl_lighter(self.bits)) }
     }
@@ -544,6 +614,10 @@ impl std::fmt::Display for Color {
         match *self {
             Color::ForeGround => write!(f, "Color::ForeGround"),
             Color::BackGround => write!(f, "Color::BackGround"),
+            Color::BackGround2 => write!(f, "Color::BackGround2"),
+            Color::Foreground => write!(f, "Color::Foreground"),
+            Color::Background => write!(f, "Color::Background"),
+            Color::Background2 => write!(f, "Color::Background2"),
             Color::Inactive => write!(f, "Color::Inactive"),
             Color::Selection => write!(f, "Color::Selection"),
             Color::Gray0 => write!(f, "Color::Gray0"),
@@ -576,7 +650,7 @@ impl std::fmt::Display for Color {
     }
 }
 
-bitflags! {
+bitflags::bitflags! {
     /// Defines event types captured by FLTK
     pub struct Event: i32 {
         /// No Event
@@ -644,14 +718,16 @@ bitflags! {
 
 impl Event {
     /// Creates an event from an i32 value
-    pub fn from_i32(val: i32) -> Event {
-        unsafe { mem::transmute(val) }
+    pub const fn from_i32(val: i32) -> Event {
+        let mut ev = Event::NoEvent;
+        ev.bits = val;
+        ev
     }
 }
 
 impl From<i32> for Event {
     fn from(val: i32) -> Event {
-        unsafe { mem::transmute(val) }
+        Event::from_i32(val)
     }
 }
 
@@ -695,7 +771,7 @@ impl std::fmt::Display for Event {
     }
 }
 
-bitflags! {
+bitflags::bitflags! {
     /// Defines the inputted virtual keycode
     pub struct Key: i32 {
         /// None
@@ -756,7 +832,31 @@ bitflags! {
         const KPEnter = 0xff8d;
         /// Keypad Last
         const KPLast = 0xffbd;
-        /// FLast
+        /// F1
+        const F1 = 0xffbd + 1;
+        /// F2
+        const F2 = 0xffbd + 2;
+        /// F3
+        const F3 = 0xffbd + 3;
+        /// F4
+        const F4 = 0xffbd + 4;
+        /// F5
+        const F5 = 0xffbd + 5;
+        /// F6
+        const F6 = 0xffbd + 6;
+        /// F7
+        const F7 = 0xffbd + 7;
+        /// F8
+        const F8 = 0xffbd + 8;
+        /// F9
+        const F9 = 0xffbd + 9;
+        /// F10
+        const F10 = 0xffbd + 10;
+        /// F11
+        const F11 = 0xffbd + 11;
+        /// F12
+        const F12 = 0xffbd + 12;
+        /// Function key last
         const FLast = 0xffe0;
         /// Shift Left
         const ShiftL = 0xffe1;
@@ -783,17 +883,41 @@ bitflags! {
 
 impl Key {
     /// Gets a Key from an i32
-    pub fn from_i32(val: i32) -> Key {
-        unsafe { mem::transmute(val) }
+    pub const fn from_i32(val: i32) -> Key {
+        let mut k = Key::None;
+        k.bits = val;
+        k
     }
 
     /// Gets a Key from a char
-    pub fn from_char(val: char) -> Key {
-        unsafe { mem::transmute(val) }
+    pub const fn from_char(val: char) -> Key {
+        let mut k = Key::None;
+        k.bits = val as i32;
+        k
+    }
+
+    /// Get the char representation of a Key.
+    pub const fn to_char(&self) -> Option<char> {
+        let bits = self.bits;
+        if bits >= 0xD800 && bits <= 0xDFFF {
+            None
+        } else {
+            Some(bits as u8 as char)
+        }
+    }
+
+    /// Returns whether a key is a function key
+    pub const fn is_fn_key(key: Key) -> bool {
+        key.bits() >= Key::F1.bits() && key.bits() < Key::FLast.bits()
+    }
+
+    /// Return the corresponding function key
+    pub const fn fn_key(val: i32) -> Key {
+        Key::from_i32(Key::F1.bits() - 1 + val)
     }
 }
 
-bitflags! {
+bitflags::bitflags! {
     /// Defines the modifiers of virtual keycodes
     pub struct Shortcut: i32 {
         /// None
@@ -836,34 +960,42 @@ pub type EventState = Shortcut;
 
 impl Shortcut {
     /// Create a shortcut from a char
-    pub fn from_char(c: char) -> Shortcut {
-        Shortcut::None | c
+    pub const fn from_char(c: char) -> Shortcut {
+        let mut s = Shortcut::None;
+        s.bits = c as _;
+        s
     }
 
     /// Create a shortcut from a key
-    pub fn from_key(k: Key) -> Shortcut {
-        Shortcut::None | k
+    pub const fn from_key(k: Key) -> Shortcut {
+        let mut s = Shortcut::None;
+        s.bits = k.bits();
+        s
     }
 
     /// Create a shortcut from an i32
-    pub fn from_i32(v: i32) -> Shortcut {
-        Shortcut::None | Key::from_i32(v)
+    pub const fn from_i32(v: i32) -> Shortcut {
+        let mut s = Shortcut::None;
+        s.bits = v;
+        s
     }
 
     /// get key mask
-    pub fn key(&self) -> Key {
+    pub const fn key(&self) -> Key {
         let mut temp = self.bits;
         temp &= 0x0000_ffff;
         Key::from_i32(temp)
     }
 
     /// Get the button number
-    pub fn button(button_num: i32) -> Shortcut {
-        unsafe { mem::transmute(0x0080_0000 << button_num) }
+    pub const fn button(button_num: i32) -> Shortcut {
+        let mut s = Shortcut::None;
+        s.bits = 0x0080_0000 << button_num;
+        s
     }
 }
 
-bitflags! {
+bitflags::bitflags! {
     /// Defines the types of triggers for widget callback functions. Equivalent to FL_WHEN
     pub struct CallbackTrigger: i32 {
         /// Never
@@ -933,8 +1065,8 @@ pub enum Cursor {
     None = 255,
 }
 
-bitflags! {
-    /// Defines visual mode types (capabilites of the window).
+bitflags::bitflags! {
+    /// Defines visual mode types (capabilities of the window).
     /// Rgb and Single have a value of zero, so they
     /// are "on" unless you give Index or Double.
     pub struct Mode: i32 {
@@ -956,7 +1088,7 @@ bitflags! {
         const Stencil = 32;
         /// Rgb8 color with at least 8 bits of each color
         const Rgb8 = 64;
-        /// MultiSample antialiasing
+        /// MultiSample anti-aliasing
         const MultiSample = 128;
         /// Stereoscopic rendering
         const Stereo = 256;
@@ -967,7 +1099,7 @@ bitflags! {
     }
 }
 
-bitflags! {
+bitflags::bitflags! {
     /// Damage masks
     pub struct Damage: u8 {
         /// No damage
