@@ -1,5 +1,4 @@
-use crate::enums::{Align, CallbackTrigger, Color, Damage, Event, Font, FrameType, LabelType};
-use crate::image::Image;
+use crate::enums::Align;
 use crate::prelude::*;
 use crate::utils::FlString;
 use crate::widget::Widget;
@@ -7,18 +6,22 @@ use fltk_sys::group::*;
 use std::{
     ffi::{CStr, CString},
     mem,
-    os::raw,
 };
 
-/// Creates a widget group
-#[derive(WidgetBase, WidgetExt, GroupExt, Debug)]
+/// Creates a group widget
+#[derive(Debug)]
 pub struct Group {
     inner: *mut Fl_Group,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(Group, Fl_Group);
+crate::macros::widget::impl_widget_base!(Group, Fl_Group);
+crate::macros::group::impl_group_ext!(Group, Fl_Group);
+
 impl Group {
+    #[deprecated(since = "1.2.18", note = "please use `try_current` instead")]
     /// Get the current group
     pub fn current() -> Group {
         unsafe {
@@ -27,19 +30,35 @@ impl Group {
             Group::from_widget_ptr(ptr as _)
         }
     }
+
+    /// Tries to get the current group
+    pub fn try_current() -> Option<Group> {
+        unsafe {
+            let ptr = Fl_Group_current();
+            if ptr.is_null() {
+                None
+            } else {
+                Some(Group::from_widget_ptr(ptr as _))
+            }
+        }
+    }
 }
 
 /// Creates a widget pack
-#[derive(WidgetBase, WidgetExt, GroupExt, Debug)]
+#[derive(Debug)]
 pub struct Pack {
     inner: *mut Fl_Pack,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(Pack, Fl_Pack);
+crate::macros::widget::impl_widget_base!(Pack, Fl_Pack);
+crate::macros::group::impl_group_ext!(Pack, Fl_Pack);
+
 /// Defines pack types
 #[repr(i32)]
-#[derive(WidgetType, Debug, Copy, Clone, PartialEq)]
+#[derive(Debug, Copy, Clone, PartialEq)]
 pub enum PackType {
     /// Vertical layout pack
     Vertical = 0,
@@ -47,17 +66,63 @@ pub enum PackType {
     Horizontal = 1,
 }
 
+crate::macros::widget::impl_widget_type!(PackType);
+
+impl Pack {
+    /// Get the spacing of the pack
+    pub fn spacing(&self) -> i32 {
+        assert!(!self.was_deleted());
+        unsafe { Fl_Pack_spacing(self.inner) }
+    }
+
+    /// Set the spacing of the pack
+    pub fn set_spacing(&mut self, spacing: i32) {
+        unsafe {
+            assert!(!self.was_deleted());
+            Fl_Pack_set_spacing(self.inner, spacing);
+        }
+    }
+
+    /// Layout the children of the pack automatically.
+    /// Must be called on existing children
+    pub fn auto_layout(&mut self) {
+        let children = self.children() as i32;
+        if children == 0 {
+            return;
+        }
+        let spacing = self.spacing() * (children - 1);
+        let t = self.get_type::<PackType>();
+        let w = (self.width() - spacing) / children;
+        let h = (self.height() - spacing) / children;
+
+        for i in 0..children {
+            let mut c = self.child(i as i32).unwrap();
+            let c_w = c.width();
+            let c_h = c.height();
+            if t == PackType::Vertical {
+                c.set_size(c_w, h);
+            } else {
+                c.set_size(w, c_h);
+            }
+        }
+    }
+}
+
 /// Creates a scroll group
-#[derive(WidgetBase, WidgetExt, GroupExt, Debug)]
+#[derive(Debug)]
 pub struct Scroll {
     inner: *mut Fl_Scroll,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(Scroll, Fl_Scroll);
+crate::macros::widget::impl_widget_base!(Scroll, Fl_Scroll);
+crate::macros::group::impl_group_ext!(Scroll, Fl_Scroll);
+
 /// Defines Scroll types
 #[repr(i32)]
-#[derive(WidgetType, Debug, Copy, Clone, PartialEq)]
+#[derive(Debug, Copy, Clone, PartialEq)]
 pub enum ScrollType {
     /// Never show bars
     None = 0,
@@ -77,6 +142,8 @@ pub enum ScrollType {
     BothAlways = 7,
 }
 
+crate::macros::widget::impl_widget_type!(ScrollType);
+
 impl Scroll {
     /// Returns the vertical scrollbar
     pub fn scrollbar(&self) -> crate::valuator::Scrollbar {
@@ -130,16 +197,20 @@ impl Scroll {
 }
 
 /// Creates a tab which can contain widgets
-#[derive(WidgetBase, WidgetExt, GroupExt, Debug)]
+#[derive(Debug)]
 pub struct Tabs {
     inner: *mut Fl_Tabs,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(Tabs, Fl_Tabs);
+crate::macros::widget::impl_widget_base!(Tabs, Fl_Tabs);
+crate::macros::group::impl_group_ext!(Tabs, Fl_Tabs);
+
 impl Tabs {
     /// Gets the currently visible group
-    pub fn value(&mut self) -> Option<impl GroupExt> {
+    pub fn value(&self) -> Option<impl GroupExt> {
         assert!(!self.was_deleted());
         unsafe {
             let ptr = Fl_Tabs_value(self.inner);
@@ -201,7 +272,7 @@ impl Tabs {
     }
 
     /// Returns the position and size available to be used by its children
-    pub fn client_area(&mut self) -> (i32, i32, i32, i32) {
+    pub fn client_area(&self) -> (i32, i32, i32, i32) {
         assert!(!self.was_deleted());
         unsafe {
             let mut i1 = 0;
@@ -227,21 +298,29 @@ impl Tabs {
 }
 
 /// Creates a tile which can contain widgets
-#[derive(WidgetBase, WidgetExt, GroupExt, Debug)]
+#[derive(Debug)]
 pub struct Tile {
     inner: *mut Fl_Tile,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(Tile, Fl_Tile);
+crate::macros::widget::impl_widget_base!(Tile, Fl_Tile);
+crate::macros::group::impl_group_ext!(Tile, Fl_Tile);
+
 /// Creates a wizard widget
-#[derive(WidgetBase, WidgetExt, GroupExt, Debug)]
+#[derive(Debug)]
 pub struct Wizard {
     inner: *mut Fl_Wizard,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(Wizard, Fl_Wizard);
+crate::macros::widget::impl_widget_base!(Wizard, Fl_Wizard);
+crate::macros::group::impl_group_ext!(Wizard, Fl_Wizard);
+
 impl Wizard {
     /// Gets the next view of the wizard
     pub fn next(&mut self) {
@@ -255,11 +334,27 @@ impl Wizard {
         unsafe { Fl_Wizard_prev(self.inner) }
     }
 
+    #[deprecated(since = "1.2.18", note = "please use `try_current_widget` instead")]
     /// Gets the underlying widget of the current view
-    pub fn current_widget(&mut self) -> Widget {
+    pub fn current_widget(&self) -> Widget {
         unsafe {
             assert!(!self.was_deleted());
-            Widget::from_widget_ptr(Fl_Wizard_value(self.inner) as *mut fltk_sys::widget::Fl_Widget)
+            let ptr = Fl_Wizard_value(self.inner) as *mut fltk_sys::widget::Fl_Widget;
+            assert!(!ptr.is_null());
+            Widget::from_widget_ptr(ptr)
+        }
+    }
+
+    /// Gets the underlying widget of the current view
+    pub fn try_current_widget(&self) -> Option<impl WidgetExt> {
+        unsafe {
+            assert!(!self.was_deleted());
+            let ptr = Fl_Wizard_value(self.inner) as *mut fltk_sys::widget::Fl_Widget;
+            if ptr.is_null() {
+                None
+            } else {
+                Some(Widget::from_widget_ptr(ptr))
+            }
         }
     }
 
@@ -276,13 +371,17 @@ impl Wizard {
 }
 
 /// Creates a color chooser widget
-#[derive(WidgetBase, WidgetExt, GroupExt, Debug)]
+#[derive(Debug)]
 pub struct ColorChooser {
     inner: *mut Fl_Color_Chooser,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(ColorChooser, Fl_Color_Chooser);
+crate::macros::widget::impl_widget_base!(ColorChooser, Fl_Color_Chooser);
+crate::macros::group::impl_group_ext!(ColorChooser, Fl_Color_Chooser);
+
 impl ColorChooser {
     /// Return the rgb color
     pub fn rgb_color(&self) -> (u8, u8, u8) {
@@ -303,49 +402,11 @@ impl ColorChooser {
     }
 }
 
-impl Pack {
-    /// Get the spacing of the pack
-    pub fn spacing(&self) -> i32 {
-        assert!(!self.was_deleted());
-        unsafe { Fl_Pack_spacing(self.inner) }
-    }
-
-    /// Set the spacing of the pack
-    pub fn set_spacing(&mut self, spacing: i32) {
-        unsafe {
-            assert!(!self.was_deleted());
-            Fl_Pack_set_spacing(self.inner, spacing);
-        }
-    }
-
-    /// Layout the children of the pack automatically.
-    /// Must be called on existing children
-    pub fn auto_layout(&mut self) {
-        let children = self.children() as i32;
-        if children == 0 {
-            return;
-        }
-        let spacing = self.spacing() * (children - 1);
-        let t = self.get_type::<PackType>();
-        let w = (self.width() - spacing) / children;
-        let h = (self.height() - spacing) / children;
-
-        for i in 0..children {
-            let mut c = self.child(i as i32).unwrap();
-            let c_w = c.width();
-            let c_h = c.height();
-            if t == PackType::Vertical {
-                c.set_size(c_w, h);
-            } else {
-                c.set_size(w, c_h);
-            }
-        }
-    }
-}
+crate::macros::widget::impl_widget_type!(FlexType);
 
 /// Defines Flex types
 #[repr(i32)]
-#[derive(WidgetType, Debug, Copy, Clone, PartialEq)]
+#[derive(Debug, Copy, Clone, PartialEq)]
 pub enum FlexType {
     /// row direction
     Row = 0,
@@ -373,22 +434,32 @@ pub enum FlexType {
     }
     ```
 */
-#[derive(WidgetBase, WidgetExt, GroupExt, Debug)]
+#[derive(Debug)]
 pub struct Flex {
     inner: *mut Fl_Flex,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(Flex, Fl_Flex);
+crate::macros::widget::impl_widget_base!(Flex, Fl_Flex);
+crate::macros::group::impl_group_ext!(Flex, Fl_Flex);
+
 impl Flex {
+    /// Add a widget to the Flex box
+    pub fn add<W: WidgetExt>(&mut self, widget: &W) {
+        <Self as GroupExt>::add(self, widget);
+        self.recalc();
+    }
+
     /// Set the size of the widget
-    pub fn set_size<W: WidgetExt>(&mut self, w: &mut W, size: i32) {
+    pub fn set_size<W: WidgetExt>(&mut self, w: &W, size: i32) {
         unsafe { Fl_Flex_set_size(self.inner, w.as_widget_ptr() as _, size) }
     }
 
     /// Debug the flex layout
-    pub fn debug(&mut self, flag: bool) {
-        unsafe { Fl_Flex_set_debug(self.inner, flag as _) }
+    pub fn debug(flag: bool) {
+        unsafe { Fl_Flex_set_debug(flag as _) }
     }
 
     /// Set the type to be a column
@@ -404,9 +475,29 @@ impl Flex {
     }
 
     /// Recalculate children's coords and sizes
-    pub fn recalc(&mut self) {
+    pub fn recalc(&self) {
         self.end();
     }
+
+    /// Set the margin
+    pub fn set_margin(&mut self, m: i32) {
+        unsafe { Fl_Flex_set_margin(self.inner, m) }
+    }
+
+    /// Get the margin
+    pub fn margin(&self) -> i32 {
+        unsafe { Fl_Flex_margin(self.inner) }
+    }
+
+    /// Set the padding
+    pub fn set_pad(&mut self, p: i32) {
+        unsafe { Fl_Flex_set_pad(self.inner, p) }
+    }
+
+    /// Get the padding
+    pub fn pad(&self) -> i32 {
+        unsafe { Fl_Flex_pad(self.inner) }
+    }
 }
 
 /**
@@ -718,3 +809,8 @@ impl Row {
 }
 
 crate::widget_extends!(Row, Flex, p);
+
+/// Experimental group widgets
+pub mod experimental {
+    // use super::*;
+}
