@@ -2,26 +2,26 @@ use crate::enums::ColorDepth;
 use crate::prelude::*;
 use crate::utils::FlString;
 use fltk_sys::image::*;
-use std::{
-    ffi::CString,
-    mem,
-    sync::atomic::{AtomicUsize, Ordering},
-};
+use std::{ffi::CString, mem, sync::atomic::AtomicUsize};
 
 /// Wrapper around `Fl_Image`, used to wrap other image types
-#[derive(ImageExt, Debug)]
+#[derive(Debug)]
 pub struct Image {
     inner: *mut Fl_Image,
     refcount: AtomicUsize,
 }
 
+crate::macros::image::impl_image_ext!(Image, Fl_Image);
+
 /// Creates a struct holding a shared image
-#[derive(ImageExt, Debug)]
+#[derive(Debug)]
 pub struct SharedImage {
     inner: *mut Fl_Shared_Image,
     refcount: AtomicUsize,
 }
 
+crate::macros::image::impl_image_ext!(SharedImage, Fl_Shared_Image);
+
 impl SharedImage {
     /// Loads a `SharedImage` from a path
     /// # Errors
@@ -77,12 +77,14 @@ impl SharedImage {
 }
 
 /// Creates a struct holding a Jpeg image
-#[derive(ImageExt, Debug)]
+#[derive(Debug)]
 pub struct JpegImage {
     inner: *mut Fl_JPEG_Image,
     refcount: AtomicUsize,
 }
 
+crate::macros::image::impl_image_ext!(JpegImage, Fl_JPEG_Image);
+
 impl JpegImage {
     /// Loads the image from a filesystem path, doesn't check for the validity of the data
     /// # Errors
@@ -141,12 +143,14 @@ impl JpegImage {
 }
 
 /// Creates a struct holding a PNG image
-#[derive(ImageExt, Debug)]
+#[derive(Debug)]
 pub struct PngImage {
     inner: *mut Fl_PNG_Image,
     refcount: AtomicUsize,
 }
 
+crate::macros::image::impl_image_ext!(PngImage, Fl_PNG_Image);
+
 impl PngImage {
     /// Loads the image from a filesystem path, doesn't check for the validity of the data
     /// # Errors
@@ -205,12 +209,14 @@ impl PngImage {
 }
 
 /// Creates a struct holding an SVG image
-#[derive(ImageExt, Debug)]
+#[derive(Debug)]
 pub struct SvgImage {
     inner: *mut Fl_SVG_Image,
     refcount: AtomicUsize,
 }
 
+crate::macros::image::impl_image_ext!(SvgImage, Fl_SVG_Image);
+
 impl SvgImage {
     /// Loads the image from a filesystem path, doesn't check for the validity of the data
     /// # Errors
@@ -276,12 +282,14 @@ impl SvgImage {
 }
 
 /// Creates a struct holding a BMP image
-#[derive(ImageExt, Debug)]
+#[derive(Debug)]
 pub struct BmpImage {
     inner: *mut Fl_BMP_Image,
     refcount: AtomicUsize,
 }
 
+crate::macros::image::impl_image_ext!(BmpImage, Fl_BMP_Image);
+
 impl BmpImage {
     /// Loads the image from a filesystem path, doesn't check for the validity of the data
     /// # Errors
@@ -322,7 +330,7 @@ impl BmpImage {
             if data.is_empty() {
                 Err(FltkError::Internal(FltkErrorKind::ResourceNotFound))
             } else {
-                let x = Fl_BMP_Image_from(data.as_ptr());
+                let x = Fl_BMP_Image_from(data.as_ptr(), data.len() as _);
                 if x.is_null() {
                     Err(FltkError::Internal(FltkErrorKind::ResourceNotFound))
                 } else {
@@ -340,12 +348,14 @@ impl BmpImage {
 }
 
 /// Creates a struct holding a GIF image
-#[derive(ImageExt, Debug)]
+#[derive(Debug)]
 pub struct GifImage {
     inner: *mut Fl_GIF_Image,
     refcount: AtomicUsize,
 }
 
+crate::macros::image::impl_image_ext!(GifImage, Fl_GIF_Image);
+
 impl GifImage {
     /// Loads the image from a filesystem path, doesn't check for the validity of the data
     /// # Errors
@@ -386,7 +396,7 @@ impl GifImage {
             if data.is_empty() {
                 Err(FltkError::Internal(FltkErrorKind::ResourceNotFound))
             } else {
-                let x = Fl_GIF_Image_from(data.as_ptr());
+                let x = Fl_GIF_Image_from(data.as_ptr(), data.len() as _);
                 if x.is_null() {
                     Err(FltkError::Internal(FltkErrorKind::ResourceNotFound))
                 } else {
@@ -404,12 +414,14 @@ impl GifImage {
 }
 
 /// Creates a struct holding a XPM image
-#[derive(ImageExt, Debug)]
+#[derive(Debug)]
 pub struct XpmImage {
     inner: *mut Fl_XPM_Image,
     refcount: AtomicUsize,
 }
 
+crate::macros::image::impl_image_ext!(XpmImage, Fl_XPM_Image);
+
 impl XpmImage {
     /// Loads the image from a filesystem path, doesn't check for the validity of the data
     /// # Errors
@@ -444,12 +456,14 @@ impl XpmImage {
 }
 
 /// Creates a struct holding a XBM image
-#[derive(ImageExt, Debug)]
+#[derive(Debug)]
 pub struct XbmImage {
     inner: *mut Fl_XBM_Image,
     refcount: AtomicUsize,
 }
 
+crate::macros::image::impl_image_ext!(XbmImage, Fl_XBM_Image);
+
 impl XbmImage {
     /// Loads the image from a filesystem path, doesn't check for the validity of the data
     /// # Errors
@@ -484,12 +498,14 @@ impl XbmImage {
 }
 
 /// Creates a struct holding a PNM image
-#[derive(ImageExt, Debug)]
+#[derive(Debug)]
 pub struct PnmImage {
     inner: *mut Fl_PNM_Image,
     refcount: AtomicUsize,
 }
 
+crate::macros::image::impl_image_ext!(PnmImage, Fl_PNM_Image);
+
 impl PnmImage {
     /// Loads the image from a filesystem path, doesn't check for the validity of the data
     /// # Errors
@@ -524,34 +540,39 @@ impl PnmImage {
 }
 
 /// Creates a struct holding a tiled image
-#[derive(ImageExt, Debug)]
+#[derive(Debug)]
 pub struct TiledImage {
     inner: *mut Fl_Tiled_Image,
     refcount: AtomicUsize,
 }
 
+crate::macros::image::impl_image_ext!(TiledImage, Fl_Tiled_Image);
+
 impl TiledImage {
     /// Loads the image from a filesystem path, doesn't check for the validity of the data
-    pub fn new<Img: ImageExt>(img: Img, w: i32, h: i32) -> TiledImage {
+    pub fn new<Img: ImageExt>(mut img: Img, w: i32, h: i32) -> TiledImage {
         unsafe {
             assert!(!img.was_deleted());
+            img.increment_arc();
             let ptr = Fl_Tiled_Image_new(img.as_image_ptr(), w, h);
             assert!(!ptr.is_null());
             TiledImage {
                 inner: ptr,
-                refcount: AtomicUsize::new(1),
+                refcount: AtomicUsize::new(2),
             }
         }
     }
 }
 
 /// Creates a struct holding a pixmap image
-#[derive(ImageExt, Debug)]
+#[derive(Debug)]
 pub struct Pixmap {
     inner: *mut Fl_Pixmap,
     refcount: AtomicUsize,
 }
 
+crate::macros::image::impl_image_ext!(Pixmap, Fl_Pixmap);
+
 impl Pixmap {
     /// Creates a new Pixmap image
     /// # Errors
@@ -588,13 +609,25 @@ impl Pixmap {
     }
 }
 
+/// The scaling algorithm to use for RGB images
+#[repr(i32)]
+#[derive(Debug, Copy, Clone, PartialEq)]
+pub enum RgbScaling {
+    /// Default RGB image scaling algorithm
+    Nearest = 0,
+    /// More accurate, but slower RGB image scaling algorithm
+    Bilinear,
+}
+
 /// Creates a struct holding a raw RGB image
-#[derive(ImageExt, Debug)]
+#[derive(Debug)]
 pub struct RgbImage {
     pub(crate) inner: *mut Fl_RGB_Image,
     pub(crate) refcount: AtomicUsize,
 }
 
+crate::macros::image::impl_image_ext!(RgbImage, Fl_RGB_Image);
+
 impl RgbImage {
     /// Initializes a new raw `RgbImage`, copies the data and handles its lifetime.
     /// If you need to work with RGB data,
@@ -693,6 +726,11 @@ impl RgbImage {
         }
     }
 
+    /// Creates an RgbImage from a pixmap
+    pub fn from_pixmap(image: &Pixmap) -> RgbImage {
+        unsafe { RgbImage::from_image_ptr(Fl_RGB_Image_from_pixmap(image.inner as _) as _) }
+    }
+
     /// Deconstructs a raw `RgbImage` into parts
     /// # Safety
     /// Destructures the image into its raw elements
@@ -861,4 +899,14 @@ impl RgbImage {
             }
         }
     }
+
+    /// Sets the scaling algorithm
+    pub fn set_scaling_algorithm(algorithm: RgbScaling) {
+        unsafe { Fl_Image_set_scaling_algorithm(algorithm as i32) }
+    }
+
+    /// Gets the scaling algorithm
+    pub fn scaling_algorithm() -> RgbScaling {
+        unsafe { mem::transmute(Fl_Image_scaling_algorithm()) }
+    }
 }
