@@ -10,7 +10,7 @@
 
 Rust bindings for the FLTK Graphical User Interface library.
 
-The fltk crate is a crossplatform lightweight gui library which can be statically linked to produce small, self-contained (no dependencies) and fast gui applications.
+The fltk crate is a cross-platform lightweight gui library which can be statically linked to produce small, self-contained (no external dependencies) and fast gui applications.
 
 Why choose FLTK?
 - Lightweight. Small binary, around 1mb after stripping. [Small memory footprint](https://szibele.com/memory-footprint-of-gui-toolkits/).
@@ -18,14 +18,16 @@ Why choose FLTK?
 - Single executable. No DLLs to deploy.
 - Supports old architectures.
 - FLTK's permissive license which allows static linking for closed-source applications.
-- Themability (4 supported schemes: Base, GTK, Plastic and Gleam), and additional theming using [fltk-theme](https://crates.io/crates/fltk-theme).
+- Themeability (4 supported schemes: Base, GTK, Plastic and Gleam), and additional theming using [fltk-theme](https://crates.io/crates/fltk-theme).
 - Provides around 80 customizable widgets.
 - Has inbuilt image support.
 
-Tutorials:
-- [Video](https://github.com/fltk-rs/fltk-rs#tutorials)
-- [Written](https://github.com/fltk-rs/fltk-rs/wiki)
+Resources:
+- [Book](https://fltk-rs.github.io/fltk-book/)
+- [Documentation](https://docs.rs/fltk)
+- [Videos](https://github.com/fltk-rs/fltk-rs#tutorials)
 - [Discussions](https://github.com/fltk-rs/fltk-rs/discussions)
+- [Examples](https://github.com/fltk-rs/fltk-rs/tree/master/fltk/examples)
 - [Demos](https://github.com/fltk-rs/demos)
 - [7guis-fltk-rs](https://github.com/tdryer/7guis-fltk-rs)
 - [FLTK-RS-Examples](https://github.com/wyhinton/FLTK-RS-Examples)
@@ -34,7 +36,7 @@ Tutorials:
 Here is a [list](https://en.wikipedia.org/wiki/FLTK#Use) of software using FLTK. For software using fltk-rs, check [here](https://github.com/fltk-rs/fltk-rs/issues/418).
 
 - [Link](https://github.com/fltk/fltk) to the official FLTK repository.
-- [Link](https://www.fltk.org/doc-1.3/index.html) to the official documentation.
+- [Link](https://www.fltk.org/doc-1.4/index.html) to the official documentation.
 
 ## Usage
 
@@ -190,6 +192,8 @@ For custom event handling, the handle() method can be used:
 Handled or ignored events using the handle method should return true, unhandled events should return false.
 More examples are available in the examples directory.
 
+For an alternative event handling mechanism using `on_<event>` methods, check the [fltk-evented crate](https://crates.io/crates/fltk-evented).
+
 ### Theming
 
 FLTK offers 4 application schemes:
@@ -213,40 +217,43 @@ such as `set_color()`, `set_label_font()`, `set_frame()` etc:
     some_button.set_frame(FrameType::RoundUpBox);
     some_button.set_font(Font::TimesItalic);
 ```
-For default application colors, fltk-rs provides `app::background()`, `app::background2()` and `app::foreground()`. You can also specify the default application selection/inactive colors, font, label size, frame type, scrollbar size, menu linespacing. Additionally the [fltk-theme](https://crates.io/crates/fltk-theme) crate offers some other predefined color maps (dark theme, tan etc) and widget themes which can be loaded into your application.
+For default application colors, fltk-rs provides `app::background()`, `app::background2()` and `app::foreground()`. You can also specify the default application selection/inactive colors, font, label size, frame type, scrollbar size, menu line-spacing. Additionally the [fltk-theme](https://crates.io/crates/fltk-theme) crate offers some other predefined color maps (dark theme, tan etc) and widget themes which can be loaded into your application.
 
 ## Dependencies
 
-Rust (version > 1.45), CMake (version > 3.0), Git and a C++11 compiler need to be installed and in your PATH for a crossplatform build from source. [Ninja](https://github.com/ninja-build/ninja) is recommended, but not required, and will be used if found. This crate also offers a bundled form of fltk on selected x86_64 platforms (Windows (msvc and gnu), MacOS, Linux), this can be enabled using the fltk-bundled feature flag as mentioned in the usage section (this requires curl and tar to download and unpack the bundled libraries).
+Rust (version > 1.45), CMake (version > 3.11), Git and a C++11 compiler need to be installed and in your PATH for a cross-platform build from source. [Ninja](https://github.com/ninja-build/ninja) is recommended, but not required. This crate also offers a bundled form of fltk on selected x86_64 platforms (Windows (msvc and gnu), MacOS, Linux), this can be enabled using the fltk-bundled feature flag as mentioned in the usage section (this requires curl and tar to download and unpack the bundled libraries).
 
-- Windows: No external dependencies.
-- MacOS: No external dependencies.
+- Windows: 
+    - MSVC: Windows SDK
+    - Gnu: No dependencies
+- MacOS: No dependencies.
 - Linux/BSD: X11 and OpenGL development headers need to be installed for development. The libraries themselves are normally available on linux distros with a graphical user interface.
 
 For Debian-based GUI distributions, that means running:
 ```ignore
-$ sudo apt-get install libx11-dev libxext-dev libxft-dev libxinerama-dev libxcursor-dev libxrender-dev libxfixes-dev libpango1.0-dev libpng-dev libgl1-mesa-dev libglu1-mesa-dev
+$ sudo apt-get install libx11-dev libxext-dev libxft-dev libxinerama-dev libxcursor-dev libxrender-dev libxfixes-dev libpango1.0-dev libgl1-mesa-dev libglu1-mesa-dev
 ```
 For RHEL-based GUI distributions, that means running:
 ```ignore
-$ sudo yum groupinstall "X Software Development" && yum install pango-devel libXinerama-devel libpng-devel
+$ sudo yum groupinstall "X Software Development" && yum install pango-devel libXinerama-devel
 ```
 For Arch-based GUI distributions, that means running:
 ```ignore
-$ sudo pacman -S libx11 libxext libxft libxinerama libxcursor libxrender libxfixes libpng pango cairo libgl mesa --needed
+$ sudo pacman -S libx11 libxext libxft libxinerama libxcursor libxrender libxfixes pango cairo libgl mesa --needed
 ```
 For Alpine linux:
 ```ignore
-$ apk add pango-dev fontconfig-dev libxinerama-dev libxfixes-dev libxcursor-dev libpng-dev mesa-gl
+$ apk add pango-dev fontconfig-dev libxinerama-dev libxfixes-dev libxcursor-dev mesa-gl
 ```
 For NixOS (Linux distribution) this `nix-shell` environment can be used:
 ```ignore
-$ nix-shell --packages rustc cmake git gcc xorg.libXext xorg.libXft xorg.libXinerama xorg.libXcursor xorg.libXrender xorg.libXfixes libpng libcerf pango cairo libGL mesa pkg-config
+$ nix-shell --packages rustc cmake git gcc xorg.libXext xorg.libXft xorg.libXinerama xorg.libXcursor xorg.libXrender xorg.libXfixes libcerf pango cairo libGL mesa pkg-config
 ```
 
 ## Features
 
 The following are the features offered by the crate:
+- use-ninja: Uses the ninja build system if available for a faster build, especially on Windows.
 - fltk-bundled: Support for bundled versions of cfltk and fltk on selected platforms (requires curl and tar)
 - no-pango: Build without pango support on Linux/BSD.
 - enable-glwindow: Support for drawing using OpenGL functions.
@@ -262,59 +269,76 @@ please check the [FAQ](https://github.com/fltk-rs/fltk-rs/blob/master/FAQ.md) pa
 #![allow(non_upper_case_globals)]
 #![allow(clippy::needless_doctest_main)]
 #![warn(missing_docs)]
-#![warn(broken_intra_doc_links)]
 
 /// Application related methods and functions
 pub mod app;
+
 /// Browser widgets
 pub mod browser;
+
 /// Button widgets
 pub mod button;
+
 /// Dialog widgets
 pub mod dialog;
+
 /// Drawing primitives
 pub mod draw;
+
 /// Fltk defined enums: Color, Font, `CallbackTrigger` etc
 pub mod enums;
+
+pub mod examples;
+
 /// Basic fltk box/frame widget
 pub mod frame;
+
 /// Group widgets
 pub mod group;
+
 /// Image types supported by fltk
 pub mod image;
+
 /// Input widgets
 pub mod input;
+
+/// mod macros;
+pub mod macros;
+
 /// Menu widgets
 pub mod menu;
+
 /// Miscellaneous widgets not fitting a certain group
 pub mod misc;
+
 /// Output widgets
 pub mod output;
+
 /// All fltk widget traits and flt error types
 pub mod prelude;
+
 /// Widget surface to image functions
 pub mod surface;
+
 /// Table widgets
 pub mod table;
+
 /// Text display widgets
 pub mod text;
+
 /// Tree widgets
 pub mod tree;
+
 /// General utility functions
 pub mod utils;
+
 /// Valuator widgets
 pub mod valuator;
-/// Basic empty widget
+
 pub mod widget;
-/// Window widgets
+
 pub mod window;
 
 /// Printing related functions
 #[cfg(not(target_os = "android"))]
 pub mod printer;
-
-#[macro_use]
-extern crate fltk_derive;
-
-#[macro_use]
-extern crate bitflags;
