@@ -1,7 +1,4 @@
-use crate::enums::{
-    Align, CallbackTrigger, Color, Damage, Event, Font, FrameType, LabelType, Shortcut,
-};
-use crate::image::Image;
+use crate::enums::{Color, Font, LabelType};
 use crate::prelude::*;
 use crate::utils::FlString;
 use fltk_sys::menu::*;
@@ -12,24 +9,32 @@ use std::{
 };
 
 /// Creates a menu bar
-#[derive(WidgetBase, WidgetExt, MenuExt, Debug)]
+#[derive(Debug)]
 pub struct MenuBar {
     inner: *mut Fl_Menu_Bar,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(MenuBar, Fl_Menu_Bar);
+crate::macros::widget::impl_widget_base!(MenuBar, Fl_Menu_Bar);
+crate::macros::menu::impl_menu_ext!(MenuBar, Fl_Menu_Bar);
+
 /// Creates a menu button
-#[derive(WidgetBase, WidgetExt, MenuExt, Debug)]
+#[derive(Debug)]
 pub struct MenuButton {
     inner: *mut Fl_Menu_Button,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(MenuButton, Fl_Menu_Button);
+crate::macros::widget::impl_widget_base!(MenuButton, Fl_Menu_Button);
+crate::macros::menu::impl_menu_ext!(MenuButton, Fl_Menu_Button);
+
 /// Defines the menu button types, which can be changed dynamically using the `set_type()`.
 #[repr(i32)]
-#[derive(WidgetType, Debug, Copy, Clone, PartialEq)]
+#[derive(Debug, Copy, Clone, PartialEq)]
 pub enum MenuButtonType {
     /// pops up with the mouse 1st button.
     Popup1 = 1,
@@ -47,6 +52,8 @@ pub enum MenuButtonType {
     Popup123,
 }
 
+crate::macros::widget::impl_widget_type!(MenuButtonType);
+
 impl MenuButton {
     /// Act exactly as though the user clicked the button or typed the shortcut key
     pub fn popup(&self) -> Option<MenuItem> {
@@ -67,21 +74,59 @@ impl MenuButton {
 }
 
 /// Creates a menu choice
-#[derive(WidgetBase, WidgetExt, MenuExt, Debug)]
+///
+/// The [`set_frame`](crate::prelude::WidgetExt::set_frame) method styles the
+/// dropdown menu. `Choice` does not expose it's uderlying widget (a
+/// [`DownBox`](crate::enums::FrameType::DownBox)). It can only be changed
+/// via the app scheme or by globally changin the draw function of
+/// [`DownBox`](crate::enums::FrameType::DownBox):
+///
+/// ```rust,no_run
+///use fltk::{enums::*, prelude::*, *};
+///
+///fn my_down_box(x: i32, y: i32, w: i32, h: i32, col: Color) {
+///    draw::draw_rect_fill(x, y, w, h, Color::Red);
+///    draw::draw_rect_fill(x + 1, y + 1, w - 2, h - 2, Color::BackGround2); // change values to change thickness
+///}
+///
+///fn main() {
+///     let a = app::App::default();
+///     app::set_frame_type_cb(FrameType::DownBox, my_down_box, 0, 0, 0, 0);
+///     let mut win = window::Window::new(100, 100, 400, 300, None);
+///     win.set_color(Color::from_rgb(211, 211, 211));
+///     let mut inp = input::Input::new(50, 10, 100, 30, None); // would work for any widget which has a DownBox frame type
+///     let mut choice = menu::Choice::new(50, 100, 100, 30, None);
+///     choice.add_choice("Choice 1| Choice 2| choice 3");
+///     win.end();
+///     win.show();
+///     a.run().unwrap();
+///}
+///```
+///
+/// For more extensive options see the `custom_choice` example.
+#[derive(Debug)]
 pub struct Choice {
     inner: *mut Fl_Choice,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(Choice, Fl_Choice);
+crate::macros::widget::impl_widget_base!(Choice, Fl_Choice);
+crate::macros::menu::impl_menu_ext!(Choice, Fl_Choice);
+
 /// Creates a macOS system menu bar on macOS and a normal menu bar on other systems
-#[derive(WidgetBase, WidgetExt, MenuExt, Debug)]
+#[derive(Debug)]
 pub struct SysMenuBar {
     inner: *mut Fl_Sys_Menu_Bar,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(SysMenuBar, Fl_Sys_Menu_Bar);
+crate::macros::widget::impl_widget_base!(SysMenuBar, Fl_Sys_Menu_Bar);
+crate::macros::menu::impl_menu_ext!(SysMenuBar, Fl_Sys_Menu_Bar);
+
 /// Creates a menu item
 #[derive(Debug, Clone)]
 pub struct MenuItem {
@@ -239,7 +284,7 @@ impl MenuItem {
         unsafe { Fl_Menu_Item_set(self.inner) }
     }
 
-    /// Clears the menu item
+    /// Turns the check or radio item "off" for the menu item
     pub fn clear(&mut self) {
         assert!(!self.was_deleted());
         unsafe { Fl_Menu_Item_clear(self.inner) }
@@ -252,7 +297,7 @@ impl MenuItem {
     }
 
     /// Returns whether the menu item is active
-    pub fn active(&mut self) -> bool {
+    pub fn active(&self) -> bool {
         assert!(!self.was_deleted());
         unsafe { Fl_Menu_Item_active(self.inner) != 0 }
     }
@@ -482,6 +527,101 @@ impl MenuItem {
             }
         }
     }
+
+    /// Add a menu item
+    pub fn add<F: FnMut(&mut Choice) + 'static>(
+        &mut self,
+        name: &str,
+        shortcut: crate::enums::Shortcut,
+        flag: MenuFlag,
+        cb: F,
+    ) -> i32 {
+        assert!(!self.was_deleted());
+        let temp = CString::safe_new(name);
+        unsafe {
+            unsafe extern "C" fn shim(wid: *mut Fl_Widget, data: *mut std::os::raw::c_void) {
+                let mut wid = crate::widget::Widget::from_widget_ptr(wid as *mut _);
+                let a: *mut Box<dyn FnMut(&mut crate::widget::Widget)> =
+                    data as *mut Box<dyn FnMut(&mut crate::widget::Widget)>;
+                let f: &mut (dyn FnMut(&mut crate::widget::Widget)) = &mut **a;
+                let _ =
+                    std::panic::catch_unwind(std::panic::AssertUnwindSafe(|| f(&mut wid)));
+            }
+            let a: *mut Box<dyn FnMut(&mut Choice)> = Box::into_raw(Box::new(Box::new(cb)));
+            let data: *mut std::os::raw::c_void = a as *mut std::os::raw::c_void;
+            let callback: Fl_Callback = Some(shim);
+            Fl_Menu_Item_add(
+                self.inner,
+                temp.as_ptr(),
+                shortcut.bits() as i32,
+                callback,
+                data,
+                flag as i32,
+            )
+        }
+    }
+
+    /// Insert a menu item
+    pub fn insert<F: FnMut(&mut Choice) + 'static>(
+        &mut self,
+        idx: i32,
+        name: &str,
+        shortcut: crate::enums::Shortcut,
+        flag: MenuFlag,
+        cb: F,
+    ) -> i32 {
+        assert!(!self.was_deleted());
+        let temp = CString::safe_new(name);
+        unsafe {
+            unsafe extern "C" fn shim(wid: *mut Fl_Widget, data: *mut std::os::raw::c_void) {
+                let mut wid = crate::widget::Widget::from_widget_ptr(wid as *mut _);
+                let a: *mut Box<dyn FnMut(&mut crate::widget::Widget)> =
+                    data as *mut Box<dyn FnMut(&mut crate::widget::Widget)>;
+                let f: &mut (dyn FnMut(&mut crate::widget::Widget)) = &mut **a;
+                let _ =
+                    std::panic::catch_unwind(std::panic::AssertUnwindSafe(|| f(&mut wid)));
+            }
+            let a: *mut Box<dyn FnMut(&mut Choice)> = Box::into_raw(Box::new(Box::new(cb)));
+            let data: *mut std::os::raw::c_void = a as *mut std::os::raw::c_void;
+            let callback: Fl_Callback = Some(shim);
+            Fl_Menu_Item_insert(
+                self.inner,
+                idx as i32,
+                temp.as_ptr(),
+                shortcut.bits() as i32,
+                callback,
+                data,
+                flag as i32,
+            )
+        }
+    }
+
+    /// Add a menu item along with an emit (sender and message).
+    pub fn add_emit<T: 'static + Clone + Send + Sync>(
+        &mut self,
+        label: &str,
+        shortcut: crate::enums::Shortcut,
+        flag: MenuFlag,
+        sender: crate::app::Sender<T>,
+        msg: T,
+    ) -> i32 {
+        self.add(label, shortcut, flag, move |_| sender.send(msg.clone()))
+    }
+
+    /// Insert a menu item along with an emit (sender and message).
+    pub fn insert_emit<T: 'static + Clone + Send + Sync>(
+        &mut self,
+        idx: i32,
+        label: &str,
+        shortcut: crate::enums::Shortcut,
+        flag: MenuFlag,
+        sender: crate::app::Sender<T>,
+        msg: T,
+    ) -> i32 {
+        self.insert(idx, label, shortcut, flag, move |_| {
+            sender.send(msg.clone())
+        })
+    }
 }
 
 /// Delete a menu item
@@ -517,3 +657,18 @@ impl IntoIterator for MenuItem {
         v.into_iter()
     }
 }
+
+/// Set a callback for the "About" item of the system application menu on macOS.
+pub fn mac_set_about<F: FnMut() + 'static>(cb: F) {
+    unsafe {
+        unsafe extern "C" fn shim(_wid: *mut fltk_sys::menu::Fl_Widget, data: *mut raw::c_void) {
+            let a: *mut Box<dyn FnMut()> = data as *mut Box<dyn FnMut()>;
+            let f: &mut (dyn FnMut()) = &mut **a;
+            let _ = std::panic::catch_unwind(std::panic::AssertUnwindSafe(f));
+        }
+        let a: *mut Box<dyn FnMut()> = Box::into_raw(Box::new(Box::new(cb)));
+        let data: *mut raw::c_void = a as *mut std::ffi::c_void;
+        let callback: fltk_sys::menu::Fl_Callback = Some(shim);
+        Fl_mac_set_about(callback, data, 0);
+    }
+}
