@@ -1,5 +1,4 @@
-use crate::enums::{Align, CallbackTrigger, Color, Damage, Event, Font, FrameType, LabelType};
-use crate::image::Image;
+use crate::enums::{Color, Font, FrameType};
 use crate::prelude::*;
 use crate::utils::FlString;
 use crate::widget::Widget;
@@ -13,7 +12,7 @@ use std::{
 
 /// Defines the chart types supported by fltk
 #[repr(i32)]
-#[derive(WidgetType, Debug, Copy, Clone, PartialEq)]
+#[derive(Debug, Copy, Clone, PartialEq)]
 pub enum ChartType {
     /// Bar chart
     Bar = 0,
@@ -31,9 +30,11 @@ pub enum ChartType {
     SpecialPie = 6,
 }
 
+crate::macros::widget::impl_widget_type!(ChartType);
+
 /// Defines the clock types supported by fltk
 #[repr(i32)]
-#[derive(WidgetType, Debug, Copy, Clone, PartialEq)]
+#[derive(Debug, Copy, Clone, PartialEq)]
 pub enum ClockType {
     /// Square clock
     Square = 0,
@@ -41,14 +42,19 @@ pub enum ClockType {
     Round = 1,
 }
 
+crate::macros::widget::impl_widget_type!(ClockType);
+
 /// Creates a spinner widget
-#[derive(WidgetBase, WidgetExt, Debug)]
+#[derive(Debug)]
 pub struct Spinner {
     inner: *mut Fl_Spinner,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(Spinner, Fl_Spinner);
+crate::macros::widget::impl_widget_base!(Spinner, Fl_Spinner);
+
 impl Spinner {
     /// Returns the minimum value of the spinner widget
     pub fn minimum(&self) -> f64 {
@@ -56,7 +62,7 @@ impl Spinner {
         unsafe { Fl_Spinner_minimum(self.inner) }
     }
 
-    /// Sets the minimu value of the spinner widget
+    /// Sets the minimum value of the spinner widget
     pub fn set_minimum(&mut self, a: f64) {
         assert!(!self.was_deleted());
         unsafe { Fl_Spinner_set_minimum(self.inner, a) }
@@ -153,24 +159,44 @@ impl Spinner {
             Fl_Spinner_set_value(self.inner, arg2);
         }
     }
+
+    /// Returns whether wrap is set
+    pub fn wrap(&self) -> bool {
+        assert!(!self.was_deleted());
+        unsafe { Fl_Spinner_wrap(self.inner) != 0 }
+    }
+
+    /// Sets wrap for the spinner
+    pub fn set_wrap(&mut self, flag: bool) {
+        unsafe {
+            assert!(!self.was_deleted());
+            Fl_Spinner_set_wrap(self.inner, flag as _);
+        }
+    }
 }
 
 /// Creates a clock widget
-#[derive(WidgetBase, WidgetExt, Debug)]
+#[derive(Debug)]
 pub struct Clock {
     inner: *mut Fl_Clock,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(Clock, Fl_Clock);
+crate::macros::widget::impl_widget_base!(Clock, Fl_Clock);
+
 /// Creates a chart widget
-#[derive(WidgetBase, WidgetExt, Debug)]
+#[derive(Debug)]
 pub struct Chart {
     inner: *mut Fl_Chart,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(Chart, Fl_Chart);
+crate::macros::widget::impl_widget_base!(Chart, Fl_Chart);
+
 impl Chart {
     /// Clears the chart
     pub fn clear(&mut self) {
@@ -259,7 +285,7 @@ impl Chart {
         unsafe { Fl_Chart_set_text_color(self.inner, color.bits() as u32) }
     }
 
-    /// Returns wheter the chart is autosizable
+    /// Returns whether the chart is autosizable
     pub fn is_autosize(&self) -> bool {
         unsafe {
             assert!(!self.was_deleted());
@@ -275,13 +301,16 @@ impl Chart {
 }
 
 /// Creates a progress bar
-#[derive(WidgetBase, WidgetExt, Debug)]
+#[derive(Debug)]
 pub struct Progress {
     inner: *mut Fl_Progress,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(Progress, Fl_Progress);
+crate::macros::widget::impl_widget_base!(Progress, Fl_Progress);
+
 impl Progress {
     /// Returns the minimum value of the progress bar
     pub fn minimum(&self) -> f64 {
@@ -289,7 +318,7 @@ impl Progress {
         unsafe { Fl_Progress_minimum(self.inner) }
     }
 
-    /// Sets the minimu value of the progress bar
+    /// Sets the minimum value of the progress bar
     pub fn set_minimum(&mut self, a: f64) {
         assert!(!self.was_deleted());
         unsafe { Fl_Progress_set_minimum(self.inner, a) }
@@ -389,7 +418,7 @@ impl Tooltip {
     }
 
     /// Returns the current widget under the tooltip
-    pub fn current_widget() -> Widget {
+    pub fn current_widget() -> impl WidgetExt {
         unsafe {
             let widget_ptr = Fl_Tooltip_current_widget();
             assert!(!widget_ptr.is_null());
@@ -484,13 +513,16 @@ impl Tooltip {
 }
 
 /// Creates an `InputChoice` widget
-#[derive(WidgetBase, WidgetExt, Debug)]
+#[derive(Debug)]
 pub struct InputChoice {
     inner: *mut Fl_Input_Choice,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(InputChoice, Fl_Input_Choice);
+crate::macros::widget::impl_widget_base!(InputChoice, Fl_Input_Choice);
+
 impl InputChoice {
     /// Set the `down_box` of the widget
     pub fn set_down_frame(&mut self, f: FrameType) {
@@ -606,15 +638,19 @@ impl InputChoice {
     use fltk::{prelude::*, *};
     let mut h = misc::HelpView::new(10, 10, 380, 280, "");
     h.set_value("Hello <b><font color=red>again</font></b>");
+
     ```
 */
-#[derive(WidgetBase, WidgetExt, Debug)]
+#[derive(Debug)]
 pub struct HelpView {
     inner: *mut Fl_Help_View,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(HelpView, Fl_Help_View);
+crate::macros::widget::impl_widget_base!(HelpView, Fl_Help_View);
+
 impl HelpView {
     /// Return the directory
     pub fn directory(&self) -> std::path::PathBuf {
