@@ -12,14 +12,16 @@ use std::{fmt, io};
 pub enum FltkError {
     /// i/o error
     IoError(io::Error),
-    /// Ut8 conversion error
+    /// Utf-8 conversion error
     Utf8Error(FromUtf8Error),
     /// Null string conversion error
     NullError(std::ffi::NulError),
     /// Internal fltk error
     Internal(FltkErrorKind),
-    /// Error using an errorneous env variable
+    /// Error using an erroneous env variable
     EnvVarError(std::env::VarError),
+    /// Parsing error
+    ParseIntError(std::num::ParseIntError),
     /// Unknown error
     Unknown(String),
 }
@@ -29,6 +31,7 @@ unsafe impl Sync for FltkError {}
 
 /// Error kinds enum for `FltkError`
 #[derive(Clone, Copy, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
+#[non_exhaustive]
 pub enum FltkErrorKind {
     /// Failed to run the application
     FailedToRun,
@@ -40,12 +43,14 @@ pub enum FltkErrorKind {
     FailedOperation,
     /// System resource (file, image) not found
     ResourceNotFound,
-    /// Image format error when opening an image of an unsopported format
+    /// Image format error when opening an image of an unsupported format
     ImageFormatError,
     /// Error filling table
     TableError,
     /// Error due to printing
     PrintError,
+    /// Invalid color
+    InvalidColor,
 }
 
 impl std::error::Error for FltkError {
@@ -63,9 +68,14 @@ impl fmt::Display for FltkError {
         match *self {
             FltkError::IoError(ref err) => err.fmt(f),
             FltkError::NullError(ref err) => err.fmt(f),
-            FltkError::Internal(ref err) => write!(f, "An internal error occured {:?}", err),
-            FltkError::EnvVarError(ref err) => write!(f, "An env var error occured {:?}", err),
-            FltkError::Utf8Error(ref err) => write!(f, "A UTF8 conversion error occured {:?}", err),
+            FltkError::Internal(ref err) => write!(f, "An internal error occurred {:?}", err),
+            FltkError::EnvVarError(ref err) => write!(f, "An env var error occurred {:?}", err),
+            FltkError::Utf8Error(ref err) => {
+                write!(f, "A UTF8 conversion error occurred {:?}", err)
+            }
+            FltkError::ParseIntError(ref err) => {
+                write!(f, "An int parsing error occurred {:?}", err)
+            }
             FltkError::Unknown(ref err) => write!(f, "An unknown error occurred {:?}", err),
         }
     }
@@ -95,7 +105,13 @@ impl From<std::string::FromUtf8Error> for FltkError {
     }
 }
 
-/// A trait defined for all enums passable to the `WidgetExt::set_type()` method
+impl From<std::num::ParseIntError> for FltkError {
+    fn from(err: std::num::ParseIntError) -> FltkError {
+        FltkError::ParseIntError(err)
+    }
+}
+
+/// A trait defined for all enums passable to the [`WidgetExt::set_type()`](`crate::prelude::WidgetExt::set_type`) method
 pub trait WidgetType {
     /// Get the integral representation of the widget type
     fn to_i32(self) -> i32;
@@ -104,6 +120,14 @@ pub trait WidgetType {
 }
 
 /// Defines the methods implemented by all widgets
+///
+/// For multithreaded usage, see the [`widget` module documentation's note](crate::widget)
+/// # Safety
+/// fltk-rs traits depend on some FLTK internal code
+/// # Warning
+/// fltk-rs traits are non-exhaustive,
+/// to avoid future breakage if you try to implement them manually,
+/// use the Deref and DerefMut pattern or the `widget_extends!` macro
 pub unsafe trait WidgetExt {
     /// Initialize to a position x, y
     fn with_pos(self, x: i32, y: i32) -> Self
@@ -143,6 +167,14 @@ pub unsafe trait WidgetExt {
         Self: Sized;
     /// Initialize center of another widget
     fn center_of<W: WidgetExt>(self, w: &W) -> Self
+    where
+        Self: Sized;
+    /// Initialize center of another widget on the x axis
+    fn center_x<W: WidgetExt>(self, w: &W) -> Self
+    where
+        Self: Sized;
+    /// Initialize center of another widget on the y axis
+    fn center_y<W: WidgetExt>(self, w: &W) -> Self
     where
         Self: Sized;
     /// Initialize center of parent
@@ -215,7 +247,7 @@ pub unsafe trait WidgetExt {
     fn image(&self) -> Option<Box<dyn ImageExt>>
     where
         Self: Sized;
-    /// Sets the deactived image of the widget
+    /// Sets the deactivated image of the widget
     fn set_deimage<I: ImageExt>(&mut self, image: Option<I>)
     where
         Self: Sized;
@@ -285,7 +317,7 @@ pub unsafe trait WidgetExt {
     /// Returns the parent of the widget
     fn parent(&self) -> Option<crate::group::Group>;
     /// Gets the selection color of the widget
-    fn selection_color(&mut self) -> Color;
+    fn selection_color(&self) -> Color;
     /// Sets the selection color of the widget
     fn set_selection_color(&mut self, color: Color);
     /// Runs the already registered callback
@@ -307,9 +339,9 @@ pub unsafe trait WidgetExt {
     /// Set the visible focus using a flag
     fn visible_focus(&mut self, v: bool);
     /// Return whether the widget has visible focus
-    fn has_visible_focus(&mut self) -> bool;
+    fn has_visible_focus(&self) -> bool;
     /// Return whether the widget has focus
-    fn has_focus(&mut self) -> bool;
+    fn has_focus(&self) -> bool;
     /// Check if a widget was deleted
     fn was_deleted(&self) -> bool;
     /// Return whether the widget was damaged
@@ -330,10 +362,21 @@ pub unsafe trait WidgetExt {
     fn as_window(&self) -> Option<Box<dyn WindowExt>>;
     /// Return the widget as a group widget if it's a group widget
     fn as_group(&self) -> Option<crate::group::Group>;
+    #[doc(hidden)]
     /// INTERNAL: Retakes ownership of the user callback data
     /// # Safety
     /// Can return multiple mutable references to the `user_data`
     unsafe fn user_data(&self) -> Option<Box<dyn FnMut()>>;
+    #[doc(hidden)]
+    /// INTERNAL: Get the raw user data of the widget
+    /// # Safety
+    /// Can return multiple mutable references to the `user_data`
+    unsafe fn raw_user_data(&self) -> *mut std::os::raw::c_void;
+    #[doc(hidden)]
+    /// INTERNAL: Set the raw user data of the widget
+    /// # Safety
+    /// Can return multiple mutable references to the `user_data`
+    unsafe fn set_raw_user_data(&mut self, data: *mut std::os::raw::c_void);
     /// Upcast a `WidgetExt` to a Widget
     /// # Safety
     /// Allows for potentially unsafe casts between incompatible widget types
@@ -355,7 +398,7 @@ pub unsafe trait WidgetExt {
     fn active_r(&self) -> bool;
     #[doc(hidden)]
     /**
-        Return the default callback function, this allows storing then running within the overriden callback.
+        Return the default callback function, this allows storing then running within the overridden callback.
         Works only for FLTK types (with no callback defined in the Rust side)
         ```rust,no_run
             use fltk::{prelude::*, *};
@@ -383,9 +426,21 @@ pub unsafe trait WidgetExt {
     fn callback(&self) -> Option<Box<dyn FnMut()>>;
     /// Does a simple resize ignoring class-specific resize functionality
     fn widget_resize(&mut self, x: i32, y: i32, w: i32, h: i32);
+    /// Handle a specific event
+    fn handle_event(&mut self, event: Event);
+    /// Check whether a widget is derived
+    fn is_derived(&self) -> bool {
+        unimplemented!();
+    }
 }
 
 /// Defines the extended methods implemented by all widgets
+/// # Safety
+/// fltk-rs traits depend on some FLTK internal code
+/// # Warning
+/// fltk-rs traits are non-exhaustive,
+/// to avoid future breakage if you try to implement them manually,
+/// use the Deref and DerefMut pattern or the `widget_extends!` macro
 pub unsafe trait WidgetBase: WidgetExt {
     /// Creates a new widget, takes an x, y coordinates, as well as a width and height, plus a title
     /// # Arguments
@@ -427,20 +482,35 @@ pub unsafe trait WidgetBase: WidgetExt {
     /// takes the widget as a closure argument.
     /// macOS requires that `WidgetBase::draw` actually calls drawing functions
     fn draw<F: FnMut(&mut Self) + 'static>(&mut self, cb: F);
+    #[doc(hidden)]
     /// INTERNAL: Retrieve the draw data
     /// # Safety
     /// Can return multiple mutable references to the `draw_data`
-    unsafe fn draw_data(&mut self) -> Option<Box<dyn FnMut()>>;
+    unsafe fn draw_data(&self) -> Option<Box<dyn FnMut()>>;
+    #[doc(hidden)]
     /// INTERNAL: Retrieve the handle data
     /// # Safety
     /// Can return multiple mutable references to the `handle_data`
-    unsafe fn handle_data(&mut self) -> Option<Box<dyn FnMut(Event) -> bool>>;
+    unsafe fn handle_data(&self) -> Option<Box<dyn FnMut(Event) -> bool>>;
     /// Perform a callback on resize.
     /// Avoid resizing the parent or the same widget to avoid infinite recursion
     fn resize_callback<F: FnMut(&mut Self, i32, i32, i32, i32) + 'static>(&mut self, cb: F);
+    /// Makes the widget derived
+    /// # Safety
+    /// Calling this on a non-derived widget can cause undefined behavior
+    unsafe fn assume_derived(&mut self) {
+        unimplemented!();
+    }
 }
 
-/// Defines the methods implemented by all button widgets
+/// Defines the methods implemented by all button widgets.
+/// More details can be found in the [wiki](https://github.com/fltk-rs/fltk-rs/wiki/buttons).
+/// # Safety
+/// fltk-rs traits depend on some FLTK internal code
+/// # Warning
+/// fltk-rs traits are non-exhaustive,
+/// to avoid future breakage if you try to implement them manually,
+/// use the Deref and DerefMut pattern or the `widget_extends!` macro
 pub unsafe trait ButtonExt: WidgetExt {
     /// Gets the shortcut associated with a button
     fn shortcut(&self) -> Shortcut;
@@ -467,7 +537,46 @@ pub unsafe trait ButtonExt: WidgetExt {
     fn down_frame(&self) -> FrameType;
 }
 
-/// Defines the methods implemented by all group widgets
+/// Defines the methods implemented by all group widgets.
+/// These widgets include Window types and others found in the group module: Group, Scroll, Pack, Tile, Flex ...etc.
+/// Widgets implementing the GroupExt trait, are characterized by having to call `::end()` method to basically close them.
+/// More details can be found in the [wiki](https://github.com/fltk-rs/fltk-rs/wiki/group_widgets).
+/// ```rust
+/// use fltk::{app, button::Button, window::Window, prelude::GroupExt};
+/// let a = app::App::default();
+/// let win = Window::default();
+/// let btn = Button::default();
+/// // Instantiate other widgets
+/// win.end();
+/// ```
+/// In the above example, the button `btn` will be parented by the window.
+/// After `end`ing such GroupExt widgets, any other widgets instantiated after the `end` call, will be instantiated outside.
+/// These can still be added using the `::add(&other_widget)` method (or using `::insert`):
+/// ```rust
+/// use fltk::{app, button::Button, window::Window, prelude::GroupExt};
+/// let a = app::App::default();
+/// let mut win = Window::default();
+/// win.end();
+/// let btn = Button::default();
+/// win.add(&btn);
+/// ```
+/// Another option is to reopen the widget:
+/// ```rust
+/// use fltk::{app, button::Button, window::Window, prelude::GroupExt};
+/// let a = app::App::default();
+/// let win = Window::default();
+/// win.end();
+/// win.begin();
+/// let btn = Button::default();
+/// // other widgets
+/// win.end();
+/// ```
+/// # Safety
+/// fltk-rs traits depend on some FLTK internal code
+/// # Warning
+/// fltk-rs traits are non-exhaustive,
+/// to avoid future breakage if you try to implement them manually,
+/// use the Deref and DerefMut pattern or the `widget_extends!` macro
 pub unsafe trait GroupExt: WidgetExt {
     /// Begins a group, used for widgets implementing the group trait
     fn begin(&self);
@@ -515,20 +624,20 @@ pub unsafe trait GroupExt: WidgetExt {
     /// Clips children outside the group boundaries
     fn set_clip_children(&mut self, flag: bool);
     /// Get whether `clip_children` is set
-    fn clip_children(&mut self) -> bool;
-    /// Draw a child widget, the call should be in a `WidgetBase::draw` method
+    fn clip_children(&self) -> bool;
+    /// Draw a child widget, the call should be in a [`WidgetBase::draw`](`crate::prelude::WidgetBase::draw`) method
     fn draw_child<W: WidgetExt>(&self, w: &mut W)
     where
         Self: Sized;
-    /// Update a child widget, the call should be in a `WidgetBase::draw` method
+    /// Update a child widget, the call should be in a [`WidgetBase::draw`](`crate::prelude::WidgetBase::draw`) method
     fn update_child<W: WidgetExt>(&self, w: &mut W)
     where
         Self: Sized;
-    /// Draw the outside label, the call should be in a `WidgetBase::draw` method
+    /// Draw the outside label, the call should be in a [`WidgetBase::draw`](`crate::prelude::WidgetBase::draw`) method
     fn draw_outside_label<W: WidgetExt>(&self, w: &mut W)
     where
         Self: Sized;
-    /// Draw children, the call should be in a `WidgetBase::draw` method
+    /// Draw children, the call should be in a [`WidgetBase::draw`](`crate::prelude::WidgetBase::draw`) method
     fn draw_children(&mut self);
     /// Resets the internal array of widget sizes and positions
     fn init_sizes(&mut self);
@@ -537,12 +646,20 @@ pub unsafe trait GroupExt: WidgetExt {
     /// Converts a widget implementing GroupExt into a Group widget
     /// # Safety
     /// If the widget wasn't created by fltk-rs,
-    /// vtable differences mean certain methods can't be overriden (e.g. handle & draw)
+    /// vtable differences mean certain methods can't be overridden (e.g. handle & draw)
     #[allow(clippy::wrong_self_convention)]
     unsafe fn into_group(&self) -> crate::group::Group;
 }
 
-/// Defines the methods implemented by all window widgets
+/// Defines the methods implemented by all window widgets.
+/// More details can be found in the [wiki](https://github.com/fltk-rs/fltk-rs/wiki/windows).
+/// Windows (which can be found in the window module) implement GroupExt as well.
+/// # Safety
+/// fltk-rs traits depend on some FLTK internal code
+/// # Warning
+/// fltk-rs traits are non-exhaustive,
+/// to avoid future breakage if you try to implement them manually,
+/// use the Deref and DerefMut pattern or the `widget_extends!` macro
 pub unsafe trait WindowExt: GroupExt {
     /// Positions the window to the center of the screen
     fn center_screen(self) -> Self
@@ -574,6 +691,7 @@ pub unsafe trait WindowExt: GroupExt {
     fn free_position(&mut self);
     /// Get the raw system handle of the window
     fn raw_handle(&self) -> crate::window::RawHandle;
+    #[doc(hidden)]
     /// Set the window associated with a raw handle.
     /// `RawHandle` is a void pointer to: (Windows: `HWND`, X11: `Xid` (`u64`), macOS: `NSWindow`)
     /// # Safety
@@ -594,7 +712,8 @@ pub unsafe trait WindowExt: GroupExt {
     fn decorated_w(&self) -> i32;
     /// Returns the decorated height
     fn decorated_h(&self) -> i32;
-    /// Set the window's minimum width, minimum height, max width and max height
+    /// Set the window's minimum width, minimum height, max width and max height.
+    /// You can pass 0 as max_w and max_h to allow unlimited upward resize of the window.
     fn size_range(&mut self, min_w: i32, min_h: i32, max_w: i32, max_h: i32);
     /// Set the hotspot widget of the window
     fn hotspot<W: WidgetExt>(&mut self, w: &W)
@@ -622,9 +741,42 @@ pub unsafe trait WindowExt: GroupExt {
     fn screen_num(&self) -> i32;
     /// Set the screen number
     fn set_screen_num(&mut self, n: i32);
+    /// wait for the window to be displayed after calling `show()`.
+    /// More info [here](https://www.fltk.org/doc-1.4/classFl__Window.html#aafbec14ca8ff8abdaff77a35ebb23dd8)
+    fn wait_for_expose(&self);
+    /// Get the window's opacity
+    fn opacity(&self) -> f64;
+    /// Set the window's opacity,
+    /// Ranges from 0.0 to 1.0, where 1.0 is fully opaque and 0.0 is fully transparent.
+    /// This should be called on a shown window.
+    /// On X11, opacity support depends on the window manager and can be queried:
+    /// ```ignore
+    /// $ xprop -root _NET_SUPPORTED | grep -o _NET_WM_WINDOW_OPACITY
+    /// ```
+    fn set_opacity(&mut self, val: f64);
+    /// Get the window's XA_WM_CLASS property
+    fn xclass(&self) -> Option<String>;
+    /// Set the window's XA_WM_CLASS property.
+    /// This should be called before showing the window
+    fn set_xclass(&mut self, s: &str);
+    /// Clear the modal state of the window
+    fn clear_modal_states(&mut self);
+    /// removes the window border and sets the window on top, by settings the NOBORDER and OVERRIDE flags
+    fn set_override(&mut self);
+    /// Checks whether the OVERRIDE flag was set
+    fn is_override(&self) -> bool;
+    /// Forces the position of the window
+    fn force_position(&mut self, flag: bool);
 }
 
-/// Defines the methods implemented by all input and output widgets
+/// Defines the methods implemented by all input and output widgets.
+/// More details can be found in the [wiki](https://github.com/fltk-rs/fltk-rs/wiki/inout_widgets).
+/// # Safety
+/// fltk-rs traits depend on some FLTK internal code
+/// # Warning
+/// fltk-rs traits are non-exhaustive,
+/// to avoid future breakage if you try to implement them manually,
+/// use the Deref and DerefMut pattern or the `widget_extends!` macro
 pub unsafe trait InputExt: WidgetExt {
     /// Returns the value inside the input/output widget
     fn value(&self) -> String;
@@ -636,7 +788,7 @@ pub unsafe trait InputExt: WidgetExt {
     fn set_maximum_size(&mut self, val: i32);
     /// Returns the index position inside an input/output widget
     fn position(&self) -> i32;
-    /// Sets the index postion inside an input/output widget
+    /// Sets the index position inside an input/output widget
     /// # Errors
     /// Errors on failure to set the cursor position in the text
     fn set_position(&mut self, val: i32) -> Result<(), FltkError>;
@@ -693,6 +845,14 @@ pub unsafe trait InputExt: WidgetExt {
 }
 
 /// Defines the methods implemented by all menu widgets
+/// These are found in the menu module: MenuBar, SysMenuBar, Choice, MenuButton ...etc.
+/// Menus function in 2 main ways which are discussed in the [wiki](https://github.com/fltk-rs/fltk-rs/wiki/menus)
+/// # Safety
+/// fltk-rs traits depend on some FLTK internal code
+/// # Warning
+/// fltk-rs traits are non-exhaustive,
+/// to avoid future breakage if you try to implement them manually,
+/// use the Deref and DerefMut pattern or the `widget_extends!` macro
 pub unsafe trait MenuExt: WidgetExt {
     /// Get a menu item by name
     fn find_item(&self, name: &str) -> Option<crate::menu::MenuItem>;
@@ -780,17 +940,18 @@ pub unsafe trait MenuExt: WidgetExt {
     fn set_value(&mut self, v: i32) -> bool;
     /// Clears the items in a menu, effectively deleting them.
     fn clear(&mut self);
-    /// Clears a submenu by index, failure return `FltkErrorKind::FailedOperation`
+    /// Clears a submenu by index
     /// # Errors
-    /// Errors on failure to clear the submenu
+    /// Errors on failure to clear the submenu, failure returns an [`FltkErrorKind::FailedOperation`](`crate::prelude::FltkErrorKind::FailedOperation`)
     fn clear_submenu(&mut self, idx: i32) -> Result<(), FltkError>;
     /// Clears the items in a menu, effectively deleting them, and recursively force-cleans capturing callbacks
     /// # Safety
     /// Deletes `user_data` and any captured objects in the callback
     unsafe fn unsafe_clear(&mut self);
-    /// Clears a submenu by index, failure return `FltkErrorKind::FailedOperation`. Also recursively force-cleans capturing callbacks
+    #[doc(hidden)]
+    /// Clears a submenu by index. Also recursively force-cleans capturing callbacks
     /// # Safety
-    /// Deletes `user_data` and any captured objects in the callback
+    /// Deletes `user_data` and any captured objects in the callback, , failure returns an [`FltkErrorKind::FailedOperation`](`crate::prelude::FltkErrorKind::FailedOperation`)
     /// # Errors
     /// Errors on failure to clear the submenu
     unsafe fn unsafe_clear_submenu(&mut self, idx: i32) -> Result<(), FltkError>;
@@ -800,9 +961,9 @@ pub unsafe trait MenuExt: WidgetExt {
     fn text(&self, idx: i32) -> Option<String>;
     /// Get the menu item at an index
     fn at(&self, idx: i32) -> Option<crate::menu::MenuItem>;
-    /// Set the mode of a menu item by index and flag
+    /// Get the mode of a menu item by index and flag
     fn mode(&self, idx: i32) -> crate::menu::MenuFlag;
-    /// Get the mode of a menu item
+    /// Set the mode of a menu item
     fn set_mode(&mut self, idx: i32, flag: crate::menu::MenuFlag);
     /// End the menu
     fn end(&mut self);
@@ -821,6 +982,13 @@ pub unsafe trait MenuExt: WidgetExt {
 }
 
 /// Defines the methods implemented by all valuator widgets
+/// More details can be found in the [wiki](https://github.com/fltk-rs/fltk-rs/wiki/valuators).
+/// # Safety
+/// fltk-rs traits depend on some FLTK internal code
+/// # Warning
+/// fltk-rs traits are non-exhaustive,
+/// to avoid future breakage if you try to implement them manually,
+/// use the Deref and DerefMut pattern or the `widget_extends!` macro
 pub unsafe trait ValuatorExt: WidgetExt {
     /// Set bounds of a valuator
     fn set_bounds(&mut self, a: f64, b: f64);
@@ -858,6 +1026,13 @@ pub unsafe trait ValuatorExt: WidgetExt {
 }
 
 /// Defines the methods implemented by `TextDisplay` and `TextEditor`
+/// More details can be found in the [wiki](https://github.com/fltk-rs/fltk-rs/wiki/text).
+/// # Safety
+/// fltk-rs traits depend on some FLTK internal code
+/// # Warning
+/// fltk-rs traits are non-exhaustive,
+/// to avoid future breakage if you try to implement them manually,
+/// use the Deref and DerefMut pattern or the `widget_extends!` macro
 pub unsafe trait DisplayExt: WidgetExt {
     /// Get the associated `TextBuffer`
     fn buffer(&self) -> Option<crate::text::TextBuffer>;
@@ -913,6 +1088,12 @@ pub unsafe trait DisplayExt: WidgetExt {
         style_buffer: B,
         entries: Vec<crate::text::StyleTableEntry>,
     );
+    /// Sets the style of the text widget
+    fn set_highlight_data_ext<B: Into<Option<crate::text::TextBuffer>>>(
+        &mut self,
+        style_buffer: B,
+        entries: Vec<crate::text::StyleTableEntryExt>,
+    );
     /// Unset the style of the text widget
     fn unset_highlight_data(&mut self, style_buffer: crate::text::TextBuffer);
     /// Sets the cursor style
@@ -988,9 +1169,28 @@ pub unsafe trait DisplayExt: WidgetExt {
     fn wrapped_column(&self, row: i32, column: i32) -> i32;
     /// Correct a row number from an unconstrained position
     fn wrapped_row(&self, row: i32) -> i32;
+    /// Set the grammar underline color
+    fn set_grammar_underline_color(&mut self, color: Color);
+    /// Get the grammar underline color
+    fn grammar_underline_color(&self) -> Color;
+    /// Set the spelling underline color
+    fn set_spelling_underline_color(&mut self, color: Color);
+    /// Get the spelling underline color
+    fn spelling_underline_color(&self) -> Color;
+    /// Set the secondary selection color
+    fn set_secondary_selection_color(&mut self, color: Color);
+    /// Get the secondary selection color
+    fn secondary_selection_color(&self) -> Color;
 }
 
 /// Defines the methods implemented by all browser types
+/// More info can be found in the [wiki](https://github.com/fltk-rs/fltk-rs/wiki/browsers)
+/// # Safety
+/// fltk-rs traits depend on some FLTK internal code
+/// # Warning
+/// fltk-rs traits are non-exhaustive,
+/// to avoid future breakage if you try to implement them manually,
+/// use the Deref and DerefMut pattern or the `widget_extends!` macro
 pub unsafe trait BrowserExt: WidgetExt {
     /// Removes the specified line.
     /// Lines start at 1
@@ -1097,7 +1297,14 @@ pub unsafe trait BrowserExt: WidgetExt {
     fn value(&self) -> i32;
 }
 
-/// Defines the methods implemented by table types
+/// Defines the methods implemented by table types.
+/// More details can be found in the [wiki](https://github.com/fltk-rs/fltk-rs/wiki/tables).
+/// # Safety
+/// fltk-rs traits depend on some FLTK internal code
+/// # Warning
+/// fltk-rs traits are non-exhaustive,
+/// to avoid future breakage if you try to implement them manually,
+/// use the Deref and DerefMut pattern or the `widget_extends!` macro
 pub unsafe trait TableExt: GroupExt {
     /// Clears the table
     fn clear(&mut self);
@@ -1116,6 +1323,9 @@ pub unsafe trait TableExt: GroupExt {
     /// The range of row and column numbers for all visible and partially visible cells in the table.
     /// Returns (`row_top`, `col_left`, `row_bot`, `col_right`)
     fn visible_cells(&self) -> (i32, i32, i32, i32);
+    /// The range of row and column numbers for all visible and partially visible cells in the table.
+    /// Returns (`row_top`, `col_left`, `row_bot`, `col_right`)
+    fn try_visible_cells(&self) -> Option<(i32, i32, i32, i32)>;
     /// Returns whether the resize is interactive
     fn is_interactive_resize(&self) -> bool;
     /// Returns whether a row is resizable
@@ -1162,21 +1372,21 @@ pub unsafe trait TableExt: GroupExt {
     fn set_row_height(&mut self, row: i32, height: i32);
     /// Gets the row's height
     fn row_height(&self, row: i32) -> i32;
-    /// Sets the columns's width
+    /// Sets the column's width
     fn set_col_width(&mut self, col: i32, width: i32);
-    /// Gets the columns's width
+    /// Gets the column's width
     fn col_width(&self, col: i32) -> i32;
     /// Sets all rows height
     fn set_row_height_all(&mut self, height: i32);
-    /// Sets all columns's width
+    /// Sets all column's width
     fn set_col_width_all(&mut self, width: i32);
     /// Sets the row's position
     fn set_row_position(&mut self, row: i32);
-    /// Sets the columns's position
+    /// Sets the column's position
     fn set_col_position(&mut self, col: i32);
     /// Gets the row's position
     fn row_position(&self) -> i32;
-    /// Gets the columns's position
+    /// Gets the column's position
     fn col_position(&self) -> i32;
     /// Sets the top row
     fn set_top_row(&mut self, row: i32);
@@ -1185,8 +1395,12 @@ pub unsafe trait TableExt: GroupExt {
     /// Returns whether a cell is selected
     fn is_selected(&self, r: i32, c: i32) -> bool;
     /// Gets the selection.
-    /// Returns (`row_top`, `col_left`, `row_bot`, `col_right`)
+    /// Returns (`row_top`, `col_left`, `row_bot`, `col_right`).
+    /// Returns -1 if no selection.
     fn get_selection(&self) -> (i32, i32, i32, i32);
+    /// Tries to get the selection.
+    /// Returns an Option((`row_top`, `col_left`, `row_bot`, `col_right`))
+    fn try_get_selection(&self) -> Option<(i32, i32, i32, i32)>;
     /// Sets the selection
     fn set_selection(&mut self, row_top: i32, col_left: i32, row_bot: i32, col_right: i32);
     /// Unset selection
@@ -1221,6 +1435,7 @@ pub unsafe trait TableExt: GroupExt {
         &mut self,
         cb: F,
     );
+    #[doc(hidden)]
     /// INTERNAL: Retrieve the draw cell data
     /// # Safety
     /// Can return multiple mutable references to the `draw_cell_data`
@@ -1235,9 +1450,31 @@ pub unsafe trait TableExt: GroupExt {
     fn scrollbar(&self) -> crate::valuator::Scrollbar;
     /// Returns the table's horizontal scrollbar
     fn hscrollbar(&self) -> crate::valuator::Scrollbar;
+    /// Find a cell's coords and size by row and column
+    fn find_cell(
+        &self,
+        ctx: crate::table::TableContext,
+        row: i32,
+        col: i32,
+    ) -> Option<(i32, i32, i32, i32)>;
+    /// Get the cursor to row/col
+    fn cursor2rowcol(
+        &self,
+    ) -> Option<(
+        crate::table::TableContext,
+        i32,
+        i32,
+        crate::table::TableResizeFlag,
+    )>;
 }
 
 /// Defines the methods implemented by all image types
+/// # Safety
+/// fltk-rs traits depend on some FLTK internal code
+/// # Warning
+/// fltk-rs traits are non-exhaustive,
+/// to avoid future breakage if you try to implement them manually,
+/// use the Deref and DerefMut pattern
 pub unsafe trait ImageExt {
     /// Performs a deep copy of the image
     fn copy(&self) -> Self
@@ -1255,7 +1492,7 @@ pub unsafe trait ImageExt {
     fn w(&self) -> i32;
     /// Return the height of the image
     fn h(&self) -> i32;
-    /// Retunrs a pointer of the image
+    /// Returns a pointer of the image
     /// # Safety
     /// Can return multiple mutable pointers to the image
     unsafe fn as_image_ptr(&self) -> *mut fltk_sys::image::Fl_Image;
@@ -1273,6 +1510,10 @@ pub unsafe trait ImageExt {
     /// # Errors
     /// Errors on failure to transform to `RgbImage`
     fn to_rgb(&self) -> Result<crate::image::RgbImage, FltkError>;
+    /// Transforms the image into an `RgbImage`
+    /// # Errors
+    /// Errors on failure to transform to `RgbImage`
+    fn to_rgb_image(&self) -> Result<crate::image::RgbImage, FltkError>;
     /// Scales the image
     fn scale(&mut self, width: i32, height: i32, proportional: bool, can_expand: bool);
     /// Return the count of pointers in an image (Pixmaps have more than 1, bitmaps have 0, Rgb based images have 1)
@@ -1295,10 +1536,12 @@ pub unsafe trait ImageExt {
         Self: Sized;
     /// Checks if the image was deleted
     fn was_deleted(&self) -> bool;
+    #[doc(hidden)]
     /// INTERNAL: Manually increment the atomic refcount
     /// # Safety
     /// The underlying image pointer must be valid
     unsafe fn increment_arc(&mut self);
+    #[doc(hidden)]
     /// INTERNAL: Manually decrement the atomic refcount
     /// # Safety
     /// The underlying image pointer must be valid
@@ -1460,6 +1703,44 @@ macro_rules! widget_extends {
                 self
             }
 
+            /// Initialize center of another widget on the x axis
+            pub fn center_x<W: $crate::prelude::WidgetExt>(mut self, w: &W) -> Self {
+                assert!(!w.was_deleted());
+                assert!(!self.was_deleted());
+                debug_assert!(
+                    w.width() != 0 && w.height() != 0,
+                    "center_of requires the size of the widget to be known!"
+                );
+                let sw = self.width() as f64;
+                let sh = self.height() as f64;
+                let ww = w.width() as f64;
+                let sx = (ww - sw) / 2.0;
+                let sy = self.y();
+                let wx = if w.as_window().is_some() { 0 } else { w.x() };
+                self.resize(sx as i32 + wx, sy, sw as i32, sh as i32);
+                self.redraw();
+                self
+            }
+
+            /// Initialize center of another widget on the y axis
+            pub fn center_y<W: $crate::prelude::WidgetExt>(mut self, w: &W) -> Self {
+                assert!(!w.was_deleted());
+                assert!(!self.was_deleted());
+                debug_assert!(
+                    w.width() != 0 && w.height() != 0,
+                    "center_of requires the size of the widget to be known!"
+                );
+                let sw = self.width() as f64;
+                let sh = self.height() as f64;
+                let wh = w.height() as f64;
+                let sx = self.x();
+                let sy = (wh - sh) / 2.0;
+                let wy = if w.as_window().is_some() { 0 } else { w.y() };
+                self.resize(sx, sy as i32 + wy, sw as i32, sh as i32);
+                self.redraw();
+                self
+            }
+
             /// Initialize center of parent
             pub fn center_of_parent(mut self) -> Self {
                 assert!(!self.was_deleted());
