@@ -1,11 +1,9 @@
-use crate::enums::{Align, CallbackTrigger, Color, Damage, Event, Font, FrameType, Key, LabelType};
-use crate::image::Image;
+use crate::enums::{Color, Font, Key};
 use crate::prelude::*;
 use crate::utils::FlString;
 use fltk_sys::text::*;
 use std::{
     ffi::{CStr, CString},
-    mem,
     os::raw,
     sync::atomic::{AtomicUsize, Ordering},
 };
@@ -282,7 +280,7 @@ impl TextBuffer {
     }
 
     /// Returns the selection position
-    pub fn selection_position(&mut self) -> Option<(i32, i32)> {
+    pub fn selection_position(&self) -> Option<(i32, i32)> {
         assert!(!self.inner.is_null());
         unsafe {
             let mut start = 0;
@@ -298,7 +296,7 @@ impl TextBuffer {
     }
 
     /// Returns the selection text
-    pub fn selection_text(&mut self) -> String {
+    pub fn selection_text(&self) -> String {
         assert!(!self.inner.is_null());
         unsafe {
             let x = Fl_Text_Buffer_selection_text(self.inner);
@@ -322,6 +320,69 @@ impl TextBuffer {
         unsafe { Fl_Text_Buffer_replace_selection(self.inner, text.as_ptr()) }
     }
 
+    /// Secondary selects the text from start to end
+    pub fn secondary_select(&mut self, start: i32, end: i32) {
+        assert!(!self.inner.is_null());
+        unsafe { Fl_Text_Buffer_secondary_select(self.inner, start as i32, end as i32) }
+    }
+
+    /// Returns whether text is secondary selected
+    pub fn secondary_selected(&self) -> bool {
+        assert!(!self.inner.is_null());
+        unsafe { Fl_Text_Buffer_secondary_selected(self.inner) != 0 }
+    }
+
+    /// Unselects text (secondary selection)
+    pub fn secondary_unselect(&mut self) {
+        assert!(!self.inner.is_null());
+        unsafe { Fl_Text_Buffer_secondary_unselect(self.inner) }
+    }
+
+    /// Returns the secondary selection position
+    pub fn secondary_selection_position(&self) -> Option<(i32, i32)> {
+        assert!(!self.inner.is_null());
+        unsafe {
+            let mut start = 0;
+            let mut end = 0;
+            let ret = Fl_Text_Buffer_secondary_selection_position(
+                self.inner,
+                &mut start as _,
+                &mut end as _,
+            );
+            if ret == 0 {
+                None
+            } else {
+                let x = (start as i32, end as i32);
+                Some(x)
+            }
+        }
+    }
+
+    /// Returns the secondary selection text
+    pub fn secondary_selection_text(&self) -> String {
+        assert!(!self.inner.is_null());
+        unsafe {
+            let x = Fl_Text_Buffer_secondary_selection_text(self.inner);
+            assert!(!x.is_null());
+            CStr::from_ptr(x as *mut raw::c_char)
+                .to_string_lossy()
+                .to_string()
+        }
+    }
+
+    /// Removes the secondary selection
+    pub fn remove_secondary_selection(&mut self) {
+        assert!(!self.inner.is_null());
+        unsafe { Fl_Text_Buffer_remove_secondary_selection(self.inner) }
+    }
+
+    /// Replaces the secondary selection
+    pub fn replace_secondary_selection(&mut self, text: &str) {
+        assert!(!self.inner.is_null());
+        let text = CString::safe_new(text);
+        unsafe { Fl_Text_Buffer_replace_secondary_selection(self.inner, text.as_ptr()) }
+    }
+
     /// Highlights selection
     pub fn highlight(&mut self, start: i32, end: i32) {
         assert!(!self.inner.is_null());
@@ -329,7 +390,7 @@ impl TextBuffer {
     }
 
     /// Returns whether text is highlighted
-    pub fn is_highlighted(&mut self) -> bool {
+    pub fn is_highlighted(&self) -> bool {
         assert!(!self.inner.is_null());
         unsafe { Fl_Text_Buffer_is_highlighted(self.inner) != 0 }
     }
@@ -341,7 +402,7 @@ impl TextBuffer {
     }
 
     /// Returns the highlight position
-    pub fn highlight_position(&mut self) -> Option<(i32, i32)> {
+    pub fn highlight_position(&self) -> Option<(i32, i32)> {
         assert!(!self.inner.is_null());
         unsafe {
             let mut start = 0;
@@ -357,7 +418,7 @@ impl TextBuffer {
     }
 
     /// Returns the highlighted text
-    pub fn highlight_text(&mut self) -> String {
+    pub fn highlight_text(&self) -> String {
         assert!(!self.inner.is_null());
         unsafe {
             let x = Fl_Text_Buffer_highlight_text(self.inner);
@@ -644,21 +705,32 @@ pub enum DragType {
 }
 
 /// Creates a non-editable text display widget
-#[derive(WidgetBase, WidgetExt, DisplayExt, Debug)]
+#[derive(Debug)]
 pub struct TextDisplay {
     inner: *mut Fl_Text_Display,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(TextDisplay, Fl_Text_Display);
+crate::macros::widget::impl_widget_base!(TextDisplay, Fl_Text_Display);
+crate::macros::display::impl_display_ext!(TextDisplay, Fl_Text_Display);
+
 /// Creates an editable text display widget
-#[derive(WidgetBase, WidgetExt, DisplayExt, Debug)]
+#[derive(Debug)]
 pub struct TextEditor {
     inner: *mut Fl_Text_Editor,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(TextEditor, Fl_Text_Editor);
+crate::macros::widget::impl_widget_base!(TextEditor, Fl_Text_Editor);
+crate::macros::display::impl_display_ext!(TextEditor, Fl_Text_Editor);
+
+/// Alias Fl_Text_Editor for use in `add_key_binding`
+pub type TextEditorPtr = *mut Fl_Text_Editor;
+
 /// Creates an editable text display widget to handle terminal-like behavior, such as
 /// logging events or debug information.
 /// `SimpleTerminal` already has an internal buffer.
@@ -666,13 +738,38 @@ pub struct TextEditor {
 /// handle stdio redirection, pipes, pseudo ttys, termio character cooking,
 /// keyboard input processing, screen addressing, random cursor positioning,
 /// curses compatibility, or VT100/xterm emulation.
-#[derive(WidgetBase, WidgetExt, DisplayExt, Debug)]
+#[derive(Debug)]
 pub struct SimpleTerminal {
     inner: *mut Fl_Simple_Terminal,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(SimpleTerminal, Fl_Simple_Terminal);
+crate::macros::widget::impl_widget_base!(SimpleTerminal, Fl_Simple_Terminal);
+crate::macros::display::impl_display_ext!(SimpleTerminal, Fl_Simple_Terminal);
+
+/// The attribute of the style entry
+#[derive(Debug, Clone, Copy, PartialEq)]
+#[repr(u32)]
+#[non_exhaustive]
+pub enum TextAttr {
+    /// No attribute
+    None = 0x0000,
+    /// Use the background color in the `bgcolor` field
+    BgColor = 0x0001,
+    /// Use the background color in the `bgcolor` field to highlight the whole line
+    BgColorExt = 0x0003,
+    /// A single underline, underline types are mutually exclusive
+    Underline = 0x0004,
+    /// Grammar suggestion (blue dotted underline)
+    Grammar = 0x0008,
+    /// Spelling suggestion (red dotted underline)
+    Spelling = 0x000C,
+    /// Line through the middle of the text
+    StrikeThrough = 0x0010,
+}
+
 /// Defines the styles used in the `set_highlight_data`, which is used with style buffers
 #[derive(Debug, Clone, Copy)]
 pub struct StyleTableEntry {
@@ -684,7 +781,25 @@ pub struct StyleTableEntry {
     pub size: i32,
 }
 
+/// Defines the styles used in the `set_highlight_data`, which is used with style buffers
+#[derive(Debug, Clone, Copy)]
+pub struct StyleTableEntryExt {
+    /// Font color
+    pub color: Color,
+    /// Font type
+    pub font: Font,
+    /// Font size
+    pub size: i32,
+    /// attribute
+    pub attr: TextAttr,
+    /// background color
+    pub bgcolor: Color,
+}
+
 impl TextEditor {
+    /// Any state/shortcut
+    pub const AnyState: crate::enums::Shortcut = crate::enums::Shortcut::from_i32(-1);
+
     /// Set to insert mode
     pub fn set_insert_mode(&mut self, b: bool) {
         assert!(!self.was_deleted());
@@ -937,6 +1052,32 @@ impl TextEditor {
             Fl_Text_Editor_kf_select_all(self.inner);
         }
     }
+
+    /// Add a key binding
+    pub fn add_key_binding(
+        &mut self,
+        key: crate::enums::Key,
+        shortcut: crate::enums::Shortcut,
+        cb: fn(key: crate::enums::Key, editor: TextEditorPtr) -> i32,
+    ) {
+        assert!(!self.was_deleted());
+        unsafe {
+            Fl_Text_Editor_add_key_binding(
+                self.inner,
+                key.bits(),
+                shortcut.bits(),
+                std::mem::transmute(Some(cb)),
+            );
+        }
+    }
+
+    /// Remove a key binding
+    pub fn remove_key_binding(&mut self, key: crate::enums::Key, shortcut: crate::enums::Shortcut) {
+        assert!(!self.was_deleted());
+        unsafe {
+            Fl_Text_Editor_remove_key_binding(self.inner, key.bits(), shortcut.bits());
+        }
+    }
 }
 
 impl SimpleTerminal {
@@ -990,6 +1131,14 @@ impl SimpleTerminal {
         unsafe { Fl_Simple_Terminal_append(self.inner, s.into_raw()) }
     }
 
+    /// Appends data to the terminal buffer
+    pub fn append2(&mut self, s: &[u8]) {
+        assert!(!self.was_deleted());
+        assert!(self.buffer().is_some());
+        let s = CString::new(s).unwrap();
+        unsafe { Fl_Simple_Terminal_append2(self.inner, s.into_raw(), -1) }
+    }
+
     /// Sets the text of the terminal buffer
     pub fn set_text(&mut self, s: &str) {
         assert!(!self.was_deleted());
