@@ -1,4 +1,4 @@
-use crate::enums::{Align, CallbackTrigger, Color, Damage, Event, Font, FrameType, Key, LabelType};
+use crate::enums::{Color, Font, FrameType, Key};
 use crate::image::Image;
 use crate::prelude::*;
 use crate::utils::FlString;
@@ -103,13 +103,16 @@ pub enum TreeItemDrawMode {
 }
 
 /// Defines a tree widget
-#[derive(WidgetBase, WidgetExt, Debug)]
+#[derive(Debug)]
 pub struct Tree {
     inner: *mut Fl_Tree,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(Tree, Fl_Tree);
+crate::macros::widget::impl_widget_base!(Tree, Fl_Tree);
+
 /// Defines a tree item
 #[derive(Debug, Clone)]
 pub struct TreeItem {
@@ -117,6 +120,7 @@ pub struct TreeItem {
     parent: *const Fl_Tree_Item,
     tree: Tree,
     is_root: bool,
+    is_derived: bool,
 }
 
 /// Defines a tree item array
@@ -197,6 +201,16 @@ impl Tree {
         }
     }
 
+    /// Adds a `TreeItem`
+    pub fn add_item(&mut self, path: &str, item: &TreeItem) -> Option<TreeItem> {
+        assert!(!self.was_deleted());
+        let path = CString::safe_new(path);
+        unsafe {
+            let x = Fl_Tree_add_item(self.inner, path.as_ptr() as *mut raw::c_char, item.inner);
+            TreeItem::from_raw(x)
+        }
+    }
+
     /// Inserts a `TreeItem` above another tree item
     pub fn insert_above(&mut self, above: &TreeItem, name: &str) -> Option<TreeItem> {
         assert!(!self.was_deleted());
@@ -280,26 +294,27 @@ impl Tree {
         }
     }
 
+    #[deprecated(since = "1.2.21", note = "use callback_item() instead")]
     /// Set the item that was last clicked.
-    pub fn set_item_clicked(&mut self) -> Option<TreeItem> {
+    pub fn set_item_clicked(&self) -> Option<TreeItem> {
         assert!(!self.was_deleted());
         unsafe { TreeItem::from_raw(Fl_Tree_item_clicked(self.inner)) }
     }
 
     /// Gets the first tree item
-    pub fn first(&mut self) -> Option<TreeItem> {
+    pub fn first(&self) -> Option<TreeItem> {
         assert!(!self.was_deleted());
         unsafe { TreeItem::from_raw(Fl_Tree_first(self.inner)) }
     }
 
     /// Gets the first visible tree item
-    pub fn first_visible_item(&mut self) -> Option<TreeItem> {
+    pub fn first_visible_item(&self) -> Option<TreeItem> {
         assert!(!self.was_deleted());
         unsafe { TreeItem::from_raw(Fl_Tree_first_visible_item(self.inner)) }
     }
 
     /// Gets the next tree item
-    pub fn next(&mut self, item: &TreeItem) -> Option<TreeItem> {
+    pub fn next(&self, item: &TreeItem) -> Option<TreeItem> {
         assert!(!self.was_deleted());
         if item.inner.is_null() {
             return None;
@@ -308,7 +323,7 @@ impl Tree {
     }
 
     /// Gets the previous tree item
-    pub fn prev(&mut self, item: &TreeItem) -> Option<TreeItem> {
+    pub fn prev(&self, item: &TreeItem) -> Option<TreeItem> {
         assert!(!self.was_deleted());
         if item.inner.is_null() {
             return None;
@@ -317,19 +332,19 @@ impl Tree {
     }
 
     /// Gets the last tree item
-    pub fn last(&mut self) -> Option<TreeItem> {
+    pub fn last(&self) -> Option<TreeItem> {
         assert!(!self.was_deleted());
         unsafe { TreeItem::from_raw(Fl_Tree_last(self.inner)) }
     }
 
     /// Gets the last visible tree item
-    pub fn last_visible_item(&mut self) -> Option<TreeItem> {
+    pub fn last_visible_item(&self) -> Option<TreeItem> {
         assert!(!self.was_deleted());
         unsafe { TreeItem::from_raw(Fl_Tree_last_visible_item(self.inner)) }
     }
 
     /// Gets the next visible tree item
-    pub fn next_visible_item(&mut self, start: &TreeItem, direction_key: Key) -> Option<TreeItem> {
+    pub fn next_visible_item(&self, start: &TreeItem, direction_key: Key) -> Option<TreeItem> {
         assert!(!self.was_deleted());
         if start.inner.is_null() {
             return None;
@@ -344,20 +359,20 @@ impl Tree {
     }
 
     /// Gets the first selected tree item
-    pub fn first_selected_item(&mut self) -> Option<TreeItem> {
+    pub fn first_selected_item(&self) -> Option<TreeItem> {
         assert!(!self.was_deleted());
         unsafe { TreeItem::from_raw(Fl_Tree_first_selected_item(self.inner)) }
     }
 
     /// Gets the last selected tree item
-    pub fn last_selected_item(&mut self) -> Option<TreeItem> {
+    pub fn last_selected_item(&self) -> Option<TreeItem> {
         assert!(!self.was_deleted());
         unsafe { TreeItem::from_raw(Fl_Tree_last_selected_item(self.inner)) }
     }
 
-    /// Gets the next tree item, `direction_key` is by default `Key::Down`
+    /// Gets the next tree item, `direction_key` is by default [`Key::Down`](`crate::enums::Key::Down`)
     pub fn next_item(
-        &mut self,
+        &self,
         item: &TreeItem,
         direction_key: Key,
         visible: bool,
@@ -376,7 +391,7 @@ impl Tree {
         }
     }
 
-    /// Gets the next selected tree item, `direction_key` is by default `Key::Down`
+    /// Gets the next selected tree item, `direction_key` is by default [`Key::Down`](`crate::enums::Key::Down`)
     pub fn next_selected_item(&mut self, item: &TreeItem, direction_key: Key) -> Option<TreeItem> {
         assert!(!self.was_deleted());
         if item.inner.is_null() {
@@ -392,7 +407,7 @@ impl Tree {
     }
 
     /// Gets the selected tree items
-    pub fn get_selected_items(&mut self) -> Option<Vec<TreeItem>> {
+    pub fn get_selected_items(&self) -> Option<Vec<TreeItem>> {
         assert!(!self.was_deleted());
         unsafe {
             let mut items = TreeItemArray {
@@ -848,7 +863,7 @@ impl Tree {
         }
     }
 
-    /// Gets the opne icon
+    /// Gets the open icon
     pub fn open_icon(&self) -> Option<Box<dyn ImageExt>> {
         assert!(!self.was_deleted());
         unsafe {
@@ -863,7 +878,7 @@ impl Tree {
         }
     }
 
-    /// Sets the opne icon
+    /// Sets the open icon
     pub fn set_open_icon<Img: ImageExt>(&mut self, image: Option<Img>) {
         assert!(!self.was_deleted());
         if let Some(image) = image {
@@ -889,7 +904,7 @@ impl Tree {
         }
     }
 
-    /// Sets the opne icon
+    /// Sets the close icon
     pub fn set_close_icon<Img: ImageExt>(&mut self, image: Option<Img>) {
         assert!(!self.was_deleted());
         if let Some(image) = image {
@@ -912,7 +927,7 @@ impl Tree {
         unsafe { Fl_Tree_set_showcollapse(self.inner, flag as i32) }
     }
 
-    /// Returs whether the root is shown
+    /// Returns whether the root is shown
     pub fn show_root(&self) -> bool {
         assert!(!self.was_deleted());
         unsafe { Fl_Tree_showroot(self.inner) != 0 }
@@ -1119,7 +1134,7 @@ impl Tree {
     }
 
     /// Get the callback item
-    pub fn callback_item(&mut self) -> Option<TreeItem> {
+    pub fn callback_item(&self) -> Option<TreeItem> {
         assert!(!self.was_deleted());
         unsafe { TreeItem::from_raw(Fl_Tree_callback_item(self.inner)) }
     }
@@ -1135,6 +1150,23 @@ impl Tree {
         assert!(!self.was_deleted());
         unsafe { mem::transmute(Fl_Tree_callback_reason(self.inner)) }
     }
+
+    /// Get an item's pathname
+    pub fn item_pathname(&self, item: &TreeItem) -> Result<String, FltkError> {
+        assert!(!self.was_deleted());
+        let mut temp = vec![0u8; 256];
+        unsafe {
+            let ret = Fl_Tree_item_pathname(self.inner, temp.as_mut_ptr() as _, 256, item.inner);
+            if ret == 0 {
+                if let Some(pos) = temp.iter().position(|x| *x == 0) {
+                    temp = temp.split_at(pos).0.to_vec();
+                }
+                Ok(String::from_utf8_lossy(&temp).to_string())
+            } else {
+                Err(FltkError::Internal(FltkErrorKind::FailedOperation))
+            }
+        }
+    }
 }
 
 impl IntoIterator for Tree {
@@ -1168,13 +1200,121 @@ impl TreeItem {
             };
             let parent = Fl_Tree_Item_parent(ptr);
             let is_root = Fl_Tree_Item_is_root(ptr) != 0;
-            let x = TreeItem {
+            Some(TreeItem {
                 inner: ptr,
                 parent,
                 tree,
                 is_root,
-            };
-            Some(x)
+                is_derived: false,
+            })
+        }
+    }
+
+    /// Creates a new TreeItem
+    pub fn new(tree: &Tree, label: &str) -> Self {
+        let label = CString::safe_new(label);
+        unsafe {
+            let ptr = Fl_Tree_Item_new(tree.inner, label.as_ptr());
+            assert!(!ptr.is_null());
+            Self {
+                inner: ptr,
+                parent: ptr,
+                tree: tree.clone(),
+                is_root: true,
+                is_derived: true,
+            }
+        }
+    }
+
+    /**
+    Overrides the draw_item_content method
+    Example usage:
+       ```rust,no_run
+       use fltk::{draw, enums::*, tree};
+       let mut tree = tree::Tree::default();
+       let mut item = tree::TreeItem::new(&tree, "Hello");
+       item.draw_item_content(|item, render| {
+           // Our item's dimensions + text content
+           let x = item.label_x();
+           let y = item.label_y();
+           let w = item.label_w();
+           let h = item.label_h();
+           let txt = if let Some(txt) = item.label() {
+               txt
+           } else {
+               String::new()
+           };
+           if render {
+               // Draw bg -- a filled rectangle
+               draw::draw_rect_fill(x, y, w, h, item.label_bgcolor());
+               // Draw label
+               draw::set_font(Font::Helvetica, 14);
+               draw::set_draw_color(Color::ForeGround); // use recommended fg color
+               draw::draw_text2(&txt, x, y, w, h, Align::Left); // draw the item's label
+           }
+           // Rendered or not, we must calculate content's max X position
+           let (lw, _) = draw::measure(&txt, true); // get width of label text
+           return x + lw; // return X + label width
+       });
+       // Add our custom item to a path
+       let _third = tree.add_item("first/second/thrid", &item).unwrap();
+       ```
+    */
+    pub fn draw_item_content<F: FnMut(&mut Self, bool) -> i32>(&mut self, cb: F) {
+        assert!(!self.was_deleted());
+        assert!(self.is_derived);
+        unsafe {
+            unsafe extern "C" fn shim(
+                item: *mut Fl_Tree_Item,
+                render: i32,
+                data: *mut raw::c_void,
+            ) -> i32 {
+                let mut item = TreeItem::from_raw(item).unwrap();
+                let a = data as *mut Box<dyn FnMut(&mut TreeItem, bool) -> i32>;
+                let f: &mut (dyn FnMut(&mut TreeItem, bool) -> i32) = &mut **a;
+                if let Ok(ret) = std::panic::catch_unwind(std::panic::AssertUnwindSafe(|| {
+                    f(&mut item, render != 0)
+                })) {
+                    ret
+                } else {
+                    0
+                }
+            }
+            let a: *mut Box<dyn FnMut(&mut Self, bool) -> i32> =
+                Box::into_raw(Box::new(Box::new(cb)));
+            let data: *mut raw::c_void = a as *mut raw::c_void;
+            let callback: Option<
+                unsafe extern "C" fn(
+                    self_: *mut Fl_Tree_Item,
+                    arg1: i32,
+                    arg2: *mut raw::c_void,
+                ) -> i32,
+            > = Some(shim);
+            Fl_Tree_Item_draw_item_content(self.inner, callback, data);
+        }
+    }
+
+    /// Set the internal data of the tree item
+    /// # Warning
+    /// This method doesn't store the type information of the passed data
+    pub fn set_user_data<T: Clone + 'static>(&mut self, data: T) {
+        assert!(!self.was_deleted());
+        unsafe {
+            Fl_Tree_Item_set_user_data(self.inner, Box::into_raw(Box::from(data)) as _);
+        }
+    }
+
+    /// Get the stored data in the tree item
+    /// # Safety
+    /// Setting the user data doesn't store type information, as such it's on the developer to maintain the correct type
+    pub unsafe fn user_data<T: Clone + 'static>(&self) -> Option<T> {
+        assert!(!self.was_deleted());
+        let ptr = Fl_Tree_Item_user_data(self.inner);
+        if ptr.is_null() {
+            None
+        } else {
+            let data = ptr as *const _ as *mut T;
+            Some((*data).clone())
         }
     }
 
@@ -1220,7 +1360,7 @@ impl TreeItem {
         unsafe { Fl_Tree_Item_label_w(self.inner) }
     }
 
-    /// Sets the label's width
+    /// Gets the label's height
     pub fn label_h(&self) -> i32 {
         assert!(!self.was_deleted());
         unsafe { Fl_Tree_Item_label_h(self.inner) }
@@ -1246,13 +1386,14 @@ impl TreeItem {
         unsafe {
             let x = Fl_Tree_Item_label(self.inner);
             if x.is_null() {
-                return None;
+                None
+            } else {
+                Some(
+                    CStr::from_ptr(x as *mut raw::c_char)
+                        .to_string_lossy()
+                        .to_string(),
+                )
             }
-            Some(
-                CStr::from_ptr(x as *mut raw::c_char)
-                    .to_string_lossy()
-                    .to_string(),
-            )
         }
     }
 
@@ -1281,12 +1422,26 @@ impl TreeItem {
         unsafe { Fl_Tree_Item_labelsize(self.inner) }
     }
 
+    /// Sets the label's foreground color
+    pub fn set_label_fgcolor(&mut self, val: Color) {
+        assert!(!self.was_deleted());
+        unsafe { Fl_Tree_Item_set_labelfgcolor(self.inner, val.bits() as u32) }
+    }
+
+    /// Gets the label's foreground color
+    pub fn label_fgcolor(&self) -> Color {
+        assert!(!self.was_deleted());
+        unsafe { mem::transmute(Fl_Tree_Item_labelfgcolor(self.inner)) }
+    }
+
+    #[deprecated(since = "1.2.19", note = "please use `set_label_fgcolor` instead")]
     /// Sets the label's foreground color
     pub fn set_label_fg_color(&mut self, val: Color) {
         assert!(!self.was_deleted());
         unsafe { Fl_Tree_Item_set_labelfgcolor(self.inner, val.bits() as u32) }
     }
 
+    #[deprecated(since = "1.2.19", note = "please use `label_fgcolor` instead")]
     /// Gets the label's foreground color
     pub fn label_fg_color(&self) -> Color {
         assert!(!self.was_deleted());
@@ -1324,13 +1479,27 @@ impl TreeItem {
         unsafe { Fl_Tree_Item_set_widget(self.inner, val.as_widget_ptr() as *mut Fl_Widget) }
     }
 
+    #[deprecated(since = "1.2.18", note = "please use `try_widget` instead")]
     /// Gets the item's associated widget
     pub fn widget(&self) -> Widget {
         assert!(!self.was_deleted());
         unsafe {
-            Widget::from_widget_ptr(
-                Fl_Tree_Item_widget(self.inner) as *mut fltk_sys::widget::Fl_Widget
-            )
+            let ptr = Fl_Tree_Item_widget(self.inner) as *mut fltk_sys::widget::Fl_Widget;
+            assert!(!ptr.is_null());
+            Widget::from_widget_ptr(ptr)
+        }
+    }
+
+    /// Gets the item's associated widget
+    pub fn try_widget(&self) -> Option<impl WidgetExt> {
+        assert!(!self.was_deleted());
+        unsafe {
+            let ptr = Fl_Tree_Item_widget(self.inner) as *mut fltk_sys::widget::Fl_Widget;
+            if ptr.is_null() {
+                None
+            } else {
+                Some(Widget::from_widget_ptr(ptr))
+            }
         }
     }
 
@@ -1683,6 +1852,37 @@ impl TreeItem {
             }
         }
     }
+
+    /// Gets the user icon
+    pub fn user_icon(&self) -> Option<Box<dyn ImageExt>> {
+        assert!(!self.was_deleted());
+        unsafe {
+            let image_ptr = Fl_Tree_Item_usericon(self.inner);
+            if image_ptr.is_null() {
+                None
+            } else {
+                Some(Box::new(Image::from_image_ptr(
+                    image_ptr as *mut fltk_sys::image::Fl_Image,
+                )))
+            }
+        }
+    }
+
+    /// Sets the user icon
+    pub fn set_user_icon<Img: ImageExt>(&mut self, image: Option<Img>) {
+        assert!(!self.was_deleted());
+        if let Some(image) = image {
+            assert!(!image.was_deleted());
+            unsafe { Fl_Tree_Item_set_usericon(self.inner, image.as_image_ptr() as *mut _) }
+        } else {
+            unsafe { Fl_Tree_Item_set_usericon(self.inner, std::ptr::null_mut::<raw::c_void>()) }
+        }
+    }
+
+    /// Return the internal pointer of the tree item
+    pub fn as_ptr(&self) -> *mut Fl_Tree_Item {
+        self.inner
+    }
 }
 
 impl Iterator for TreeItem {
