@@ -4,9 +4,11 @@ use std::os::raw;
 
 use crate::prelude::FltkError;
 use crate::prelude::FltkErrorKind;
-// use crate::prelude::{FltkError, FltkErrorKind};
 
-pub(crate) trait FlString {
+#[doc(hidden)]
+/// A helper trait to get CStrings from Strings without panicking
+pub trait FlString {
+    /// Get CStrings from Strings without panicking
     fn safe_new(s: &str) -> CString;
 }
 
@@ -30,9 +32,11 @@ impl FlString for CString {
     let ret = rgb2hex(0, 255, 0); println!("0x{:06x}", ret);
     ```
 */
-pub fn rgb2hex(r: u8, g: u8, b: u8) -> u32 {
-    // Shouldn't fail
-    u32::from_str_radix(&format!("{:02x}{:02x}{:02x}", r, g, b), 16).unwrap()
+pub const fn rgb2hex(r: u8, g: u8, b: u8) -> u32 {
+    let r = r as u32;
+    let g = g as u32;
+    let b = b as u32;
+    ((r & 0xff) << 16) + ((g & 0xff) << 8) + (b & 0xff)
 }
 
 /**
@@ -43,9 +47,12 @@ pub fn rgb2hex(r: u8, g: u8, b: u8) -> u32 {
     let ret = rgba2hex(0, 255, 0, 255); println!("0x{:08x}", ret);
     ```
 */
-pub fn rgba2hex(r: u8, g: u8, b: u8, a: u8) -> u32 {
-    // Shouldn't fail
-    u32::from_str_radix(&format!("{:02x}{:02x}{:02x}{:02x}", r, g, b, a), 16).unwrap()
+pub const fn rgba2hex(r: u8, g: u8, b: u8, a: u8) -> u32 {
+    let r = r as u32;
+    let g = g as u32;
+    let b = b as u32;
+    let a = a as u32;
+    ((r & 0xff) << 24) + ((g & 0xff) << 16) + ((b & 0xff) << 8) + (a & 0xff)
 }
 
 /**
@@ -56,11 +63,10 @@ pub fn rgba2hex(r: u8, g: u8, b: u8, a: u8) -> u32 {
     let (r, g, b) = hex2rgb(0x000000);
     ```
 */
-pub fn hex2rgb(val: u32) -> (u8, u8, u8) {
-    let hex = format!("{:06x}", val);
-    let r = u8::from_str_radix(&hex[0..2], 16).unwrap();
-    let g = u8::from_str_radix(&hex[2..4], 16).unwrap();
-    let b = u8::from_str_radix(&hex[4..6], 16).unwrap();
+pub const fn hex2rgb(val: u32) -> (u8, u8, u8) {
+    let r = ((val >> 16) & 0xff) as u8;
+    let g = ((val >> 8) & 0xff) as u8;
+    let b = (val & 0xff) as u8;
     (r, g, b)
 }
 
@@ -72,7 +78,7 @@ pub fn hex2rgb(val: u32) -> (u8, u8, u8) {
     let (r, g, b, a) = hex2rgba(0xff0000ff);
     ```
 */
-pub fn hex2rgba(val: u32) -> (u8, u8, u8, u8) {
+pub const fn hex2rgba(val: u32) -> (u8, u8, u8, u8) {
     let r = ((val >> 24) & 0xff) as u8;
     let g = ((val >> 16) & 0xff) as u8;
     let b = ((val >> 8) & 0xff) as u8;
@@ -103,3 +109,21 @@ pub fn filename_expand(path: &str) -> Result<String, FltkError> {
         }
     }
 }
+
+/// Get the length of a char in terms of C strings
+pub fn char_len(c: char) -> usize {
+    extern "C" {
+        pub fn strlen(s: *const std::os::raw::c_char) -> usize;
+    }
+    let s = std::ffi::CString::new(c.to_string()).unwrap();
+    unsafe { strlen(s.as_ptr() as _) }
+}
+
+#[cfg(target_os = "macos")]
+/// Get a window's content view
+pub fn content_view<W: crate::prelude::WindowExt>(w: &W) -> *const raw::c_void {
+    extern "C" {
+        pub fn cfltk_getContentView(xid: *mut raw::c_void) -> *mut raw::c_void;
+    }
+    unsafe { cfltk_getContentView(w.raw_handle() as _) as _ }
+}
