@@ -1,5 +1,4 @@
-use crate::enums::{Align, CallbackTrigger, Color, Damage, Event, Font, FrameType, LabelType};
-use crate::image::Image;
+use crate::enums::{Color, Font, FrameType};
 use crate::prelude::*;
 use crate::utils::FlString;
 use fltk_sys::valuator::*;
@@ -10,13 +9,17 @@ use std::{
 };
 
 /// Creates a slider widget
-#[derive(WidgetBase, WidgetExt, ValuatorExt, Debug)]
+#[derive(Debug)]
 pub struct Slider {
     inner: *mut Fl_Slider,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(Slider, Fl_Slider);
+crate::macros::widget::impl_widget_base!(Slider, Fl_Slider);
+crate::macros::valuator::impl_valuator_ext!(Slider, Fl_Slider);
+
 /// Slider implementation
 impl Slider {
     /// Get the slider size as a fraction of the long axis
@@ -41,13 +44,17 @@ impl Slider {
 }
 
 /// Creates a nice slider widget
-#[derive(WidgetBase, WidgetExt, ValuatorExt, Debug)]
+#[derive(Debug)]
 pub struct NiceSlider {
     inner: *mut Fl_Nice_Slider,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(NiceSlider, Fl_Nice_Slider);
+crate::macros::widget::impl_widget_base!(NiceSlider, Fl_Nice_Slider);
+crate::macros::valuator::impl_valuator_ext!(NiceSlider, Fl_Nice_Slider);
+
 /// Slider implementation
 impl NiceSlider {
     /// Get the slider size as a fraction of the long axis
@@ -73,7 +80,7 @@ impl NiceSlider {
 
 /// Defines slider types
 #[repr(i32)]
-#[derive(WidgetType, Debug, Copy, Clone, PartialEq)]
+#[derive(Debug, Copy, Clone, PartialEq)]
 pub enum SliderType {
     /// Vertical slider
     Vertical = 0,
@@ -89,14 +96,20 @@ pub enum SliderType {
     HorizontalNice = 5,
 }
 
+crate::macros::widget::impl_widget_type!(SliderType);
+
 /// Creates a dial widget
-#[derive(WidgetBase, WidgetExt, ValuatorExt, Debug)]
+#[derive(Debug)]
 pub struct Dial {
     inner: *mut Fl_Dial,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(Dial, Fl_Dial);
+crate::macros::widget::impl_widget_base!(Dial, Fl_Dial);
+crate::macros::valuator::impl_valuator_ext!(Dial, Fl_Dial);
+
 impl Dial {
     /// Get the angles of the dial
     pub fn angles(&self) -> (i32, i32) {
@@ -121,13 +134,17 @@ impl Dial {
 }
 
 /// Creates a line dial widget
-#[derive(WidgetBase, WidgetExt, ValuatorExt, Debug)]
+#[derive(Debug)]
 pub struct LineDial {
     inner: *mut Fl_Line_Dial,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(LineDial, Fl_Line_Dial);
+crate::macros::widget::impl_widget_base!(LineDial, Fl_Line_Dial);
+crate::macros::valuator::impl_valuator_ext!(LineDial, Fl_Line_Dial);
+
 impl LineDial {
     /// Get the angles of the dial
     pub fn angles(&self) -> (i32, i32) {
@@ -153,7 +170,7 @@ impl LineDial {
 
 /// Defines dial types
 #[repr(i32)]
-#[derive(WidgetType, Debug, Copy, Clone, PartialEq)]
+#[derive(Debug, Copy, Clone, PartialEq)]
 pub enum DialType {
     /// Normal dial
     Normal = 0,
@@ -163,17 +180,23 @@ pub enum DialType {
     Fill = 2,
 }
 
+crate::macros::widget::impl_widget_type!(DialType);
+
 /// Creates a counter widget
-#[derive(WidgetBase, WidgetExt, ValuatorExt, Debug)]
+#[derive(Debug)]
 pub struct Counter {
     inner: *mut Fl_Counter,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(Counter, Fl_Counter);
+crate::macros::widget::impl_widget_base!(Counter, Fl_Counter);
+crate::macros::valuator::impl_valuator_ext!(Counter, Fl_Counter);
+
 /// Defines counter types
 #[repr(i32)]
-#[derive(WidgetType, Debug, Copy, Clone, PartialEq)]
+#[derive(Debug, Copy, Clone, PartialEq)]
 pub enum CounterType {
     /// Normal counter
     Normal = 0,
@@ -181,14 +204,20 @@ pub enum CounterType {
     Simple = 1,
 }
 
+crate::macros::widget::impl_widget_type!(CounterType);
+
 /// Creates a scrollbar widget
-#[derive(WidgetBase, WidgetExt, ValuatorExt, Debug)]
+#[derive(Debug)]
 pub struct Scrollbar {
     inner: *mut Fl_Scrollbar,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(Scrollbar, Fl_Scrollbar);
+crate::macros::widget::impl_widget_base!(Scrollbar, Fl_Scrollbar);
+crate::macros::valuator::impl_valuator_ext!(Scrollbar, Fl_Scrollbar);
+
 impl Scrollbar {
     /// Get the slider size as a fraction of the long axis
     pub fn slider_size(&self) -> f32 {
@@ -213,7 +242,7 @@ impl Scrollbar {
 
 /// Defines scrollbar types
 #[repr(i32)]
-#[derive(WidgetType, Debug, Copy, Clone, PartialEq)]
+#[derive(Debug, Copy, Clone, PartialEq)]
 pub enum ScrollbarType {
     /// Vertical scrollbar
     Vertical = 0,
@@ -229,22 +258,32 @@ pub enum ScrollbarType {
     HorizontalNice = 5,
 }
 
+crate::macros::widget::impl_widget_type!(ScrollbarType);
+
 /// Creates a roller widget
-#[derive(WidgetBase, WidgetExt, ValuatorExt, Debug)]
+#[derive(Debug)]
 pub struct Roller {
     inner: *mut Fl_Roller,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(Roller, Fl_Roller);
+crate::macros::widget::impl_widget_base!(Roller, Fl_Roller);
+crate::macros::valuator::impl_valuator_ext!(Roller, Fl_Roller);
+
 /// Creates a value slider widget
-#[derive(WidgetBase, WidgetExt, ValuatorExt, Debug)]
+#[derive(Debug)]
 pub struct ValueSlider {
     inner: *mut Fl_Value_Slider,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(ValueSlider, Fl_Value_Slider);
+crate::macros::widget::impl_widget_base!(ValueSlider, Fl_Value_Slider);
+crate::macros::valuator::impl_valuator_ext!(ValueSlider, Fl_Value_Slider);
+
 impl ValueSlider {
     /// Gets the text font
     pub fn text_font(&self) -> Font {
@@ -304,22 +343,30 @@ impl ValueSlider {
 }
 
 /// Creates an adjuster widget
-#[derive(WidgetBase, WidgetExt, ValuatorExt, Debug)]
+#[derive(Debug)]
 pub struct Adjuster {
     inner: *mut Fl_Adjuster,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(Adjuster, Fl_Adjuster);
+crate::macros::widget::impl_widget_base!(Adjuster, Fl_Adjuster);
+crate::macros::valuator::impl_valuator_ext!(Adjuster, Fl_Adjuster);
+
 /// Creates an value input widget, which takes a numeric value.
 /// If a step is set, the value can be also dragged
-#[derive(WidgetBase, WidgetExt, ValuatorExt, Debug)]
+#[derive(Debug)]
 pub struct ValueInput {
     inner: *mut Fl_Value_Input,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(ValueInput, Fl_Value_Input);
+crate::macros::widget::impl_widget_base!(ValueInput, Fl_Value_Input);
+crate::macros::valuator::impl_valuator_ext!(ValueInput, Fl_Value_Input);
+
 impl ValueInput {
     /// Gets the text font
     pub fn text_font(&self) -> Font {
@@ -372,13 +419,17 @@ impl ValueInput {
 }
 
 /// Creates an value output widget
-#[derive(WidgetBase, WidgetExt, ValuatorExt, Debug)]
+#[derive(Debug)]
 pub struct ValueOutput {
     inner: *mut Fl_Value_Output,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(ValueOutput, Fl_Value_Output);
+crate::macros::widget::impl_widget_base!(ValueOutput, Fl_Value_Output);
+crate::macros::valuator::impl_valuator_ext!(ValueOutput, Fl_Value_Output);
+
 impl ValueOutput {
     /// Gets the text font
     pub fn text_font(&self) -> Font {
@@ -418,21 +469,29 @@ impl ValueOutput {
 }
 
 /// Creates a fill slider
-#[derive(WidgetBase, WidgetExt, ValuatorExt, Debug)]
+#[derive(Debug)]
 pub struct FillSlider {
     inner: *mut Fl_Fill_Slider,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(FillSlider, Fl_Fill_Slider);
+crate::macros::widget::impl_widget_base!(FillSlider, Fl_Fill_Slider);
+crate::macros::valuator::impl_valuator_ext!(FillSlider, Fl_Fill_Slider);
+
 /// Creates a fill dial
-#[derive(WidgetBase, WidgetExt, ValuatorExt, Debug)]
+#[derive(Debug)]
 pub struct FillDial {
     inner: *mut Fl_Fill_Dial,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(FillDial, Fl_Fill_Dial);
+crate::macros::widget::impl_widget_base!(FillDial, Fl_Fill_Dial);
+crate::macros::valuator::impl_valuator_ext!(FillDial, Fl_Fill_Dial);
+
 impl FillDial {
     /// Get the angles of the dial
     pub fn angles(&self) -> (i32, i32) {
@@ -457,13 +516,17 @@ impl FillDial {
 }
 
 /// Creates a horizontal slider
-#[derive(WidgetBase, WidgetExt, ValuatorExt, Debug)]
+#[derive(Debug)]
 pub struct HorSlider {
     inner: *mut Fl_Hor_Slider,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(HorSlider, Fl_Hor_Slider);
+crate::macros::widget::impl_widget_base!(HorSlider, Fl_Hor_Slider);
+crate::macros::valuator::impl_valuator_ext!(HorSlider, Fl_Hor_Slider);
+
 /// Slider implementation
 impl HorSlider {
     /// Get the slider size as a fraction of the long axis
@@ -488,13 +551,17 @@ impl HorSlider {
 }
 
 /// Creates a horizontal fill slider
-#[derive(WidgetBase, WidgetExt, ValuatorExt, Debug)]
+#[derive(Debug)]
 pub struct HorFillSlider {
     inner: *mut Fl_Hor_Fill_Slider,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(HorFillSlider, Fl_Hor_Fill_Slider);
+crate::macros::widget::impl_widget_base!(HorFillSlider, Fl_Hor_Fill_Slider);
+crate::macros::valuator::impl_valuator_ext!(HorFillSlider, Fl_Hor_Fill_Slider);
+
 /// Slider implementation
 impl HorFillSlider {
     /// Get the slider size as a fraction of the long axis
@@ -519,13 +586,17 @@ impl HorFillSlider {
 }
 
 /// Creates a horizontal nice slider
-#[derive(WidgetBase, WidgetExt, ValuatorExt, Debug)]
+#[derive(Debug)]
 pub struct HorNiceSlider {
     inner: *mut Fl_Hor_Nice_Slider,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(HorNiceSlider, Fl_Hor_Nice_Slider);
+crate::macros::widget::impl_widget_base!(HorNiceSlider, Fl_Hor_Nice_Slider);
+crate::macros::valuator::impl_valuator_ext!(HorNiceSlider, Fl_Hor_Nice_Slider);
+
 /// Slider implementation
 impl HorNiceSlider {
     /// Get the slider size as a fraction of the long axis
@@ -550,13 +621,17 @@ impl HorNiceSlider {
 }
 
 /// Creates a horizontal value slider
-#[derive(WidgetBase, WidgetExt, ValuatorExt, Debug)]
+#[derive(Debug)]
 pub struct HorValueSlider {
     inner: *mut Fl_Hor_Value_Slider,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(HorValueSlider, Fl_Hor_Value_Slider);
+crate::macros::widget::impl_widget_base!(HorValueSlider, Fl_Hor_Value_Slider);
+crate::macros::valuator::impl_valuator_ext!(HorValueSlider, Fl_Hor_Value_Slider);
+
 impl HorValueSlider {
     /// Gets the text font
     pub fn text_font(&self) -> Font {
