@@ -1,5 +1,16 @@
 #![allow(unused_imports)]
 
+//! Window widgets
+//!
+//! **Multithreaded** applications should always create/show/open/close windows
+//! from the main thread (This might or might not work on your specific target,
+//! due to fltk calling the underlying platform's window code. If you want
+//! portability, avoid it.) If you need to trigger showing a windows from
+//! another thread, use [`messages`](crate::app::channel) to notify the main
+//! thread that the window needs showing. An example of this can be found in the
+//! [`threads_windows`](crate::examples::threads_windows) example. An alternative to that is
+//! [`awake_callback`](crate::app::awake_callback)
+
 use crate::app::screen_size;
 use crate::enums::{
     Align, CallbackTrigger, Color, Cursor, Damage, Event, Font, FrameType, LabelType, Mode,
@@ -24,7 +35,8 @@ use std::{
     target_os = "windows",
     target_os = "macos",
     target_os = "ios",
-    target_os = "android"
+    target_os = "android",
+    feature = "use-wayland"
 ))]
 pub type RawHandle = *mut raw::c_void;
 
@@ -34,7 +46,8 @@ pub type RawHandle = *mut raw::c_void;
         target_os = "windows",
         target_os = "macos",
         target_os = "ios",
-        target_os = "android"
+        target_os = "android",
+        feature = "use-wayland"
     )),
     any(
         target_arch = "arm",
@@ -54,7 +67,8 @@ pub type RawHandle = u32;
         target_os = "windows",
         target_os = "macos",
         target_os = "ios",
-        target_os = "android"
+        target_os = "android",
+        feature = "use-wayland"
     )),
     any(
         target_arch = "aarch64",
@@ -72,7 +86,7 @@ pub type Window = DoubleWindow;
 
 /// Defines the window type
 #[repr(i32)]
-#[derive(WidgetType, Debug, Copy, Clone, PartialEq)]
+#[derive(Debug, Copy, Clone, PartialEq)]
 pub enum WindowType {
     /// Single window
     Normal = 240,
@@ -80,23 +94,33 @@ pub enum WindowType {
     Double = 241,
 }
 
+crate::macros::widget::impl_widget_type!(WindowType);
+
 /// Creates a single (buffered) window widget
-#[derive(WidgetBase, WidgetExt, GroupExt, WindowExt, Debug)]
+#[derive(Debug)]
 pub struct SingleWindow {
     inner: *mut Fl_Single_Window,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(SingleWindow, Fl_Single_Window);
+crate::macros::widget::impl_widget_base!(SingleWindow, Fl_Single_Window);
+crate::macros::group::impl_group_ext!(SingleWindow, Fl_Single_Window);
+crate::macros::window::impl_window_ext!(SingleWindow, Fl_Single_Window);
+
 impl SingleWindow {
     /// Creates a default initialized single window
+    ///
+    /// Note: Only call this from the main thread.
     pub fn default() -> SingleWindow {
+        assert!(crate::app::is_ui_thread());
         let mut win = <SingleWindow as Default>::default();
         win.free_position();
         win
     }
 
-    /// Find an `Fl_Window` through a raw handle. The window must have been instatiated by the app.
+    /// Find an `Fl_Window` through a raw handle. The window must have been instantiated by the app.
     /// `void *` to: (Windows: `HWND`, X11: `Xid` (`u64`), macOS: `NSWindow`)
     /// # Safety
     /// The data must be valid and is OS-dependent.
@@ -178,10 +202,7 @@ impl SingleWindow {
         }
     }
 
-    /// Returns the pixels per unit
-    /// Assumes a factor of 2.0 (Retina) on MacOS since `[NSWindow backingScaleFactor]`
-    /// requires an info.plist with `NSHighResolutionCapable` set to true.
-    /// If this is not desired, you need to bundle your app and set to false.
+    /// Returns the pixels per unit/point
     pub fn pixels_per_unit(&self) -> f32 {
         assert!(!self.was_deleted());
         #[allow(unused_mut)]
@@ -189,12 +210,11 @@ impl SingleWindow {
         #[cfg(target_os = "macos")]
         {
             extern "C" {
-                pub fn my_getScalingFactor(handle: *mut raw::c_void) -> f64;
+                pub fn cfltk_getScalingFactor(handle: *mut raw::c_void) -> f64;
             }
             let mac_version = unsafe { fltk_sys::fl::Fl_mac_os_version() };
             if mac_version >= 100700 {
-                factor = unsafe { my_getScalingFactor(self.raw_handle()) };
-                factor = if factor as i32 == 0 { 2.0 } else { 1.0 };
+                factor = unsafe { cfltk_getScalingFactor(self.raw_handle()) };
             }
         }
         let s = crate::app::screen_scale(self.screen_num());
@@ -210,25 +230,51 @@ impl SingleWindow {
     pub fn pixel_h(&self) -> i32 {
         (self.pixels_per_unit() * self.h() as f32) as i32
     }
+
+    /// Get the default XA_WM_CLASS property for all windows of your application
+    pub fn default_xclass() -> Option<String> {
+        unsafe {
+            let ptr = Fl_Single_Window_default_xclass();
+            if ptr.is_null() {
+                None
+            } else {
+                Some(CStr::from_ptr(ptr).to_string_lossy().to_string())
+            }
+        }
+    }
+    /// Set the default XA_WM_CLASS property for all windows of your application.
+    /// This should be called before showing with window
+    pub fn set_default_xclass(s: &str) {
+        let s = CString::safe_new(s);
+        unsafe { Fl_Single_Window_set_default_xclass(s.as_ptr()) }
+    }
 }
 
 /// Creates a double (buffered) window widget
-#[derive(WidgetBase, WidgetExt, GroupExt, WindowExt, Debug)]
+#[derive(Debug)]
 pub struct DoubleWindow {
     inner: *mut Fl_Double_Window,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(DoubleWindow, Fl_Double_Window);
+crate::macros::widget::impl_widget_base!(DoubleWindow, Fl_Double_Window);
+crate::macros::group::impl_group_ext!(DoubleWindow, Fl_Double_Window);
+crate::macros::window::impl_window_ext!(DoubleWindow, Fl_Double_Window);
+
 impl DoubleWindow {
     /// Creates a default initialized double window
+    ///
+    /// Note: Only call this from the main thread.
     pub fn default() -> DoubleWindow {
+        assert!(crate::app::is_ui_thread());
         let mut win = <DoubleWindow as Default>::default();
         win.free_position();
         win
     }
 
-    /// Find an `Fl_Window` through a raw handle. The window must have been instatiated by the app.
+    /// Find an `Fl_Window` through a raw handle. The window must have been instantiated by the app.
     /// `void *` to: (Windows: `HWND`, X11: `Xid` (`u64`), macOS: `NSWindow`)
     /// # Safety
     /// The data must be valid and is OS-dependent.
@@ -316,36 +362,7 @@ impl DoubleWindow {
         unsafe { Fl_Double_Window_flush(self.inner) }
     }
 
-    /// Get the window's opacity
-    pub fn opacity(&self) -> f64 {
-        assert!(!self.was_deleted());
-        unsafe { Fl_Double_Window_alpha(self.inner) as f64 / 255.0 }
-    }
-
-    /// Set the window's opacity,
-    /// Ranges from 0.0 to 1.0, where 1.0 is fully opaque and 0.0 is fully transparent.
-    /// This should be called on a shown window.
-    /// On X11, opacity support depends on the window manager and can be queried:
-    /// ```ignore
-    /// $ xprop -root _NET_SUPPORTED | grep -o _NET_WM_WINDOW_OPACITY
-    /// ```
-    pub fn set_opacity(&mut self, val: f64) {
-        assert!(!self.was_deleted());
-        assert!(self.shown());
-        let val: u8 = if val > 1.0 {
-            255
-        } else if val < 0.0 {
-            0
-        } else {
-            (val * 255.0).round() as u8
-        };
-        unsafe { Fl_Double_Window_set_alpha(self.inner, val) }
-    }
-
-    /// Returns the pixels per unit.
-    /// Assumes a factor of 2.0 (Retina) (Retina) on MacOS since `[NSWindow backingScaleFactor]`
-    /// requires an info.plist with `NSHighResolutionCapable` set to true.
-    /// If this is not desired, you need to bundle your app and set to false.
+    /// Returns the pixels per unit./point
     pub fn pixels_per_unit(&self) -> f32 {
         assert!(!self.was_deleted());
         #[allow(unused_mut)]
@@ -353,13 +370,11 @@ impl DoubleWindow {
         #[cfg(target_os = "macos")]
         {
             extern "C" {
-                pub fn my_getScalingFactor(handle: *mut raw::c_void) -> f64;
+                pub fn cfltk_getScalingFactor(handle: *mut raw::c_void) -> f64;
             }
             let mac_version = unsafe { fltk_sys::fl::Fl_mac_os_version() };
             if mac_version >= 100700 {
-                factor = unsafe { my_getScalingFactor(self.raw_handle()) };
-                // A factor of 0 means no info.plist was found.
-                factor = if factor as i32 == 0 { 2.0 } else { 1.0 };
+                factor = unsafe { cfltk_getScalingFactor(self.raw_handle()) };
             }
         }
         let s = crate::app::screen_scale(self.screen_num());
@@ -375,19 +390,103 @@ impl DoubleWindow {
     pub fn pixel_h(&self) -> i32 {
         (self.pixels_per_unit() * self.h() as f32) as i32
     }
+
+    /// Show a window after it had been hidden. Works on Windows and X11 systems
+    pub fn platform_show(&self) {
+        #[allow(unused_unsafe)]
+        unsafe {
+            #[cfg(target_os = "windows")]
+            {
+                extern "system" {
+                    fn ShowWindow(hwnd: *mut raw::c_void, nCmdShow: raw::c_int) -> raw::c_int;
+                }
+                ShowWindow(self.raw_handle(), 9);
+            }
+            #[cfg(target_os = "macos")]
+            {
+                extern "C" {
+                    fn cfltk_winShow(xid: *mut raw::c_void);
+                }
+                cfltk_winShow(self.raw_handle());
+            }
+            #[cfg(not(any(target_os = "macos", target_os = "android", target_os = "windows", feature = "use-wayland")))]
+            {
+                enum Display {}
+                extern "C" {
+                    fn XMapWindow(display: *mut Display, win: u64);
+                }
+                XMapWindow(crate::app::display() as _, self.raw_handle() as _);
+                crate::app::flush();
+            }
+        }
+    }
+
+    /// Hide a window using the platforms hide call. Works on Windows and X11 systems
+    pub fn platform_hide(&self) {
+        #[allow(unused_unsafe)]
+        unsafe {
+            #[cfg(target_os = "windows")]
+            {
+                extern "system" {
+                    fn ShowWindow(hwnd: *mut raw::c_void, nCmdShow: raw::c_int) -> raw::c_int;
+                }
+                ShowWindow(self.raw_handle(), 0);
+            }
+            #[cfg(target_os = "macos")]
+            {
+                extern "C" {
+                    fn cfltk_winHide(xid: *mut raw::c_void);
+                }
+                cfltk_winHide(self.raw_handle());
+            }
+            #[cfg(not(any(target_os = "macos", target_os = "android", target_os = "windows", feature = "use-wayland")))]
+            {
+                enum Display {}
+                extern "C" {
+                    fn XUnmapWindow(display: *mut Display, win: u64);
+                }
+                XUnmapWindow(crate::app::display() as _, self.raw_handle() as _);
+                crate::app::flush();
+            }
+        }
+    }
+
+    /// Get the default XA_WM_CLASS property for all windows of your application
+    pub fn default_xclass() -> Option<String> {
+        unsafe {
+            let ptr = Fl_Double_Window_default_xclass();
+            if ptr.is_null() {
+                None
+            } else {
+                Some(CStr::from_ptr(ptr).to_string_lossy().to_string())
+            }
+        }
+    }
+    /// Set the default XA_WM_CLASS property for all windows of your application.
+    /// This should be called before showing with window
+    pub fn set_default_xclass(s: &str) {
+        let s = CString::safe_new(s);
+        unsafe { Fl_Double_Window_set_default_xclass(s.as_ptr()) }
+    }
 }
 
 /// Creates a Menu window widget
-#[derive(WidgetBase, WidgetExt, GroupExt, WindowExt, Debug)]
+#[derive(Debug)]
 pub struct MenuWindow {
     inner: *mut Fl_Menu_Window,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(MenuWindow, Fl_Menu_Window);
+crate::macros::widget::impl_widget_base!(MenuWindow, Fl_Menu_Window);
+crate::macros::group::impl_group_ext!(MenuWindow, Fl_Menu_Window);
+crate::macros::window::impl_window_ext!(MenuWindow, Fl_Menu_Window);
+
 impl MenuWindow {
     /// Creates a default initialized menu window
     pub fn default() -> MenuWindow {
+        assert!(crate::app::is_ui_thread());
         let mut win = <MenuWindow as Default>::default();
         win.free_position();
         win
@@ -395,22 +494,28 @@ impl MenuWindow {
 }
 
 /// Creates an overlay (buffered) window widget
-#[derive(WidgetBase, WidgetExt, GroupExt, WindowExt, Debug)]
+#[derive(Debug)]
 pub struct OverlayWindow {
     inner: *mut Fl_Overlay_Window,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+crate::macros::widget::impl_widget_ext!(OverlayWindow, Fl_Overlay_Window);
+crate::macros::widget::impl_widget_base!(OverlayWindow, Fl_Overlay_Window);
+crate::macros::group::impl_group_ext!(OverlayWindow, Fl_Overlay_Window);
+crate::macros::window::impl_window_ext!(OverlayWindow, Fl_Overlay_Window);
+
 impl OverlayWindow {
     /// Creates a default initialized overlay window
     pub fn default() -> OverlayWindow {
+        assert!(crate::app::is_ui_thread());
         let mut win = <OverlayWindow as Default>::default();
         win.free_position();
         win
     }
 
-    /// Find an `Fl_Window` through a raw handle. The window must have been instatiated by the app.
+    /// Find an `Fl_Window` through a raw handle. The window must have been instantiated by the app.
     /// `void *` to: (Windows: `HWND`, X11: `Xid` (`u64`), macOS: `NSWindow`)
     /// # Safety
     /// The data must be valid and is OS-dependent.
@@ -501,6 +606,7 @@ impl OverlayWindow {
     /// Draw overlay
     pub fn draw_overlay<F: FnMut(&mut Self) + 'static>(&mut self, cb: F) {
         assert!(!self.was_deleted());
+        assert!(self.is_derived);
         unsafe {
             unsafe extern "C" fn shim(wid: *mut Fl_Widget, data: *mut raw::c_void) {
                 let mut wid = OverlayWindow::from_widget_ptr(wid as *mut _);
@@ -509,7 +615,10 @@ impl OverlayWindow {
                 let f: &mut (dyn FnMut(&mut OverlayWindow)) = &mut **a;
                 let _ = std::panic::catch_unwind(std::panic::AssertUnwindSafe(|| f(&mut wid)));
             }
-            let _old_data = self.draw_data();
+            let mut _old_data = None;
+            if self.is_derived {
+                _old_data = self.draw_data();
+            }
             let a: *mut Box<dyn FnMut(&mut Self)> = Box::into_raw(Box::new(Box::new(cb)));
             let data: *mut raw::c_void = a as *mut raw::c_void;
             let callback: custom_draw_callback = Some(shim);
@@ -533,171 +642,29 @@ impl OverlayWindow {
 /// A wrapper around a raw OpenGL context
 pub type GlContext = *mut raw::c_void;
 
-/// Creates a OpenGL window widget
-#[cfg(feature = "enable-glwindow")]
-#[derive(WidgetBase, WidgetExt, GroupExt, WindowExt, Debug)]
-pub struct GlWindow {
-    inner: *mut Fl_Gl_Window,
-    tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
-    is_derived: bool,
-}
-
-#[cfg(feature = "enable-glwindow")]
-impl GlWindow {
-    /// Creates a default initialized gl window
-    pub fn default() -> GlWindow {
-        let mut win = <GlWindow as Default>::default();
-        win.free_position();
-        win
-    }
-
-    /// Gets an opengl function address
-    pub fn get_proc_address(&self, s: &str) -> *const raw::c_void {
-        extern "C" {
-            pub fn get_proc(name: *const raw::c_char) -> *mut raw::c_void;
-        }
-        let s = CString::safe_new(s);
-        let ret = unsafe { get_proc(s.as_ptr() as _) };
-        if !ret.is_null() {
-            ret as *const _
-        } else {
-            unsafe { Fl_Gl_Window_get_proc_address(self.inner, s.as_ptr()) as *const _ }
-        }
-    }
-
-    /// Forces the window to be drawn, this window is also made current and calls draw()
-    pub fn flush(&mut self) {
-        assert!(!self.was_deleted());
-        unsafe { Fl_Gl_Window_flush(self.inner) }
-    }
-
-    /// Returns whether the OpeGL context is still valid
-    pub fn valid(&self) -> bool {
-        assert!(!self.was_deleted());
-        unsafe { Fl_Gl_Window_valid(self.inner) != 0 }
-    }
-
-    /// Mark the OpeGL context as still valid
-    pub fn set_valid(&mut self, v: bool) {
-        assert!(!self.was_deleted());
-        unsafe { Fl_Gl_Window_set_valid(self.inner, v as raw::c_char) }
-    }
-
-    /// Returns whether the context is valid upon creation
-    pub fn context_valid(&self) -> bool {
-        assert!(!self.was_deleted());
-        unsafe { Fl_Gl_Window_context_valid(self.inner) != 0 }
-    }
-
-    /// Mark the context as valid upon creation
-    pub fn set_context_valid(&mut self, v: bool) {
-        assert!(!self.was_deleted());
-        unsafe { Fl_Gl_Window_set_context_valid(self.inner, v as raw::c_char) }
-    }
-
-    /// Returns the GlContext
-    pub fn context(&self) -> Option<GlContext> {
-        assert!(!self.was_deleted());
-        unsafe {
-            let ctx = Fl_Gl_Window_context(self.inner);
-            if ctx.is_null() {
-                None
-            } else {
-                Some(ctx)
-            }
-        }
-    }
-
-    /// Sets the GlContext
-    pub fn set_context(&mut self, ctx: GlContext, destroy_flag: bool) {
-        assert!(!self.was_deleted());
-        assert!(!ctx.is_null());
-        unsafe { Fl_Gl_Window_set_context(self.inner, ctx, destroy_flag as i32) }
-    }
-
-    /// Swaps the back and front buffers
-    pub fn swap_buffers(&mut self) {
-        assert!(!self.was_deleted());
-        unsafe { Fl_Gl_Window_swap_buffers(self.inner) }
-    }
-
-    /// Sets the projection so 0,0 is in the lower left of the window
-    /// and each pixel is 1 unit wide/tall.
-    pub fn ortho(&mut self) {
-        assert!(!self.was_deleted());
-        unsafe { Fl_Gl_Window_ortho(self.inner) }
-    }
-
-    /// Returns whether the GlWindow can do overlay
-    pub fn can_do_overlay(&self) -> bool {
-        assert!(!self.was_deleted());
-        unsafe { Fl_Gl_Window_can_do_overlay(self.inner) != 0 }
-    }
-
-    /// Redraws the overlay
-    pub fn redraw_overlay(&mut self) {
-        assert!(!self.was_deleted());
-        unsafe { Fl_Gl_Window_redraw_overlay(self.inner) }
-    }
-
-    /// Hides the overlay
-    pub fn hide_overlay(&mut self) {
-        assert!(!self.was_deleted());
-        unsafe { Fl_Gl_Window_hide_overlay(self.inner) }
-    }
-
-    /// Makes the overlay current
-    pub fn make_overlay_current(&mut self) {
-        assert!(!self.was_deleted());
-        unsafe { Fl_Gl_Window_make_overlay_current(self.inner) }
-    }
-
-    /// Returns the pixels per unit
-    pub fn pixels_per_unit(&mut self) -> f32 {
-        assert!(!self.was_deleted());
-        unsafe { Fl_Gl_Window_pixels_per_unit(self.inner) }
-    }
-
-    /// Gets the window's width in pixels
-    pub fn pixel_w(&mut self) -> i32 {
-        assert!(!self.was_deleted());
-        unsafe { Fl_Gl_Window_pixel_w(self.inner) }
-    }
-
-    /// Gets the window's height in pixels
-    pub fn pixel_h(&mut self) -> i32 {
-        assert!(!self.was_deleted());
-        unsafe { Fl_Gl_Window_pixel_h(self.inner) }
-    }
-
-    /// Get the Mode of the GlWindow
-    pub fn mode(&self) -> Mode {
-        assert!(!self.was_deleted());
-        unsafe { mem::transmute(Fl_Gl_Window_mode(self.inner)) }
-    }
-
-    /// Set the Mode of the GlWindow
-    pub fn set_mode(&mut self, mode: Mode) {
-        assert!(!self.was_deleted());
-        unsafe {
-            Fl_Gl_Window_set_mode(self.inner, mode.bits());
-        }
-    }
-}
-
 /// Creates a OpenGL Glut window widget
 #[cfg(feature = "enable-glwindow")]
-#[derive(WidgetBase, WidgetExt, GroupExt, WindowExt, Debug)]
+#[derive(Debug)]
 pub struct GlutWindow {
     inner: *mut Fl_Glut_Window,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
     is_derived: bool,
 }
 
+#[cfg(feature = "enable-glwindow")]
+crate::macros::widget::impl_widget_ext!(GlutWindow, Fl_Glut_Window);
+#[cfg(feature = "enable-glwindow")]
+crate::macros::widget::impl_widget_base!(GlutWindow, Fl_Glut_Window);
+#[cfg(feature = "enable-glwindow")]
+crate::macros::group::impl_group_ext!(GlutWindow, Fl_Glut_Window);
+#[cfg(feature = "enable-glwindow")]
+crate::macros::window::impl_window_ext!(GlutWindow, Fl_Glut_Window);
+
 #[cfg(feature = "enable-glwindow")]
 impl GlutWindow {
     /// Creates a default initialized glut window
     pub fn default() -> GlutWindow {
+        assert!(crate::app::is_ui_thread());
         let mut win = <GlutWindow as Default>::default();
         win.free_position();
         win
@@ -804,20 +771,20 @@ impl GlutWindow {
         unsafe { Fl_Glut_Window_make_overlay_current(self.inner) }
     }
 
-    /// Returns the pixels per unit
-    pub fn pixels_per_unit(&mut self) -> f32 {
+    /// Returns the pixels per unit/point
+    pub fn pixels_per_unit(&self) -> f32 {
         assert!(!self.was_deleted());
         unsafe { Fl_Glut_Window_pixels_per_unit(self.inner) }
     }
 
     /// Gets the window's width in pixels
-    pub fn pixel_w(&mut self) -> i32 {
+    pub fn pixel_w(&self) -> i32 {
         assert!(!self.was_deleted());
         unsafe { Fl_Glut_Window_pixel_w(self.inner) }
     }
 
     /// Gets the window's height in pixels
-    pub fn pixel_h(&mut self) -> i32 {
+    pub fn pixel_h(&self) -> i32 {
         assert!(!self.was_deleted());
         unsafe { Fl_Glut_Window_pixel_h(self.inner) }
     }
@@ -837,6 +804,194 @@ impl GlutWindow {
     }
 }
 
+#[cfg(feature = "enable-glwindow")]
+/// Alias GlutWindow as GlWindow
+pub type GlWindow = GlutWindow;
+
+#[doc(hidden)]
+/// An experimental space for unstable Windowing widgets, currently the GlWidgetWindow
+pub mod experimental {
+    use super::*;
+
+    /// Creates a OpenGL Glut window widget
+    #[cfg(feature = "enable-glwindow")]
+    #[derive(Debug)]
+    pub struct GlWidgetWindow {
+        inner: *mut Fl_Gl_Window,
+        tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
+        is_derived: bool,
+    }
+
+    #[cfg(feature = "enable-glwindow")]
+    crate::macros::widget::impl_widget_ext!(GlWidgetWindow, Fl_Gl_Window);
+    #[cfg(feature = "enable-glwindow")]
+    crate::macros::widget::impl_widget_base!(GlWidgetWindow, Fl_Gl_Window);
+    #[cfg(feature = "enable-glwindow")]
+    crate::macros::group::impl_group_ext!(GlWidgetWindow, Fl_Gl_Window);
+    #[cfg(feature = "enable-glwindow")]
+    crate::macros::window::impl_window_ext!(GlWidgetWindow, Fl_Gl_Window);
+
+    #[cfg(feature = "enable-glwindow")]
+    impl GlWidgetWindow {
+        /// Creates a default initialized glut window
+        pub fn default() -> GlWidgetWindow {
+            assert!(crate::app::is_ui_thread());
+            let mut win = <GlWidgetWindow as Default>::default();
+            win.free_position();
+            win.set_frame(FrameType::FlatBox);
+            win.begin();
+            win
+        }
+
+        /// Creates a new GlWidgetWindow
+        pub fn new<T: Into<Option<&'static str>>>(
+            x: i32,
+            y: i32,
+            w: i32,
+            h: i32,
+            label: T,
+        ) -> GlWidgetWindow {
+            let mut win = <GlWidgetWindow as WidgetBase>::new(x, y, w, h, label);
+            win.set_frame(FrameType::FlatBox);
+            win.begin();
+            win
+        }
+
+        /// Gets an opengl function address
+        pub fn get_proc_address(&self, s: &str) -> *const raw::c_void {
+            extern "C" {
+                pub fn get_proc(name: *const raw::c_char) -> *mut raw::c_void;
+            }
+            let s = CString::safe_new(s);
+            let ret = unsafe { get_proc(s.as_ptr() as _) };
+            if !ret.is_null() {
+                ret as *const _
+            } else {
+                unsafe { Fl_Gl_Window_get_proc_address(self.inner, s.as_ptr()) as *const _ }
+            }
+        }
+
+        /// Forces the window to be drawn, this window is also made current and calls draw()
+        pub fn flush(&mut self) {
+            assert!(!self.was_deleted());
+            unsafe { Fl_Gl_Window_flush(self.inner) }
+        }
+
+        /// Returns whether the OpeGL context is still valid
+        pub fn valid(&self) -> bool {
+            assert!(!self.was_deleted());
+            unsafe { Fl_Gl_Window_valid(self.inner) != 0 }
+        }
+
+        /// Mark the OpeGL context as still valid
+        pub fn set_valid(&mut self, v: bool) {
+            assert!(!self.was_deleted());
+            unsafe { Fl_Gl_Window_set_valid(self.inner, v as raw::c_char) }
+        }
+
+        /// Returns whether the context is valid upon creation
+        pub fn context_valid(&self) -> bool {
+            assert!(!self.was_deleted());
+            unsafe { Fl_Gl_Window_context_valid(self.inner) != 0 }
+        }
+
+        /// Mark the context as valid upon creation
+        pub fn set_context_valid(&mut self, v: bool) {
+            assert!(!self.was_deleted());
+            unsafe { Fl_Gl_Window_set_context_valid(self.inner, v as raw::c_char) }
+        }
+
+        /// Returns the GlContext
+        pub fn context(&self) -> Option<GlContext> {
+            assert!(!self.was_deleted());
+            unsafe {
+                let ctx = Fl_Gl_Window_context(self.inner);
+                if ctx.is_null() {
+                    None
+                } else {
+                    Some(ctx)
+                }
+            }
+        }
+
+        /// Sets the GlContext
+        pub fn set_context(&mut self, ctx: GlContext, destroy_flag: bool) {
+            assert!(!self.was_deleted());
+            assert!(!ctx.is_null());
+            unsafe { Fl_Gl_Window_set_context(self.inner, ctx, destroy_flag as i32) }
+        }
+
+        /// Swaps the back and front buffers
+        pub fn swap_buffers(&mut self) {
+            assert!(!self.was_deleted());
+            unsafe { Fl_Gl_Window_swap_buffers(self.inner) }
+        }
+
+        /// Sets the projection so 0,0 is in the lower left of the window
+        /// and each pixel is 1 unit wide/tall.
+        pub fn ortho(&mut self) {
+            assert!(!self.was_deleted());
+            unsafe { Fl_Gl_Window_ortho(self.inner) }
+        }
+
+        /// Returns whether the GlutWindow can do overlay
+        pub fn can_do_overlay(&self) -> bool {
+            assert!(!self.was_deleted());
+            unsafe { Fl_Gl_Window_can_do_overlay(self.inner) != 0 }
+        }
+
+        /// Redraws the overlay
+        pub fn redraw_overlay(&mut self) {
+            assert!(!self.was_deleted());
+            unsafe { Fl_Gl_Window_redraw_overlay(self.inner) }
+        }
+
+        /// Hides the overlay
+        pub fn hide_overlay(&mut self) {
+            assert!(!self.was_deleted());
+            unsafe { Fl_Gl_Window_hide_overlay(self.inner) }
+        }
+
+        /// Makes the overlay current
+        pub fn make_overlay_current(&mut self) {
+            assert!(!self.was_deleted());
+            unsafe { Fl_Gl_Window_make_overlay_current(self.inner) }
+        }
+
+        /// Returns the pixels per unit/point
+        pub fn pixels_per_unit(&self) -> f32 {
+            assert!(!self.was_deleted());
+            unsafe { Fl_Gl_Window_pixels_per_unit(self.inner) }
+        }
+
+        /// Gets the window's width in pixels
+        pub fn pixel_w(&self) -> i32 {
+            assert!(!self.was_deleted());
+            unsafe { Fl_Gl_Window_pixel_w(self.inner) }
+        }
+
+        /// Gets the window's height in pixels
+        pub fn pixel_h(&self) -> i32 {
+            assert!(!self.was_deleted());
+            unsafe { Fl_Gl_Window_pixel_h(self.inner) }
+        }
+
+        /// Get the Mode of the GlutWindow
+        pub fn mode(&self) -> Mode {
+            assert!(!self.was_deleted());
+            unsafe { mem::transmute(Fl_Gl_Window_mode(self.inner)) }
+        }
+
+        /// Set the Mode of the GlutWindow
+        pub fn set_mode(&mut self, mode: Mode) {
+            assert!(!self.was_deleted());
+            unsafe {
+                Fl_Gl_Window_set_mode(self.inner, mode.bits());
+            }
+        }
+    }
+}
+
 /// An Android window
 pub struct AndroidWindow {
     win: Window,
@@ -854,16 +1009,4 @@ impl AndroidWindow {
     }
 }
 
-impl Deref for AndroidWindow {
-    type Target = Window;
-
-    fn deref(&self) -> &Self::Target {
-        &self.win
-    }
-}
-
-impl DerefMut for AndroidWindow {
-    fn deref_mut(&mut self) -> &mut Self::Target {
-        &mut self.win
-    }
-}
+crate::widget_extends!(AndroidWindow, Window, win);
